@@ -12,13 +12,13 @@
 
 #include <cassert>
 #include <limits>
+#include <sstream>
 
 namespace libboardgame_gtp {
 
 //-----------------------------------------------------------------------------
 
-CmdLine::~CmdLine()
-{ }
+CmdLine::~CmdLine() = default;
 
 void CmdLine::add_elem(string::const_iterator begin,
                        string::const_iterator end)
