@@ -10,7 +10,6 @@
 
 #include "Engine.h"
 
-#include <cassert>
 #include <cctype>
 #include <iostream>
 #include "CmdLine.h"
@@ -64,8 +63,7 @@ Engine::Engine()
     add("version", &Engine::cmd_version);
 }
 
-Engine::~Engine()
-{ }
+Engine::~Engine() = default;
 
 void Engine::add(const string& name, Handler f)
 {
@@ -90,26 +88,6 @@ void Engine::add(const string& name, HandlerNoArgsNoResponse f)
                            placeholders::_1, placeholders::_2)));
 }
 
-void Engine::add(const string& name, void (*f)(const Arguments&, Response&))
-{
-    add(name, Handler(f));
-}
-
-void Engine::add(const string& name, void (*f)(Response&))
-{
-    add(name, HandlerNoArgs(f));
-}
-
-void Engine::add(const string& name, void (*f)(const Arguments&))
-{
-    add(name, HandlerNoResponse(f));
-}
-
-void Engine::add(const string& name, void (*f)())
-{
-    add(name, HandlerNoArgsNoResponse(f));
-}
-
 /** Return @c true if command is known, @c false otherwise. */
 void Engine::cmd_known_command(const Arguments& args, Response& response)
 {
@@ -119,8 +97,8 @@ void Engine::cmd_known_command(const Arguments& args, Response& response)
 /** List all known commands. */
 void Engine::cmd_list_commands(Response& response)
 {
-    for (HandlerIterator i = m_handlers.begin(); i != m_handlers.end(); ++i)
-        response << i->first << '\n';
+    for (auto& i : m_handlers)
+        response << i.first << '\n';
 }
 
 /** Return name. */
@@ -153,26 +131,6 @@ bool Engine::contains(const string& name) const
     return m_handlers.count(name) > 0;
 }
 
-void Engine::remove(const string& name)
-{
-    auto i = m_handlers.find(name);
-    if (i != m_handlers.end())
-        m_handlers.erase(i);
-}
-
-string Engine::exec(const string& line)
-{
-    assert(is_cmd_line(line));
-    Response response;
-    string buffer;
-    CmdLine cmd;
-    cmd.init(line);
-    bool status = handle_cmd(cmd, nullptr, response, buffer);
-    if (! status)
-        throw Failure(response.to_string());
-    return response.to_string();
-}
-
 bool Engine::exec(istream& in, bool throw_on_fail, ostream* log)
 {
     string line;
@@ -227,7 +185,7 @@ bool Engine::handle_cmd(CmdLine& line, ostream* out, Response& response,
     try
     {
         response.clear();
-        HandlerIterator pos = m_handlers.find(line.get_name());
+        auto pos = m_handlers.find(line.get_name());
         if (pos != m_handlers.end())
         {
             Arguments args(line);
