@@ -34,12 +34,12 @@ namespace libboardgame_mcts {
 using namespace std;
 using libboardgame_mcts::tree_util::find_node;
 using libboardgame_util::get_abort;
-using libboardgame_util::rounded_log_2;
 using libboardgame_util::time_to_string;
 using libboardgame_util::to_string;
 using libboardgame_util::ArrayList;
 using libboardgame_util::Barrier;
 using libboardgame_util::IntervalChecker;
+using libboardgame_util::RandomGenerator;
 using libboardgame_util::StatisticsBase;
 using libboardgame_util::StatisticsDirtyLockFree;
 using libboardgame_util::StatisticsExt;
@@ -232,23 +232,6 @@ public:
     /** @name Parameters */
     /** @{ */
 
-    /** Interval in which a full move selection is performed in the in-tree
-        phase at high parent counts.
-        Takes advantage of the fact that at high parent counts the selected
-        move is almost always the same as at the last visit of the node,
-        so it will speed up the in-tree phase if we do a full move selection
-        only at a certain intervals and play the move from the sequence of
-        the last simulation otherwise.
-        The default value is 1, which means that full move selection is always
-        used.
-        @see set_full_select_min. */
-    void set_full_select_interval(unsigned n);
-
-    /** The minimum node count to do a full move selection only in a certain
-        interval.
-        See set_full_select_interval(). */
-    void set_full_select_min(Float n);
-
     /** Minimum count of a node to be expanded. */
     void set_expand_threshold(Float n);
 
@@ -259,19 +242,13 @@ public:
 
     Float get_expand_threshold_inc() const;
 
-    /** Constant factor used in bias term.
-        The bias term is defined differently from standard UCT, but it only
-        makes a difference at small counts and doesn't affect convergence:
-        - log_2 is used for faster computation,
-        - +1 is added to the parent count (visit_count is incremented before
-          select_child()) to avoid handling the special case 0,
-        - The child uses the value count, which includes prior knowledge
-          initialization (which must be greater 0) and (our version of) RAVE.
-          This avoids having to select each child once at the beginning, which
-          is costly at large branching factors. */
-    void set_bias_term_constant(Float c);
+    /** Constant used in the exploration term.
+        The exploration term has the form c * sqrt(parent_count) / child_count
+        with a configurable constant c. It assumes that children counts are
+        initialized greater than 0. */
+    void set_exploration_constant(Float c) { m_exploration_constant = c; }
 
-    Float get_bias_term_constant() const;
+    Float get_exploration_constant() const { return m_exploration_constant; }
 
     /** Reuse the subtree from the previous search if the current position is
         a follow-up position of the previous one. */
@@ -300,15 +277,6 @@ public:
 
     Float get_rave_weight() const;
 
-    /** Set deterministic mode.
-        Note that using a fixed number of simulations instead of a time limit
-        is not enough to make the search fully deterministic because the
-        interval in which expensive abort conditions are checked (e.g. if the
-        best move cannot change anymore) is adjusted dynamically depending on
-        the number of simulations per second. In deterministic mode, a
-        fixed interval is used. */
-    void set_deterministic();
-
     /** @} */ // @name
 
 
@@ -414,8 +382,6 @@ private:
 
         unsigned thread_id;
 
-        unsigned full_select_counter;
-
         /** Was the search in this thread terminated because the search tree
             was full? */
         bool is_out_of_mem;
@@ -424,11 +390,6 @@ private:
 
         StatisticsExt<> stat_len;
 
-        /** Statistics about in-tree length where the full child select was
-            skipped.
-            @see SearchBase::set_full_select_interval() */
-        StatisticsExt<> stat_fs_len;
-
         StatisticsExt<> stat_in_tree_len;
 
         /** Local variable for update_rave().
@@ -511,17 +472,11 @@ private:
 
     unsigned m_nu_threads;
 
-    /** See set_full_select_interval(). */
-    unsigned m_full_select_interval = 1;
-
-    /** See set_full_select_min(). */
-    Float m_full_select_min = numeric_limits<Float>::max();
-
     Float m_expand_threshold = 0;
 
     Float m_expand_threshold_inc = 0;
 
-    bool m_deterministic = false;
+    bool m_deterministic;
 
     bool m_reuse_subtree = true;
 
@@ -558,7 +513,7 @@ private:
 
     TimeSource* m_time_source;
 
-    Float m_bias_term_constant;
+    Float m_exploration_constant;
 
     Timer m_timer;
 
@@ -608,9 +563,7 @@ private:
 
 
 template<class S, class M, class R>
-SearchBase<S, M, R>::ThreadState::~ThreadState()
-{
-}
+SearchBase<S, M, R>::ThreadState::~ThreadState() = default;
 
 template<class S, class M, class R>
 SearchBase<S, M, R>::Thread::Thread(SearchFunc& search_func)
@@ -684,15 +637,14 @@ void SearchBase<S, M, R>::Thread::wait_search_finished()
 
 #if LIBBOARDGAME_DEBUG
 template<class S, class M, class R>
-SearchBase<S, M, R>::AssertionHandler::AssertionHandler(const SearchBase& search)
+SearchBase<S, M, R>::AssertionHandler::AssertionHandler(
+        const SearchBase& search)
     : m_search(search)
 {
 }
 
 template<class S, class M, class R>
-SearchBase<S, M, R>::AssertionHandler::~AssertionHandler()
-{
-}
+SearchBase<S, M, R>::AssertionHandler::~AssertionHandler() = default;
 
 template<class S, class M, class R>
 void SearchBase<S, M, R>::AssertionHandler::run()
@@ -706,7 +658,7 @@ template<class S, class M, class R>
 SearchBase<S, M, R>::SearchBase(unsigned nu_threads, size_t memory)
     : m_tree(memory / 2, nu_threads),
       m_nu_threads(nu_threads),
-      m_bias_term_constant(0),
+      m_exploration_constant(0),
       m_tmp_tree(memory / 2, m_nu_threads)
 #if LIBBOARDGAME_DEBUG
       , m_assertion_handler(*this)
@@ -714,8 +666,7 @@ SearchBase<S, M, R>::SearchBase(unsigned nu_threads, size_t memory)
 { }
 
 template<class S, class M, class R>
-SearchBase<S, M, R>::~SearchBase()
-{ }
+SearchBase<S, M, R>::~SearchBase() = default;
 
 template<class S, class M, class R>
 bool SearchBase<S, M, R>::check_abort(const ThreadState& thread_state) const
@@ -885,8 +836,8 @@ bool SearchBase<S, M, R>::expand_node(ThreadState& thread_state,
     auto thread_id = thread_state.thread_id;
     typename Tree::NodeExpander expander(thread_id, m_tree,
                                          SearchParamConst::child_min_count);
-    auto init_val = m_root_val[state.get_player()].get_mean();
-    if (state.gen_children(expander, init_val))
+    auto root_val = m_root_val[state.get_player()].get_mean();
+    if (state.gen_children(expander, root_val))
     {
         expander.link_children(m_tree, node);
         best_child = expander.get_best_child();
@@ -895,12 +846,6 @@ bool SearchBase<S, M, R>::expand_node(ThreadState& thread_state,
     return false;
 }
 
-template<class S, class M, class R>
-inline auto SearchBase<S, M, R>::get_bias_term_constant() const -> Float
-{
-    return m_bias_term_constant;
-}
-
 template<class S, class M, class R>
 inline auto SearchBase<S, M, R>::get_expand_threshold() const -> Float
 {
@@ -1014,13 +959,11 @@ void SearchBase<S, M, R>::playout(ThreadState& thread_state)
     auto nu_moves = moves.size();
     Move last = nu_moves > 0 ? moves[nu_moves - 1].move : Move::null();
     Move second_last = nu_moves > 1 ? moves[nu_moves - 2].move : Move::null();
-    while (true)
+    PlayerMove mv;
+    while (state.gen_playout_move(m_lgr, last, second_last, mv))
     {
-        PlayerMove mv;
-        if (! state.gen_playout_move(m_lgr, last, second_last, mv))
-            break;
-        simulation.moves.push_back(mv);
         state.play_playout(mv.move);
+        moves.push_back(mv);
         second_last = last;
         last = mv.move;
     }
@@ -1031,43 +974,14 @@ void SearchBase<S, M, R>::play_in_tree(ThreadState& thread_state)
 {
     auto& state = *thread_state.state;
     auto& simulation = thread_state.simulation;
+    simulation.nodes.resize(1);
+    simulation.moves.clear();
     auto& root = m_tree.get_root();
     auto node = &root;
-    m_tree.inc_visit_count(*node);
     Float expand_threshold = m_expand_threshold;
-    if (thread_state.full_select_counter > 0)
-    {
-        // Don't do a full move select but play same sequence as last time
-        // as long as the node count is above m_full_select_min.
-        --thread_state.full_select_counter;
-        unsigned depth = 0;
-        while (node->has_children())
-        {
-            if (node->get_visit_count() <= m_full_select_min
-                    || depth + 1 >= simulation.nodes.size())
-                break;
-            node = simulation.nodes[depth + 1];
-            m_tree.inc_visit_count(*node);
-            if (multithread && SearchParamConst::virtual_loss)
-                m_tree.add_value(*node, 0);
-            state.play_in_tree(node->get_move());
-            ++depth;
-            expand_threshold += m_expand_threshold_inc;
-        }
-        simulation.nodes.resize(depth + 1);
-        simulation.moves.resize(depth);
-    }
-    else
-    {
-        simulation.nodes.resize(1);
-        simulation.moves.clear();
-        thread_state.full_select_counter = m_full_select_interval;
-    }
-    thread_state.stat_fs_len.add(double(simulation.moves.size()));
     while (node->has_children())
     {
         node = select_child(*node);
-        m_tree.inc_visit_count(*node);
         if (multithread && SearchParamConst::virtual_loss)
             m_tree.add_value(*node, 0);
         simulation.nodes.push_back(node);
@@ -1115,7 +1029,6 @@ string SearchBase<S, M, R>::get_info() const
       << (double(m_nu_simulations) / m_last_time)
       << ", Len: " << thread_state.stat_len.to_string(true, 1, true)
       << "\nDp: " << thread_state.stat_in_tree_len.to_string(true, 1, true)
-      << ", FS: " << thread_state.stat_fs_len.to_string(true, 1, true)
       << "\n";
     return s.str();
 }
@@ -1135,7 +1048,7 @@ bool SearchBase<S, M, R>::prune(TimeSource& time_source, double time,
     LIBBOARDGAME_UNUSED(time);
 #endif
     Timer timer(time_source);
-    m_tmp_tree.clear(m_tree.get_root().get_value());
+    m_tmp_tree.clear();
     m_tree.copy_subtree(m_tmp_tree, m_tmp_tree.get_root(), m_tree.get_root(),
                         prune_min_count);
     int percent = int(m_tmp_tree.get_nu_nodes() * 100 / m_tree.get_nu_nodes());
@@ -1188,6 +1101,7 @@ bool SearchBase<S, M, R>::search(Move& mv, Float max_count,
 {
     if (m_nu_threads != m_threads.size())
         create_threads();
+    m_deterministic = RandomGenerator::has_global_seed();
     bool is_followup = check_followup(m_followup_sequence);
     on_start_search(is_followup);
     if (max_count > 0)
@@ -1226,7 +1140,7 @@ bool SearchBase<S, M, R>::search(Move& mv, Float max_count,
         else
         {
             Timer timer(time_source);
-            m_tmp_tree.clear(SearchParamConst::tie_value);
+            m_tmp_tree.clear();
             auto node = find_node(m_tree, m_followup_sequence);
             if (node)
             {
@@ -1258,7 +1172,7 @@ bool SearchBase<S, M, R>::search(Move& mv, Float max_count,
         }
     }
     if (clear_tree)
-        m_tree.clear(SearchParamConst::tie_value);
+        m_tree.clear();
 
     m_timer.reset(time_source);
     m_time_source = &time_source;
@@ -1267,10 +1181,8 @@ bool SearchBase<S, M, R>::search(Move& mv, Float max_count,
     for (auto& i : m_threads)
     {
         auto& thread_state = i->thread_state;
-        thread_state.full_select_counter = 0;
         thread_state.stat_len.clear();
         thread_state.stat_in_tree_len.clear();
-        thread_state.stat_fs_len.clear();
         thread_state.state->start_search();
     }
     m_max_count = max_count;
@@ -1280,8 +1192,6 @@ bool SearchBase<S, M, R>::search(Move& mv, Float max_count,
     Float prune_min_count = SearchParamConst::prune_count_start;
 
     // Don't use multi-threading for very short searches (less than 0.5s).
-    // There are too many lost updates at the beginning (e.g. if all threads
-    // expand the root node and only the children of the last thread are used)
     auto reused_count = m_tree.get_root().get_visit_count();
     unsigned nu_threads = m_nu_threads;
     double expected_time;
@@ -1384,19 +1294,14 @@ void SearchBase<S, M, R>::search_loop(ThreadState& thread_state)
 template<class S, class M, class R>
 inline auto SearchBase<S, M, R>::select_child(const Node& node) -> const Node*
 {
-    // See comments at set_bias_term_constant() for differences of bias term
-    // to standard UCT
     auto children = m_tree.get_children(node);
     LIBBOARDGAME_ASSERT(! children.empty());
     auto parent_count = node.get_visit_count();
-    LIBBOARDGAME_ASSERT(parent_count > 0);
-    Float bias_factor =
-            m_bias_term_constant
-            * sqrt(static_cast<Float>(rounded_log_2(parent_count)));
+    Float bias_factor = m_exploration_constant * sqrt(parent_count);
     static_assert(SearchParamConst::child_min_count > 0, "");
-    auto bias_limit = bias_factor / sqrt(SearchParamConst::child_min_count);
+    auto bias_limit = bias_factor / SearchParamConst::child_min_count;
     auto i = children.begin();
-    auto value = i->get_value() + bias_factor / sqrt(i->get_value_count());
+    auto value = i->get_value() + bias_factor / i->get_value_count();
     auto best_value = value;
     auto best_child = i;
     auto limit = best_value - bias_limit;
@@ -1405,7 +1310,7 @@ inline auto SearchBase<S, M, R>::select_child(const Node& node) -> const Node*
         value = i->get_value();
         if (value <= limit)
             continue;
-        value += bias_factor / sqrt(i->get_value_count());
+        value += bias_factor / i->get_value_count();
         if (value > best_value)
         {
             best_value = value;
@@ -1451,24 +1356,12 @@ bool SearchBase<S, M, R>::select_move(Move& mv) const
         return false;
 }
 
-template<class S, class M, class R>
-void SearchBase<S, M, R>::set_bias_term_constant(Float c)
-{
-    m_bias_term_constant = c;
-}
-
 template<class S, class M, class R>
 void SearchBase<S, M, R>::set_callback(function<void(double, double)> callback)
 {
     m_callback = callback;
 }
 
-template<class S, class M, class R>
-void SearchBase<S, M, R>::set_deterministic()
-{
-    m_deterministic = true;
-}
-
 template<class S, class M, class R>
 void SearchBase<S, M, R>::set_expand_threshold(Float n)
 {
@@ -1481,18 +1374,6 @@ void SearchBase<S, M, R>::set_expand_threshold_inc(Float n)
     m_expand_threshold_inc = n;
 }
 
-template<class S, class M, class R>
-void SearchBase<S, M, R>::set_full_select_interval(unsigned n)
-{
-    m_full_select_interval = n;
-}
-
-template<class S, class M, class R>
-void SearchBase<S, M, R>::set_full_select_min(Float n)
-{
-    m_full_select_min = n;
-}
-
 template<class S, class M, class R>
 void SearchBase<S, M, R>::set_rave_parent_max(Float n)
 {
@@ -1623,13 +1504,8 @@ void SearchBase<S, M, R>::update_rave(ThreadState& thread_state)
     }
 
     // Reset was_played
-    while (true)
-    {
-        ++i;
-        if (i >= nu_moves)
-            break;
+    while (++i < nu_moves)
         was_played[moves[i].move.to_int()] = max_players;
-    }
 }
 
 template<class S, class M, class R>
@@ -1639,6 +1515,7 @@ void SearchBase<S, M, R>::update_values(ThreadState& thread_state)
     auto& nodes = simulation.nodes;
     auto& eval = simulation.eval;
     unsigned nu_nodes = static_cast<unsigned>(nodes.size());
+    m_tree.inc_visit_count(*nodes[0]);
     for (unsigned i = 1; i < nu_nodes; ++i)
     {
         auto& node = *nodes[i];
@@ -1654,6 +1531,7 @@ void SearchBase<S, M, R>::update_values(ThreadState& thread_state)
             m_tree.add_value_remove_loss(node, eval[mv.player]);
         else
             m_tree.add_value(node, eval[mv.player]);
+        m_tree.inc_visit_count(node);
     }
     for (PlayerInt i = 0; i < m_nu_players; ++i)
         m_root_val[i].add(eval[i]);
