@@ -11,7 +11,6 @@
 #include "SgfUtil.h"
 
 #include <algorithm>
-#include <iostream>
 #include <sstream>
 #include "InvalidPropertyValue.h"
 #include "TreeWriter.h"
@@ -89,6 +88,25 @@ unsigned get_depth(const SgfNode& node)
     return depth;
 }
 
+const char* get_move_annotation(const SgfTree& tree, const SgfNode& node)
+{
+    double goodMove = tree.get_good_move(node);
+    if (goodMove > 1)
+        return "!!";
+    if (goodMove > 0)
+        return "!";
+    double badMove = tree.get_bad_move(node);
+    if (badMove > 1)
+        return "??";
+    if (badMove > 0)
+        return "?";
+    if (tree.is_interesting_move(node))
+        return "!?";
+    if (tree.is_doubtful_move(node))
+        return "?!";
+    return "";
+}
+
 const SgfNode* get_next_earlier_variation(const SgfNode& node)
 {
     auto child = &node;
@@ -170,6 +188,20 @@ bool has_earlier_variation(const SgfNode& node)
     }
 }
 
+bool is_empty(const SgfTree& tree)
+{
+    auto& root = tree.get_root();
+    if (root.has_children())
+        return false;
+    for (auto& p : root.get_properties())
+    {
+        auto& id = p.id;
+        if (id != "GM" && id != "CA" && id != "AP" && id != "DT")
+            return false;
+    }
+    return true;
+}
+
 bool is_main_variation(const SgfNode& node)
 {
     auto current = &node;
