@@ -11,6 +11,7 @@
 #include "Board.h"
 
 #include <functional>
+#include "CallistoGeometry.h"
 #include "MoveMarker.h"
 
 namespace libpentobi_base {
@@ -84,58 +85,66 @@ void Board::copy_from(const Board& bd)
 const Transform* Board::find_transform(Move mv) const
 {
     auto& geo = get_geometry();
-    auto& move_info = get_move_info(mv);
     PiecePoints points;
-    for (Point p : move_info)
+    for (Point p : get_move_points(mv))
         points.push_back(CoordPoint(geo.get_x(p), geo.get_y(p)));
-    return get_piece_info(move_info.get_piece()).find_transform(geo, points);
+    return get_piece_info(get_move_piece(mv)).find_transform(geo, points);
 }
 
 void Board::gen_moves(Color c, MoveMarker& marker, MoveList& moves) const
 {
     moves.clear();
-    if (is_first_piece(c))
+    bool is_callisto = (m_piece_set == PieceSet::callisto);
+    if (! is_callisto && is_first_piece(c))
     {
         for (Point p : get_starting_points(c))
             if (! m_state_color[c].forbidden[p])
-                gen_moves(c, p, get_adj_status(p, c), marker, moves);
-    }
-    else
-    {
-        for (Point p : get_attach_points(c))
-            if (! m_state_color[c].forbidden[p])
-                gen_moves(c, p, get_adj_status(p, c), marker, moves);
-    }
-}
-
-void Board::gen_moves(Color c, Point p, unsigned adj_status,
-                      MoveMarker& marker, MoveList& moves) const
-{
-    for (Piece piece : m_state_color[c].pieces_left)
-        for (Move mv : m_bc->get_moves(piece, p, adj_status))
-        {
-            if (marker[mv])
-                continue;
-            if (! is_forbidden(c, mv))
             {
-                moves.push_back(mv);
-                marker.set(mv);
+                auto adj_status = get_adj_status(p, c);
+                for (Piece piece : m_state_color[c].pieces_left)
+                    gen_moves(c, p, piece, adj_status, marker, moves);
             }
+        return;
+    }
+    if (is_callisto && is_piece_left(c, m_one_piece))
+        for (auto p : *m_geo)
+            if (! is_forbidden(p, c) && ! m_is_center_section[p])
+                gen_moves(c, p, m_one_piece, get_adj_status(p, c), marker,
+                          moves);
+    for (Point p : get_attach_points(c))
+        if (! m_state_color[c].forbidden[p])
+        {
+            auto adj_status = get_adj_status(p, c);
+            for (Piece piece : m_state_color[c].pieces_left)
+                if (! is_callisto || piece != m_one_piece)
+                    gen_moves(c, p, piece, adj_status, marker, moves);
         }
 }
 
-unsigned Board::get_bonus(Color c) const
+void Board::gen_moves(Color c, Point p, Piece piece, unsigned adj_status,
+                      MoveMarker& marker, MoveList& moves) const
+{
+    for (Move mv : m_bc->get_moves(piece, p, adj_status))
+        if (! marker[mv] && ! is_forbidden(c, mv))
+        {
+            moves.push_back(mv);
+            marker.set(mv);
+        }
+}
+
+ScoreType Board::get_bonus(Color c) const
 {
     if (get_pieces_left(c).size() > 0)
         return 0;
-    unsigned bonus = m_bonus_all_pieces;
+    auto bonus = m_bonus_all_pieces;
     unsigned i = m_moves.size();
     while (i > 0)
     {
         --i;
         if (m_moves[i].color == c)
         {
-            if (get_move_info(m_moves[i].move).size() == 1)
+            auto piece = get_move_piece(m_moves[i].move);
+            if (m_score_points[piece] == 1)
                 bonus += m_bonus_one_piece;
             break;
         }
@@ -145,24 +154,35 @@ unsigned Board::get_bonus(Color c) const
 
 Color Board::get_effective_to_play() const
 {
-    Color c = m_state_base.to_play;
+    return get_effective_to_play(get_to_play());
+}
+
+Color Board::get_effective_to_play(Color c) const
+{
+    Color result = c;
     do
     {
-        if (has_moves(c))
-            return c;
-        c = get_next(c);
+        if (has_moves(result))
+            return result;
+        result = get_next(result);
     }
-    while (c != m_state_base.to_play);
-    return c;
+    while (result != c);
+    return result;
 }
 
 void Board::get_place(Color c, unsigned& place, bool& is_shared) const
 {
-    array<int,Color::range> all_scores;
+    bool break_ties = (m_piece_set == PieceSet::callisto);
+    array<ScoreType, Color::range> all_scores;
     for (Color::IntType i = 0; i < Color::range; ++i)
+    {
         all_scores[i] = get_score(Color(i));
-    int score = all_scores[c.to_int()];
-    sort(all_scores.begin(), all_scores.begin() + m_nu_players, greater<int>());
+        if (break_ties)
+            all_scores[i] += i * 0.0001f;
+    }
+    auto score = all_scores[c.to_int()];
+    sort(all_scores.begin(), all_scores.begin() + m_nu_players,
+         greater<ScoreType>());
     is_shared = false;
     bool found = false;
     for (unsigned i = 0; i < m_nu_players; ++i)
@@ -185,13 +205,13 @@ Move Board::get_move_at(Point p) const
     {
         auto c = s.to_color();
         for (Move mv : m_setup.placements[c])
-            if (get_move_info(mv).contains(p))
+            if (get_move_points(mv).contains(p))
                 return mv;
         for (ColorMove color_mv : m_moves)
             if (color_mv.color == c)
             {
                 Move mv = color_mv.move;
-                if (get_move_info(mv).contains(p))
+                if (get_move_points(mv).contains(p))
                     return mv;
             }
     }
@@ -200,18 +220,21 @@ Move Board::get_move_at(Point p) const
 
 bool Board::has_moves(Color c) const
 {
-    if (is_first_piece(c))
+    bool is_callisto = (m_piece_set == PieceSet::callisto);
+    if (is_callisto && is_piece_left(c, m_one_piece))
+        for (auto p : *m_geo)
+            if (! is_forbidden(p, c) && ! m_is_center_section[p])
+                return true;
+    if (! is_callisto && is_first_piece(c))
     {
         for (auto p : get_starting_points(c))
             if (has_moves(c, p))
                 return true;
+        return false;
     }
-    else
-    {
-        for (auto p : get_attach_points(c))
-            if (has_moves(c, p))
-                return true;
-    }
+    for (auto p : get_attach_points(c))
+        if (has_moves(c, p))
+            return true;
     return false;
 }
 
@@ -219,11 +242,19 @@ bool Board::has_moves(Color c, Point p) const
 {
     if (is_forbidden(p, c))
         return false;
+    bool is_callisto = (m_piece_set == PieceSet::callisto);
+    if (is_callisto && is_piece_left(c, m_one_piece))
+        if (m_is_center_section[p])
+            return true;
     auto adj_status = get_adj_status(p, c);
     for (auto piece : m_state_color[c].pieces_left)
+    {
+        if (piece == m_one_piece && is_callisto)
+            continue;
         for (auto mv : m_bc->get_moves(piece, p, adj_status))
             if (! is_forbidden(c, mv))
                 return true;
+    }
     return false;
 }
 
@@ -243,21 +274,22 @@ void Board::init(Variant variant, const Setup* setup)
     // If you make changes here, make sure that you also update copy_from()
 
     m_state_base.point_state.fill(PointState::empty(), *m_geo);
-    if (variant == Variant::junior)
-        m_nu_piece_instances = 2;
-    else
-        m_nu_piece_instances = 1;
     for (Color c : get_colors())
     {
-        m_state_color[c].forbidden.fill(false, *m_geo);
-        m_state_color[c].is_attach_point.fill(false, *m_geo);
+        auto& state = m_state_color[c];
+        state.forbidden.fill(false, *m_geo);
+        state.is_attach_point.fill(false, *m_geo);
+        state.pieces_left.clear();
+        state.nu_onboard_pieces = 0;
+        state.points = 0;
+        for (Piece::IntType i = 0; i < get_nu_uniq_pieces(); ++i)
+        {
+            Piece piece(i);
+            state.pieces_left.push_back(piece);
+            state.nu_left_piece[piece] =
+                    static_cast<uint_fast8_t>(get_nu_piece_instances(piece));
+        }
         m_attach_points[c].clear();
-        m_state_color[c].pieces_left.clear();
-        m_state_color[c].nu_onboard_pieces = 0;
-        m_state_color[c].points = 0;
-        for (Piece::IntType j = 0; j < get_nu_uniq_pieces(); ++j)
-            m_state_color[c].pieces_left.push_back(Piece(j));
-        m_state_color[c].nu_left_piece.fill(m_nu_piece_instances);
     }
     m_state_base.nu_onboard_pieces_all = 0;
     if (! setup)
@@ -308,26 +340,39 @@ void Board::init_variant(Variant variant)
     }
     m_nu_players = libpentobi_base::get_nu_players(variant);
     m_bc = &BoardConst::get(variant);
-    if (m_variant == Variant::junior)
+    m_piece_set = m_bc->get_piece_set();
+    m_is_callisto = (m_piece_set == PieceSet::callisto);
+    if ((m_piece_set == PieceSet::classic && variant != Variant::junior)
+            || m_piece_set == PieceSet::trigon)
     {
-        m_bonus_all_pieces = 0;
-        m_bonus_one_piece = 0;
+        m_bonus_all_pieces = 15;
+        m_bonus_one_piece = 5;
     }
-    else if (m_variant == Variant::nexos || m_variant == Variant::nexos_2)
+    else if (m_piece_set == PieceSet::nexos)
     {
         m_bonus_all_pieces = 10;
         m_bonus_one_piece = 0;
     }
     else
     {
-        m_bonus_all_pieces = 15;
-        m_bonus_one_piece = 5;
+        m_bonus_all_pieces = 0;
+        m_bonus_one_piece = 0;
     }
+    m_max_piece_size = m_bc->get_max_piece_size();
+    m_max_adj_attach = m_bc->get_max_adj_attach();
     m_geo = &m_bc->get_geometry();
     m_move_info_array = m_bc->get_move_info_array();
     m_move_info_ext_array = m_bc->get_move_info_ext_array();
     m_move_info_ext_2_array = m_bc->get_move_info_ext_2_array();
     m_starting_points.init(variant, *m_geo);
+    if (m_piece_set == PieceSet::callisto)
+        for (Point p : *m_geo)
+            m_is_center_section[p] =
+                    CallistoGeometry::is_center_section(m_geo->get_x(p),
+                                                        m_geo->get_y(p),
+                                                        m_nu_players);
+    else
+        m_is_center_section.fill(false, *m_geo);
     for (Color c : get_colors())
     {
         if (m_nu_players == 2 && m_nu_colors == 4)
@@ -335,32 +380,13 @@ void Board::init_variant(Variant variant)
         else
             m_second_color[c] = c;
     }
-    auto piece_set = m_bc->get_piece_set();
-#if LIBBOARDGAME_DEBUG
-    unsigned total_piece_points = 0;
-#endif
     for (Piece::IntType i = 0; i < get_nu_uniq_pieces(); ++i)
     {
         Piece piece(i);
-        m_score_points[piece] = get_piece_info(piece).get_score_points();
-#if LIBBOARDGAME_DEBUG
-        total_piece_points += m_score_points[piece];
-#endif
-    }
-    switch (piece_set)
-    {
-    case PieceSet::classic:
-        LIBBOARDGAME_ASSERT(total_piece_points == 89);
-        break;
-    case PieceSet::junior:
-        LIBBOARDGAME_ASSERT(total_piece_points == 44);
-        break;
-    case PieceSet::trigon:
-        LIBBOARDGAME_ASSERT(total_piece_points == 110);
-        break;
-    case PieceSet::nexos:
-        LIBBOARDGAME_ASSERT(total_piece_points == 84);
-        break;
+        auto& piece_info = get_piece_info(piece);
+        m_score_points[piece] = piece_info.get_score_points();
+        if (piece_info.get_points().size() == 1)
+            m_one_piece = piece;
     }
 }
 
@@ -374,12 +400,13 @@ bool Board::is_game_over() const
 
 bool Board::is_legal(Color c, Move mv) const
 {
-    auto& info = get_move_info(mv);
-    if (! is_piece_left(c, info.get_piece()))
+    auto piece = get_move_piece(mv);
+    if (! is_piece_left(c, piece))
         return false;
+    auto points = get_move_points(mv);
+    auto i = points.begin();
+    auto end = points.end();
     bool has_attach_point = false;
-    auto i = info.begin();
-    auto end = info.end();
     do
     {
         if (m_state_color[c].forbidden[*i])
@@ -388,15 +415,23 @@ bool Board::is_legal(Color c, Move mv) const
             has_attach_point = true;
     }
     while (++i != end);
+    if (m_is_callisto)
+    {
+        if (m_state_color[c].nu_left_piece[m_one_piece] > 1
+                && piece != m_one_piece)
+            return false;
+        if (piece == m_one_piece)
+            return ! m_is_center_section[*points.begin()];
+    }
     if (has_attach_point)
         return true;
     if (! is_first_piece(c))
         return false;
-    i = info.begin();
+    i = points.begin();
     do
         if (is_colorless_starting_point(*i)
-            || (is_colored_starting_point(*i)
-                && get_starting_point_color(*i) == c))
+                || (is_colored_starting_point(*i)
+                    && get_starting_point_color(*i) == c))
             return true;
     while (++i != end);
     return false;
@@ -420,15 +455,31 @@ void Board::optimize_attach_point_lists()
     }
 }
 
-/** Place setup moves on board.
-    This function is only extracted from init() because it contains the large
-    inline function place(), which will cause a failure to inline and warning
-    with -Winline with GCC 4.8.2. */
+/** Place setup moves on board. */
 void Board::place_setup(const Setup& setup)
 {
-    for (Color c : get_colors())
-        for (Move mv : setup.placements[c])
-            place(c, mv);
+    if (m_max_piece_size == 5)
+        for (Color c : get_colors())
+            for (Move mv : setup.placements[c])
+                place<5, 16>(c, mv);
+    else if (m_max_piece_size == 6)
+        for (Color c : get_colors())
+            for (Move mv : setup.placements[c])
+                place<6, 22>(c, mv);
+    else
+        for (Color c : get_colors())
+            for (Move mv : setup.placements[c])
+                place<7, 12>(c, mv);
+}
+
+void Board::play(Color c, Move mv)
+{
+    if (m_max_piece_size == 5)
+        play<5, 16>(c, mv);
+    else if (m_max_piece_size == 6)
+        play<6, 22>(c, mv);
+    else
+        play<7, 12>(c, mv);
 }
 
 void Board::take_snapshot()
@@ -455,16 +506,6 @@ void Board::take_snapshot()
     }
 }
 
-void Board::undo()
-{
-    LIBBOARDGAME_ASSERT(get_nu_moves() > 0);
-    ArrayList<ColorMove, max_game_moves> moves = m_moves;
-    moves.pop_back();
-    init();
-    for (auto& mv : moves)
-        play(mv);
-}
-
 void Board::write(ostream& out, bool mark_last_move) const
 {
     // Sort lists of left pieces by name
@@ -491,10 +532,9 @@ void Board::write(ostream& out, bool mark_last_move) const
     unsigned width = m_geo->get_width();
     unsigned height = m_geo->get_height();
     bool is_info_location_right = (width <= 20);
-    auto board_type = get_board_type();
-    bool is_trigon = (board_type == BoardType::trigon
-                      || board_type == BoardType::trigon_3);
-    bool is_nexos = (board_type == BoardType::nexos);
+    bool is_trigon = (m_piece_set == PieceSet::trigon);
+    bool is_nexos = (m_piece_set == PieceSet::nexos);
+    bool is_callisto = (m_piece_set == PieceSet::callisto);
     for (unsigned y = 0; y < height; ++y)
     {
         if (height - y < 10)
@@ -514,7 +554,7 @@ void Board::write(ostream& out, bool mark_last_move) const
                 // less-than/greater-than character is used instead of the
                 // space to mark the last piece played.
                 if (! last_mv.is_null()
-                        && get_move_info(last_mv.move).contains(p)
+                        && get_move_points(last_mv.move).contains(p)
                         && (x == 0 || ! m_geo->is_onboard(x - 1, y)
                             || get_point_state(m_geo->get_point(x - 1, y))
                                != last_mv.color))
@@ -525,9 +565,11 @@ void Board::write(ostream& out, bool mark_last_move) const
                 }
                 else if (! last_mv.is_null()
                          && x > 0 && m_geo->is_onboard(x - 1, y)
-                         && get_move_info(last_mv.move).contains(m_geo->get_point(x - 1, y))
+                         && get_move_points(last_mv.move).contains(
+                                                   m_geo->get_point(x - 1, y))
                          && get_point_state(p) != last_mv.color
-                         && get_point_state(m_geo->get_point(x - 1, y)) == last_mv.color)
+                         && get_point_state(m_geo->get_point(x - 1, y))
+                                == last_mv.color)
                 {
                     set_color(out, "\x1B[1;37;47m");
                     out << '<';
@@ -551,6 +593,11 @@ void Board::write(ostream& out, bool mark_last_move) const
                     set_color(out, "\x1B[1;30;47m");
                     out << (point_type == 1 ? '\\' : '/');
                 }
+                else if (is_callisto && x == 0)
+                {
+                    set_color(out, "\x1B[0m");
+                    out << ' ';
+                }
                 else
                 {
                     set_color(out, is_nexos ? "\x1B[1;30;47m" : "\x1B[0m");
@@ -584,6 +631,8 @@ void Board::write(ostream& out, bool mark_last_move) const
                             out << '|';
                         else if (is_nexos && point_type == 0)
                             out << '+';
+                        else if (is_callisto && is_center_section(p))
+                            out << ',';
                         else
                             out << '.';
                     }
