@@ -55,24 +55,15 @@ public:
         Use copy_from() to copy a board state. */
     Board& operator=(const Board&) = delete;
 
-    Geometry::Iterator begin() const
-    {
-        return m_geo->begin();
-    }
+    Geometry::Iterator begin() const { return m_geo->begin(); }
 
-    Geometry::Iterator end() const
-    {
-        return m_geo->end();
-    }
+    Geometry::Iterator end() const { return m_geo->end(); }
 
     Variant get_variant() const;
 
     Color::IntType get_nu_colors() const;
 
-    Color::Range get_colors() const
-    {
-        return Color::Range(m_nu_colors);
-    }
+    Color::Range get_colors() const { return Color::Range(m_nu_colors); }
 
     /** Number of colors that are not played alternately.
         This is equal to get_nu_colors() apart from Variant::classic_3. */
@@ -80,19 +71,10 @@ public:
 
     unsigned get_nu_players() const;
 
-
     Piece::IntType get_nu_uniq_pieces() const;
 
-    /** Get number of pieces per player in the current game variant. */
-    unsigned get_nu_pieces() const;
-
-    /** Number of instances of each unique piece per color. */
-    unsigned get_nu_piece_instances() const;
-
-    unsigned get_max_player_moves() const;
-
-    /** Maximum number of moves in the current game variant. */
-    unsigned get_max_game_moves() const;
+    /** Number of instances of a unique piece per color. */
+    unsigned get_nu_piece_instances(Piece piece) const;
 
     Color get_next(Color c) const;
 
@@ -114,11 +96,14 @@ public:
         Variant::classic_3. */
     Color::IntType get_alt_player() const;
 
-    /** Get next color to play that still has moves.
-        Colors are tried in their playing order starting with get_to_play().
-        If no color has moves left, get_to_play() is returned. */
+    /** Equivalent to get_effective_to_play(get_to_play()) */
     Color get_effective_to_play() const;
 
+    /** Get next color to play that still has moves.
+        Colors are tried in their playing order starting with c. If no color
+        has moves left, c is returned. */
+    Color get_effective_to_play(Color c) const;
+
     const PiecesLeftList& get_pieces_left(Color c) const;
 
     bool is_piece_left(Color c, Piece piece) const;
@@ -133,10 +118,10 @@ public:
     unsigned get_nu_left_piece(Color c, Piece piece) const;
 
     /** Get number of points of a color including the bonus. */
-    unsigned get_points(Color c) const;
+    ScoreType get_points(Color c) const { return m_state_color[c].points; }
 
     /** Get number of bonus points of a color. */
-    unsigned get_bonus(Color c) const;
+    ScoreType get_bonus(Color c) const;
 
     /** Is a point a potential attachment point for a color.
         Does not check if the point is forbidden. */
@@ -167,18 +152,16 @@ public:
         @pre get_nu_moves() < max_game_moves */
     void play(Color c, Move mv);
 
-    /** Play a move for the current color to play.
-        @pre ! mv.is_null()
-        @pre get_nu_moves() < max_game_moves */
-    void play(Move mv);
+    /** More efficient version of play() if maximum piece size of current
+        game variant is known at compile time. */
+    template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH>
+    void play(Color c, Move mv);
 
     /** Play a move.
         @pre ! mv.move.is_null()
         @pre get_nu_moves() < max_game_moves */
     void play(ColorMove mv);
 
-    void undo();
-
     void set_to_play(Color c);
 
     void write(ostream& out, bool mark_last_move = true) const;
@@ -225,10 +208,6 @@ public:
         @pre ! mv.is_null() */
     bool is_legal(Color c, Move mv) const;
 
-    /** Check if a move is legal that is not a starting move.
-        @pre ! mv.is_null() */
-    bool is_legal_nonfirst(Color c, Move mv) const;
-
     /** Check if a move is legal for the current color to play.
         @pre ! mv.is_null() */
     bool is_legal(Move mv) const;
@@ -245,12 +224,19 @@ public:
         occupied point of the same color. */
     bool is_forbidden(Color c, Move mv) const;
 
-    const BoardConst& get_board_const() const;
+    const BoardConst& get_board_const() const { return *m_bc; }
 
     BoardType get_board_type() const;
 
+    PieceSet get_piece_set() const { return m_piece_set; }
+
     unsigned get_adj_status(Point p, Color c) const;
 
+    /** Is a point in the center section that is forbidden for the 1-piece in
+        Callisto?
+        Always returns false for other game variants. */
+    bool is_center_section(Point p) const { return m_is_center_section[p]; }
+
     PrecompMoves::Range get_moves(Piece piece, Point p,
                                   unsigned adj_status) const;
 
@@ -258,23 +244,23 @@ public:
         The score is the number of points for a color minus the number of
         points of the opponent (or the average score of the opponents if there
         are more than two players). */
-    int get_score(Color c) const;
+    ScoreType get_score(Color c) const;
 
     /** Specialized version of get_score().
         @pre get_nu_colors() == 2 */
-    int get_score_twocolor(Color c) const;
+    ScoreType get_score_twocolor(Color c) const;
 
     /** Specialized version of get_score().
         @pre get_nu_players() == 4 && get_nu_colors() == 4 */
-    int get_score_multicolor(Color c) const;
+    ScoreType get_score_multicolor(Color c) const;
 
     /** Specialized version of get_score().
         @pre get_nu_players() > 2 */
-    int get_score_multiplayer(Color c) const;
+    ScoreType get_score_multiplayer(Color c) const;
 
     /** Specialized version of get_score().
         @pre get_nu_players() == 2 */
-    int get_score_twoplayer(Color c) const;
+    ScoreType get_score_twoplayer(Color c) const;
 
     /** Get the place of a player in the game result.
         @param c The color of the player.
@@ -285,7 +271,7 @@ public:
         @param[out] is_shared True if the place was shared. */
     void get_place(Color c, unsigned& place, bool& is_shared) const;
 
-    const Geometry& get_geometry() const;
+    const Geometry& get_geometry() const { return *m_geo; }
 
     /** See BoardConst::to_string() */
     string to_string(Move mv, bool with_piece_name = false) const;
@@ -303,9 +289,12 @@ public:
 
     bool get_piece_by_name(const string& name, Piece& piece) const;
 
-    const MoveInfo& get_move_info(Move mv) const;
+    /** The 1x1 piece. */
+    Piece get_one_piece() const { return m_one_piece; }
 
-    const MoveInfoExt& get_move_info_ext(Move mv) const;
+    Range<const Point> get_move_points(Move mv) const;
+
+    Piece get_move_piece(Move mv) const;
 
     const MoveInfoExt2& get_move_info_ext_2(Move mv) const;
 
@@ -316,7 +305,7 @@ public:
     Color get_starting_point_color(Point p) const;
 
     const ArrayList<Point,StartingPoints::max_starting_points>&
-                                            get_starting_points(Color c) const;
+    get_starting_points(Color c) const;
 
     /** Get the second color in game variants in which a player plays two
         colors.
@@ -361,7 +350,7 @@ private:
 
         unsigned nu_onboard_pieces;
 
-        unsigned points;
+        ScoreType points;
     };
 
     /** Snapshot for fast restoration of a previous position. */
@@ -376,44 +365,55 @@ private:
         ColorMap<unsigned> attach_points_size;
     };
 
+
     StateBase m_state_base;
 
     ColorMap<StateColor> m_state_color;
 
     Variant m_variant;
 
+    PieceSet m_piece_set;
+
     Color::IntType m_nu_colors;
 
+    bool m_is_callisto;
+
     unsigned m_nu_players;
 
+    /** Caches m_bc->get_max_piece_size(). */
+    unsigned m_max_piece_size;
+
+    /** Caches m_bc->get_max_adj_attach(). */
+    unsigned m_max_adj_attach;
+
     /** Bonus for playing all pieces. */
-    unsigned m_bonus_all_pieces;
+    ScoreType m_bonus_all_pieces;
 
     /** Bonus for playing the 1-piece last. */
-    unsigned m_bonus_one_piece;
+    ScoreType m_bonus_one_piece;
 
-    /** Score points of a piece.
-        This is equal to the number of points of a piece apart from game
-        variant Nexos, in which the piece points contain some junction
-        points. */
-    PieceMap<unsigned> m_score_points;
-
-    /** See get_nu_piece_instances() */
-    uint_fast8_t m_nu_piece_instances;
+    /** Caches get_piece_info(piece).get_score_points() */
+    PieceMap<ScoreType> m_score_points;
 
     const BoardConst* m_bc;
 
-    /** Same as m_bc->get_move_info_array() */
-    const MoveInfo* m_move_info_array;
+    /** Caches m_bc->get_move_info_array() */
+    BoardConst::MoveInfoArray m_move_info_array;
 
-    /** Same as m_bc->get_move_info_ext_array() */
-    const MoveInfoExt* m_move_info_ext_array;
+    /** Caches m_bc->get_move_info_ext_array() */
+    BoardConst::MoveInfoExtArray m_move_info_ext_array;
 
-    /** Same as m_bc->get_move_info_ext_2_array() */
+    /** Caches m_bc->get_move_info_ext_2_array() */
     const MoveInfoExt2* m_move_info_ext_2_array;
 
     const Geometry* m_geo;
 
+    /** See is_center_section(). */
+    Grid<bool> m_is_center_section;
+
+    /** The 1x1 piece. */
+    Piece m_one_piece;
+
     ColorMap<PointList> m_attach_points;
 
     /** See get_second_color() */
@@ -436,8 +436,8 @@ private:
     StartingPoints m_starting_points;
 
 
-    void gen_moves(Color c, Point p, unsigned adj_status, MoveMarker& marker,
-                   MoveList& moves) const;
+    void gen_moves(Color c, Point p, Piece piece, unsigned adj_status,
+                   MoveMarker& marker, MoveList& moves) const;
 
     bool has_moves(Color c, Point p) const;
 
@@ -445,6 +445,7 @@ private:
 
     void optimize_attach_point_lists();
 
+    template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH>
     void place(Color c, Move mv);
 
     void place_setup(const Setup& setup);
@@ -502,50 +503,16 @@ inline const PointList&  Board::get_attach_points(Color c) const
     return m_attach_points[c];
 }
 
-inline const BoardConst& Board::get_board_const() const
-{
-    return *m_bc;
-}
-
 inline BoardType Board::get_board_type() const
 {
     return m_bc->get_board_type();
 }
 
-inline const Geometry& Board::get_geometry() const
-{
-    return *m_geo;
-}
-
-inline unsigned Board::get_max_game_moves() const
-{
-    return m_nu_colors * get_max_player_moves();
-}
-
-inline unsigned Board::get_max_player_moves() const
-{
-    return get_nu_pieces();
-}
-
 inline ColorMove Board::get_move(unsigned n) const
 {
     return m_moves[n];
 }
 
-inline const MoveInfo& Board::get_move_info(Move mv) const
-{
-    LIBBOARDGAME_ASSERT(! mv.is_null());
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_moves());
-    return *(m_move_info_array + mv.to_int());
-}
-
-inline const MoveInfoExt& Board::get_move_info_ext(Move mv) const
-{
-    LIBBOARDGAME_ASSERT(! mv.is_null());
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_moves());
-    return *(m_move_info_ext_array + mv.to_int());
-}
-
 inline const MoveInfoExt2& Board::get_move_info_ext_2(Move mv) const
 {
     LIBBOARDGAME_ASSERT(! mv.is_null());
@@ -553,6 +520,16 @@ inline const MoveInfoExt2& Board::get_move_info_ext_2(Move mv) const
     return *(m_move_info_ext_2_array + mv.to_int());
 }
 
+inline Piece Board::get_move_piece(Move mv) const
+{
+    return m_bc->get_move_piece(mv);
+}
+
+inline Range<const Point> Board::get_move_points(Move mv) const
+{
+    return m_bc->get_move_points(mv);
+}
+
 inline auto Board::get_moves() const
 -> const ArrayList<ColorMove, max_game_moves>&
 {
@@ -606,14 +583,9 @@ inline unsigned Board::get_nu_players() const
     return m_nu_players;
 }
 
-inline unsigned Board::get_nu_piece_instances() const
+inline unsigned Board::get_nu_piece_instances(Piece piece) const
 {
-    return m_nu_piece_instances;
-}
-
-inline unsigned Board::get_nu_pieces() const
-{
-    return m_nu_piece_instances * m_bc->get_nu_pieces();
+    return m_bc->get_piece_info(piece).get_nu_instances();
 }
 
 inline Piece::IntType Board::get_nu_uniq_pieces() const
@@ -646,17 +618,12 @@ inline const Board::PointStateGrid& Board::get_point_state() const
     return m_state_base.point_state;
 }
 
-inline unsigned Board::get_points(Color c) const
-{
-    return m_state_color[c].points;
-}
-
 inline Color Board::get_previous(Color c) const
 {
     return c.get_previous(m_nu_colors);
 }
 
-inline int Board::get_score(Color c) const
+inline ScoreType Board::get_score(Color c) const
 {
     if (m_nu_colors == 2)
         return get_score_twocolor(c);
@@ -666,18 +633,18 @@ inline int Board::get_score(Color c) const
         return get_score_multiplayer(c);
 }
 
-inline int Board::get_score_twocolor(Color c) const
+inline ScoreType Board::get_score_twocolor(Color c) const
 {
     LIBBOARDGAME_ASSERT(m_nu_colors == 2);
-    unsigned points0 = get_points(Color(0));
-    unsigned points1 = get_points(Color(1));
+    auto points0 = get_points(Color(0));
+    auto points1 = get_points(Color(1));
     if (c == Color(0))
         return points0 - points1;
     else
         return points1 - points0;
 }
 
-inline int Board::get_score_twoplayer(Color c) const
+inline ScoreType Board::get_score_twoplayer(Color c) const
 {
     LIBBOARDGAME_ASSERT(m_nu_players == 2);
     if (m_nu_colors == 2)
@@ -686,26 +653,26 @@ inline int Board::get_score_twoplayer(Color c) const
         return get_score_multicolor(c);
 }
 
-inline int Board::get_score_multicolor(Color c) const
+inline ScoreType Board::get_score_multicolor(Color c) const
 {
     LIBBOARDGAME_ASSERT(m_nu_players == 2 && m_nu_colors == 4);
-    unsigned points0 = get_points(Color(0)) + get_points(Color(2));
-    unsigned points1 = get_points(Color(1)) + get_points(Color(3));
+    auto points0 = get_points(Color(0)) + get_points(Color(2));
+    auto points1 = get_points(Color(1)) + get_points(Color(3));
     if (c == Color(0) || c == Color(2))
         return points0 - points1;
     else
         return points1 - points0;
 }
 
-inline int Board::get_score_multiplayer(Color c) const
+inline ScoreType Board::get_score_multiplayer(Color c) const
 {
     LIBBOARDGAME_ASSERT(m_nu_players > 2);
-    int score = 0;
+    ScoreType score = 0;
     auto nu_players = static_cast<Color::IntType>(m_nu_players);
     for (Color i : get_colors())
         if (i != c)
             score -= get_points(i);
-    score = get_points(c) + score / (static_cast<int>(nu_players) - 1);
+    score = get_points(c) + score / (static_cast<ScoreType>(nu_players) - 1);
     return score;
 }
 
@@ -782,9 +749,9 @@ inline const GridExt<bool>& Board::is_forbidden(Color c) const
 
 inline bool Board::is_forbidden(Color c, Move mv) const
 {
-    auto& info = get_move_info(mv);
-    auto i = info.begin();
-    auto end = info.end();
+    auto points = get_move_points(mv);
+    auto i = points.begin();
+    auto end = points.end();
     do
         if (m_state_color[c].forbidden[*i])
             return true;
@@ -797,25 +764,6 @@ inline bool Board::is_legal(Move mv) const
     return is_legal(m_state_base.to_play, mv);
 }
 
-inline bool Board::is_legal_nonfirst(Color c, Move mv) const
-{
-    auto& info = get_move_info(mv);
-    if (! is_piece_left(c, info.get_piece()))
-        return false;
-    bool has_attach_point = false;
-    auto i = info.begin();
-    auto end = info.end();
-    do
-    {
-        if (m_state_color[c].forbidden[*i])
-            return false;
-        if (is_attach_point(*i, c))
-            has_attach_point = true;
-    }
-    while (++i != end);
-    return has_attach_point;
-}
-
 inline bool Board::is_piece_left(Color c, Piece piece) const
 {
     LIBBOARDGAME_ASSERT(piece.to_int() < get_nu_uniq_pieces());
@@ -827,29 +775,35 @@ inline bool Board::is_same_player(Color c1, Color c2) const
     return c1 == c2 || c1 == m_second_color[c2];
 }
 
+template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH>
 inline void Board::place(Color c, Move mv)
 {
-    auto& info = get_move_info(mv);
-    auto& info_ext = get_move_info_ext(mv);
+    LIBBOARDGAME_ASSERT(m_max_piece_size == MAX_SIZE);
+    LIBBOARDGAME_ASSERT(m_max_adj_attach == MAX_ADJ_ATTACH);
+    auto& info = BoardConst::get_move_info<MAX_SIZE>(mv, m_move_info_array);
+    auto& info_ext = BoardConst::get_move_info_ext<MAX_ADJ_ATTACH>(
+                mv, m_move_info_ext_array);
     auto piece = info.get_piece();
-    auto piece_size = info.size();
     auto& state_color = m_state_color[c];
     LIBBOARDGAME_ASSERT(state_color.nu_left_piece[piece] > 0);
+    auto score_points = m_score_points[piece];
     if (--state_color.nu_left_piece[piece] == 0)
     {
         state_color.pieces_left.remove_fast(piece);
         if (state_color.pieces_left.empty())
         {
             state_color.points += m_bonus_all_pieces;
-            if (piece_size == 1)
+            if (MAX_SIZE == 7) // Nexos
+                LIBBOARDGAME_ASSERT(m_bonus_one_piece == 0);
+            else if (score_points == 1)
                 state_color.points += m_bonus_one_piece;
         }
     }
     ++m_state_base.nu_onboard_pieces_all;
     ++state_color.nu_onboard_pieces;
-    state_color.points += m_score_points[piece];
+    state_color.points += score_points;
     auto i = info.begin();
-    auto end = i + piece_size;
+    auto end = info.end();
     do
     {
         m_state_base.point_state[*i] = PointState(c);
@@ -858,11 +812,20 @@ inline void Board::place(Color c, Move mv)
         });
     }
     while (++i != end);
-    end = info_ext.end_adj();
-    for (i = info_ext.begin_adj(); i != end; ++i)
-        state_color.forbidden[*i] = true;
-    LIBBOARDGAME_ASSERT(i == info_ext.begin_attach());
-    end += info_ext.size_attach_points;
+    if (MAX_SIZE == 7) // Nexos
+    {
+        LIBBOARDGAME_ASSERT(info_ext.size_adj_points == 0);
+        i = info_ext.begin_attach();
+        end = i + info_ext.size_attach_points;
+    }
+    else
+    {
+        end = info_ext.end_adj();
+        for (i = info_ext.begin_adj(); i != end; ++i)
+            state_color.forbidden[*i] = true;
+        LIBBOARDGAME_ASSERT(i == info_ext.begin_attach());
+        end += info_ext.size_attach_points;
+    }
     auto& attach_points = m_attach_points[c];
     auto n = attach_points.size();
     do
@@ -876,18 +839,14 @@ inline void Board::place(Color c, Move mv)
     attach_points.resize(n);
 }
 
+template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH>
 inline void Board::play(Color c, Move mv)
 {
-    place(c, mv);
+    place<MAX_SIZE, MAX_ADJ_ATTACH>(c, mv);
     m_moves.push_back(ColorMove(c, mv));
     m_state_base.to_play = get_next(c);
 }
 
-inline void Board::play(Move mv)
-{
-    play(m_state_base.to_play, mv);
-}
-
 inline void Board::play(ColorMove mv)
 {
     play(mv.color, mv.move);
