@@ -7,20 +7,21 @@
 #ifndef LIBPENTOBI_BASE_BOARD_CONST_H
 #define LIBPENTOBI_BASE_BOARD_CONST_H
 
-#include <array>
 #include "Geometry.h"
-#include "Marker.h"
 #include "MoveInfo.h"
 #include "PieceInfo.h"
 #include "PieceTransforms.h"
 #include "PrecompMoves.h"
+#include "SymmetricPoints.h"
 #include "Variant.h"
 #include "libboardgame_util/ArrayList.h"
+#include "libboardgame_util/Range.h"
 
 namespace libpentobi_base {
 
 using namespace std;
 using libboardgame_util::ArrayList;
+using libboardgame_util::Range;
 
 //-----------------------------------------------------------------------------
 
@@ -34,13 +35,30 @@ public:
     ArrayList<Point, PrecompMoves::adj_status_nu_adj, unsigned short>
     AdjStatusList;
 
-    static const unsigned max_moves_at_point = 40;
+    /** Start of the MoveInfo array, which can be cached by the user in
+        performance-critical code and then passed into the static version of
+        get_move_info(). */
+    typedef const void* MoveInfoArray;
+
+    /** Start of the MoveInfoExt array, which can be cached by the user in
+        performance-critical code and then passed into the static version of
+        get_move_info_ext(). */
+    typedef const void* MoveInfoExtArray;
 
     /** Get the single instance for a given board size.
         The instance is created the first time this function is called.
         This function is not thread-safe. */
     static const BoardConst& get(Variant variant);
 
+    template<unsigned MAX_SIZE>
+    static const MoveInfo<MAX_SIZE>&
+    get_move_info(Move mv, MoveInfoArray move_info_array);
+
+    template<unsigned MAX_ADJ_ATTACH>
+    static const MoveInfoExt<MAX_ADJ_ATTACH>&
+    get_move_info_ext(Move mv, MoveInfoExtArray move_info_ext_array);
+
+
     Piece::IntType get_nu_pieces() const;
 
     const PieceInfo& get_piece_info(Piece piece) const;
@@ -51,23 +69,32 @@ public:
 
     const PieceTransforms& get_transforms() const;
 
-    /** Get move info.
-        @pre ! mv.is_null() */
-    const MoveInfo& get_move_info(Move mv) const;
+    unsigned get_max_piece_size() const { return m_max_piece_size; }
 
-    /** Get pointer to move info array.
-        Can be used to speed up the access to the move info by avoiding the
-        multiple pointer dereferencing of Board::get_move_info(Move) */
-    const MoveInfo* get_move_info_array() const;
+    unsigned get_max_adj_attach() const { return m_max_adj_attach; }
+
+    Range<const Point> get_move_points(Move mv) const;
+
+    /** Return start of move points array.
+        For unrolling loops, there are guaranteed to be as many elements
+        as the maximum piece size in the current game variant. If the piece
+        is smaller, the remaining points are guaranteed to be Point::null(). */
+    const Point* get_move_points_begin(Move mv) const;
+
+    template<unsigned MAX_SIZE>
+    const Point* get_move_points_begin(Move mv) const;
+
+    Piece get_move_piece(Move mv) const;
+
+    template<unsigned MAX_SIZE>
+    Piece get_move_piece(Move mv) const;
+
+    MoveInfoArray get_move_info_array() const { return m_move_info.get(); }
 
     /** Get pointer to extended move info array.
         Can be used to speed up the access to the move info by avoiding the
         multiple pointer dereferencing of Board::get_move_info_ext(Move) */
-    const MoveInfoExt* get_move_info_ext_array() const;
-
-    /** Get extended move info.
-        @pre ! mv.is_null() */
-    const MoveInfoExt& get_move_info_ext(Move mv) const;
+    MoveInfoExtArray get_move_info_ext_array() const;
 
     const MoveInfoExt2& get_move_info_ext_2(Move mv) const;
 
@@ -105,6 +132,13 @@ public:
         return m_adj_status_list[p];
     }
 
+    /** Only initialized in game variants with central symmetry of board
+        including startign points. */
+    const SymmetricPoints& get_symmetrc_points() const
+    {
+        return m_symmetric_points;
+    }
+
     /** Convert a move to its string representation.
         The string representation is a comma-separated list of points (without
         spaces between the commas or points). If with_piece_name is true,
@@ -119,27 +153,21 @@ public:
     void sort(MovePoints& points) const;
 
 private:
-    typedef ArrayList<Move, max_moves_at_point> LocalMovesList;
+    struct MallocFree
+    {
+        void operator()(void* x) { free(x); }
+    };
 
-    /** See m_full_move_table */
-    typedef array<PieceMap<Grid<LocalMovesList>>, PrecompMoves::nu_adj_status>
-        FullMoveTable;
-
-    /** Local variable used during construction.
-        Making this variable static slightly speeds up construction and
-        a thread-safe construction is not needed. */
-    static Marker s_marker;
-
-    /** Local variable used during construction.
-        See s_marker why this variable is static. */
-    static Grid<array<ArrayList<Point, PrecompMoves::adj_status_nu_adj>,
-                     PrecompMoves::nu_adj_status>>
-        s_adj_status;
 
     Piece::IntType m_nu_pieces;
 
     unsigned m_nu_moves;
 
+    unsigned m_max_piece_size;
+
+    /** See MoveInfoExt */
+    unsigned m_max_adj_attach;
+
     BoardType m_board_type;
 
     PieceSet m_piece_set;
@@ -154,17 +182,19 @@ private:
 
     PieceMap<unsigned> m_nu_attach_points{0};
 
-    unique_ptr<MoveInfo[]> m_move_info;
+    /** Array of MoveInfo<MAX_SIZE> with MAX_SIZE being the maximum piece size
+        in the corresponding game variant.
+        See comments at MoveInfo. */
+    unique_ptr<void, MallocFree> m_move_info;
 
-    unique_ptr<MoveInfoExt[]> m_move_info_ext;
+    /** Array of MoveInfoExt<MAX_ADJ_ATTACH> with MAX_ADJ_ATTACH being the
+        maximum total number of attach points and adjacent points of a piece in
+        the corresponding game variant.
+        See comments at MoveInfoExt. */
+    unique_ptr<void, MallocFree> m_move_info_ext;
 
     unique_ptr<MoveInfoExt2[]> m_move_info_ext_2;
 
-    /** Non-compact representation of lists of moves of a piece at a point
-        constrained by the forbidden status of adjacent points.
-        Only used during construction of m_moves_range and m_move_lists. */
-    unique_ptr<FullMoveTable> m_full_move_table;
-
     PrecompMoves m_precomp_moves;
 
     /** Value for comparing points using the ordering used in blksgf files.
@@ -173,22 +203,26 @@ private:
         for Point is that y goes downwards. */
     Grid<unsigned> m_compare_val;
 
+    SymmetricPoints m_symmetric_points;
+
+
     BoardConst(BoardType board_type, PieceSet piece_set);
 
+    template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH>
     void create_move(unsigned& moves_created, Piece piece,
                      const MovePoints& points, Point label_pos);
 
     void create_moves();
 
+    template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH>
     void create_moves(unsigned& moves_created, Piece piece);
 
+    template<unsigned MAX_SIZE>
+    const MoveInfo<MAX_SIZE>& get_move_info(Move mv) const;
+
     void init_adj_status();
 
-    void init_adj_status(
-                       Point p,
-                       array<bool, PrecompMoves::adj_status_nu_adj>& forbidden,
-                       unsigned i);
-
+    template<unsigned MAX_SIZE>
     void init_symmetry_info();
 };
 
@@ -197,21 +231,29 @@ inline const Geometry& BoardConst::get_geometry() const
     return m_geo;
 }
 
-inline const MoveInfo& BoardConst::get_move_info(Move mv) const
+template<unsigned MAX_SIZE>
+inline const MoveInfo<MAX_SIZE>&
+BoardConst::get_move_info(Move mv, MoveInfoArray move_info_array)
 {
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_nu_moves);
-    return m_move_info[mv.to_int()];
+    LIBBOARDGAME_ASSERT(! mv.is_null());
+    return *(static_cast<const MoveInfo<MAX_SIZE>*>(move_info_array)
+             + mv.to_int());
 }
 
-inline const MoveInfo* BoardConst::get_move_info_array() const
+template<unsigned MAX_SIZE>
+inline const MoveInfo<MAX_SIZE>& BoardConst::get_move_info(Move mv) const
 {
-    return m_move_info.get();
+    LIBBOARDGAME_ASSERT(m_max_piece_size == MAX_SIZE);
+    return get_move_info<MAX_SIZE>(mv, m_move_info.get());
 }
 
-inline const MoveInfoExt& BoardConst::get_move_info_ext(Move mv) const
+template<unsigned MAX_ADJ_ATTACH>
+inline const MoveInfoExt<MAX_ADJ_ATTACH>&
+BoardConst::get_move_info_ext(Move mv, MoveInfoExtArray move_info_ext_array)
 {
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_nu_moves);
-    return m_move_info_ext[mv.to_int()];
+    LIBBOARDGAME_ASSERT(! mv.is_null());
+    return *(static_cast<const MoveInfoExt<MAX_ADJ_ATTACH>*>(
+                 move_info_ext_array) + mv.to_int());
 }
 
 inline const MoveInfoExt2& BoardConst::get_move_info_ext_2(Move mv) const
@@ -220,7 +262,7 @@ inline const MoveInfoExt2& BoardConst::get_move_info_ext_2(Move mv) const
     return m_move_info_ext_2[mv.to_int()];
 }
 
-inline const MoveInfoExt* BoardConst::get_move_info_ext_array() const
+inline auto BoardConst::get_move_info_ext_array() const -> MoveInfoExtArray
 {
     return m_move_info_ext.get();
 }
@@ -230,6 +272,64 @@ inline const MoveInfoExt2* BoardConst::get_move_info_ext_2_array() const
     return m_move_info_ext_2.get();
 }
 
+template<unsigned MAX_SIZE>
+inline Piece BoardConst::get_move_piece(Move mv) const
+{
+    return get_move_info<MAX_SIZE>(mv).get_piece();
+}
+
+inline Piece BoardConst::get_move_piece(Move mv) const
+{
+    if (m_max_piece_size == 5)
+        return get_move_piece<5>(mv);
+    else if (m_max_piece_size == 6)
+        return get_move_piece<6>(mv);
+    else
+    {
+        LIBBOARDGAME_ASSERT(m_max_piece_size == 7);
+        return get_move_piece<7>(mv);
+    }
+}
+
+inline Range<const Point> BoardConst::get_move_points(Move mv) const
+{
+    if (m_max_piece_size == 5)
+    {
+        auto& info = get_move_info<5>(mv);
+        return Range<const Point>(info.begin(), info.end());
+    }
+    else if (m_max_piece_size == 6)
+    {
+        auto& info = get_move_info<6>(mv);
+        return Range<const Point>(info.begin(), info.end());
+    }
+    else
+    {
+        LIBBOARDGAME_ASSERT(m_max_piece_size == 7);
+        auto& info = get_move_info<7>(mv);
+        return Range<const Point>(info.begin(), info.end());
+    }
+}
+
+inline const Point* BoardConst::get_move_points_begin(Move mv) const
+{
+    if (m_max_piece_size == 5)
+        return get_move_points_begin<5>(mv);
+    else if (m_max_piece_size == 6)
+        return get_move_points_begin<6>(mv);
+    else
+    {
+        LIBBOARDGAME_ASSERT(m_max_piece_size == 7);
+        return get_move_points_begin<7>(mv);
+    }
+}
+
+template<unsigned MAX_SIZE>
+inline const Point* BoardConst::get_move_points_begin(Move mv) const
+{
+    return get_move_info<MAX_SIZE>(mv).begin();
+}
+
 inline unsigned BoardConst::get_nu_moves() const
 {
     return m_nu_moves;
