@@ -64,12 +64,11 @@ Move get_transformed(const Board& bd, Move mv,
                      const PointTransform<Point>& transform)
 {
     auto& geo = bd.get_geometry();
-    auto& info = bd.get_move_info(mv);
     MovePoints points;
-    for (auto p : info)
+    for (auto p : bd.get_move_points(mv))
         points.push_back(transform.get_transformed(p, geo));
     Move transformed_mv;
-    bd.find_move(points, info.get_piece(), transformed_mv);
+    bd.find_move(points, bd.get_move_piece(mv), transformed_mv);
     return transformed_mv;
 }
 
