@@ -30,6 +30,8 @@ Game::Game(Variant variant)
     init(variant);
 }
 
+Game::~Game() = default;
+
 void Game::add_setup(Color c, Move mv)
 {
     auto& node = m_tree.add_setup(*m_current, c, mv);
@@ -42,13 +44,34 @@ void Game::delete_all_variations()
     m_tree.delete_all_variations();
 }
 
+Color Game::get_to_play_default(const Game& game)
+{
+    auto& tree = game.get_tree();
+    auto& bd = game.get_board();
+    auto node = &game.get_current();
+    Color next = Color(0);
+    while (node)
+    {
+        auto mv = tree.get_move(*node);
+        if (! mv.is_null())
+        {
+            next = bd.get_next(mv.color);
+            break;
+        }
+        Color c;
+        if (libpentobi_base::node_util::get_player(*node, c))
+            return c;
+        node = node->get_parent_or_null();
+    }
+    return bd.get_effective_to_play(next);
+}
+
 void Game::goto_node(const SgfNode& node)
 {
     auto old = m_current;
     try
     {
         update(node);
-        m_current = &node;
     }
     catch (const InvalidTree&)
     {
@@ -64,7 +87,6 @@ void Game::goto_node(const SgfNode& node)
             catch (const InvalidTree&)
             {
             }
-            m_current = old;
         }
         throw;
     }
@@ -113,6 +135,7 @@ void Game::play(ColorMove mv, bool always_create_new_node)
         m_current = &m_tree.create_new_child(*m_current);
         m_tree.set_move(*m_current, mv);
     }
+    set_to_play(get_to_play_default(*this));
 }
 
 void Game::remove_player()
@@ -139,6 +162,11 @@ void Game::set_result(int score)
         m_tree.set_result(m_tree.get_root(), score);
 }
 
+void Game::set_to_play(Color c)
+{
+    m_bd->set_to_play(c);
+}
+
 void Game::truncate()
 {
     goto_node(m_tree.truncate(*m_current));
@@ -154,6 +182,8 @@ void Game::undo()
 void Game::update(const SgfNode& node)
 {
     m_updater.update(*m_bd, m_tree, node);
+    m_current = &node;
+    set_to_play(get_to_play_default(*this));
 }
 
 //-----------------------------------------------------------------------------
