@@ -14,7 +14,8 @@ namespace libpentobi_base {
 
 //-----------------------------------------------------------------------------
 
-typedef libboardgame_util::ArrayList<Move, Move::range> MoveList;
+/** List that can hold all possible moves, not including Move::null() */
+typedef libboardgame_util::ArrayList<Move, Move::range - 1> MoveList;
 
 //-----------------------------------------------------------------------------
 
