@@ -10,10 +10,10 @@
 
 #include "Variant.h"
 
+#include "CallistoGeometry.h"
 #include "NexosGeometry.h"
 #include "TrigonGeometry.h"
 #include "libboardgame_base/RectGeometry.h"
-#include "libboardgame_util/Assert.h"
 #include "libboardgame_util/StringUtil.h"
 
 namespace libpentobi_base {
@@ -21,7 +21,10 @@ namespace libpentobi_base {
 using libboardgame_base::PointTransfIdent;
 using libboardgame_base::PointTransfRefl;
 using libboardgame_base::PointTransfReflRot180;
+using libboardgame_base::PointTransfRot90;
 using libboardgame_base::PointTransfRot180;
+using libboardgame_base::PointTransfRot270;
+using libboardgame_base::PointTransfRot90Refl;
 using libboardgame_base::PointTransfRot270Refl;
 using libboardgame_base::PointTransfTrigonReflRot60;
 using libboardgame_base::PointTransfTrigonReflRot120;
@@ -62,6 +65,15 @@ BoardType get_board_type(Variant variant)
     case Variant::nexos_2:
         result = BoardType::nexos;
         break;
+    case Variant::callisto:
+        result = BoardType::callisto;
+        break;
+    case Variant::callisto_2:
+        result = BoardType::callisto_2;
+        break;
+    case Variant::callisto_3:
+        result = BoardType::callisto_3;
+        break;
     }
     return result;
 }
@@ -86,6 +98,15 @@ const Geometry& get_geometry(BoardType board_type)
     case BoardType::nexos:
         result = &NexosGeometry::get(13);
         break;
+    case BoardType::callisto:
+        result = &CallistoGeometry::get(4);
+        break;
+    case BoardType::callisto_2:
+        result = &CallistoGeometry::get(2);
+        break;
+    case BoardType::callisto_3:
+        result = &CallistoGeometry::get(3);
+        break;
     }
     return *result;
 }
@@ -102,9 +123,11 @@ Color::IntType get_nu_colors(Variant variant)
     {
     case Variant::duo:
     case Variant::junior:
+    case Variant::callisto_2:
         result = 2;
         break;
     case Variant::trigon_3:
+    case Variant::callisto_3:
         result = 3;
         break;
     case Variant::classic:
@@ -114,6 +137,7 @@ Color::IntType get_nu_colors(Variant variant)
     case Variant::trigon_2:
     case Variant::nexos:
     case Variant::nexos_2:
+    case Variant::callisto:
         result = 4;
         break;
     }
@@ -130,15 +154,18 @@ Color::IntType get_nu_players(Variant variant)
     case Variant::classic_2:
     case Variant::trigon_2:
     case Variant::nexos_2:
+    case Variant::callisto_2:
         result = 2;
         break;
     case Variant::classic_3:
     case Variant::trigon_3:
+    case Variant::callisto_3:
         result = 3;
         break;
     case Variant::classic:
     case Variant::trigon:
     case Variant::nexos:
+    case Variant::callisto:
         result = 4;
         break;
     }
@@ -168,6 +195,11 @@ PieceSet get_piece_set(Variant variant)
     case Variant::nexos_2:
         result = PieceSet::nexos;
         break;
+    case Variant::callisto:
+    case Variant::callisto_2:
+    case Variant::callisto_3:
+        result = PieceSet::callisto;
+        break;
     }
     return result;
 }
@@ -210,6 +242,24 @@ void get_transforms(Variant variant,
         transforms.emplace_back(new PointTransfTrigonReflRot300<Point>);
         inv_transforms.emplace_back(new PointTransfTrigonReflRot300<Point>);
         break;
+    case BoardType::callisto_2:
+    case BoardType::callisto:
+    case BoardType::callisto_3:
+        transforms.emplace_back(new PointTransfRot90<Point>);
+        inv_transforms.emplace_back(new PointTransfRot270<Point>);
+        transforms.emplace_back(new PointTransfRot180<Point>);
+        inv_transforms.emplace_back(new PointTransfRot180<Point>);
+        transforms.emplace_back(new PointTransfRot270<Point>);
+        inv_transforms.emplace_back(new PointTransfRot90<Point>);
+        transforms.emplace_back(new PointTransfRefl<Point>);
+        inv_transforms.emplace_back(new PointTransfRefl<Point>);
+        transforms.emplace_back(new PointTransfReflRot180<Point>);
+        inv_transforms.emplace_back(new PointTransfReflRot180<Point>);
+        transforms.emplace_back(new PointTransfRot90Refl<Point>);
+        inv_transforms.emplace_back(new PointTransfRot90Refl<Point>);
+        transforms.emplace_back(new PointTransfRot270Refl<Point>);
+        inv_transforms.emplace_back(new PointTransfRot270Refl<Point>);
+        break;
     case BoardType::classic:
     case BoardType::trigon_3:
     case BoardType::nexos:
@@ -217,6 +267,12 @@ void get_transforms(Variant variant,
     }
 }
 
+bool has_central_symmetry(Variant variant)
+{
+    return variant == Variant::duo || variant == Variant::junior
+            || variant == Variant::trigon_2 || variant == Variant::callisto_2;
+}
+
 bool parse_variant(const string& s, Variant& variant)
 {
     string t = to_lower(trim(s));
@@ -240,6 +296,12 @@ bool parse_variant(const string& s, Variant& variant)
         variant = Variant::nexos;
     else if (t == "nexos two-player")
         variant = Variant::nexos_2;
+    else if (t == "callisto")
+        variant = Variant::callisto;
+    else if (t == "callisto two-player")
+        variant = Variant::callisto_2;
+    else if (t == "callisto three-player")
+        variant = Variant::callisto_3;
     else
         return false;
     return true;
@@ -268,6 +330,12 @@ bool parse_variant_id(const string& s, Variant& variant)
         variant = Variant::nexos;
     else if (t == "nexos_2" || t == "n2")
         variant = Variant::nexos_2;
+    else if (t == "callisto" || t == "ca")
+        variant = Variant::callisto;
+    else if (t == "callisto_2" || t == "ca2")
+        variant = Variant::callisto_2;
+    else if (t == "callisto_3" || t == "ca3")
+        variant = Variant::callisto_3;
     else
         return false;
     return true;
@@ -307,6 +375,16 @@ const char* to_string(Variant variant)
         break;
     case Variant::nexos_2:
         result = "Nexos Two-Player";
+        break;
+    case Variant::callisto:
+        result = "Callisto";
+        break;
+    case Variant::callisto_2:
+        result = "Callisto Two-Player";
+        break;
+    case Variant::callisto_3:
+        result = "Callisto Three-Player";
+        break;
     }
     return result;
 }
@@ -346,6 +424,15 @@ const char* to_string_id(Variant variant)
     case Variant::nexos_2:
         result = "nexos_2";
         break;
+    case Variant::callisto:
+        result = "callisto";
+        break;
+    case Variant::callisto_2:
+        result = "callisto_2";
+        break;
+    case Variant::callisto_3:
+        result = "callisto_3";
+        break;
     }
     return result;
 }
