@@ -9,6 +9,7 @@
 #endif
 
 #include "BoardPainter.h"
+#include "libpentobi_base/CallistoGeometry.h"
 
 #include <algorithm>
 #include <cmath>
@@ -17,7 +18,9 @@
 using namespace std;
 using libboardgame_util::ArrayList;
 using libpentobi_base::BoardType;
+using libpentobi_base::CallistoGeometry;
 using libpentobi_base::Move;
+using libpentobi_base::PieceSet;
 using libpentobi_base::PointState;
 
 //-----------------------------------------------------------------------------
@@ -35,10 +38,7 @@ BoardPainter::BoardPainter()
     m_fontCoordLabels.setStretch(QFont::SemiCondensed);
 }
 
-BoardPainter::~BoardPainter()
-{
-    // Same as default destructor but non-inline
-}
+BoardPainter::~BoardPainter() = default;
 
 CoordPoint BoardPainter::getCoordPoint(int x, int y)
 {
@@ -100,13 +100,13 @@ void BoardPainter::paintEmptyBoard(QPainter& painter, unsigned width,
     m_hasPainted = true;
     painter.setRenderHint(QPainter::Antialiasing, true);
     m_variant = variant;
-    auto boardType = get_board_type(variant);
+    auto pieceSet = get_piece_set(variant);
     m_geo = &geo;
     m_width = static_cast<int>(m_geo->get_width());
     m_height = static_cast<int>(m_geo->get_height());
-    m_isTrigon = (boardType == BoardType::trigon
-                  || boardType == BoardType::trigon_3);
-    m_isNexos = (boardType == BoardType::nexos);
+    m_isTrigon = (pieceSet == PieceSet::trigon);
+    m_isNexos = (pieceSet == PieceSet::nexos);
+    m_isCallisto = (pieceSet == PieceSet::callisto);
     qreal ratio;
     if (m_isTrigon)
     {
@@ -136,7 +136,10 @@ void BoardPainter::paintEmptyBoard(QPainter& painter, unsigned width,
     m_fieldHeight = ratio * m_fieldWidth;
     m_boardOffset = QPointF(0.5 * (width - m_fieldWidth * m_width),
                             0.5 * (height - m_fieldHeight * m_height));
-    int fontSize = static_cast<int>((m_isTrigon ? 0.7 : 0.5) * m_fieldWidth);
+    // QFont::setPixelSize(0) prints a warning even if it works and the docs
+    // of Qt 5.3 don't forbid it (unlike QFont::setPointSize(0)).
+    int fontSize =
+            max(1, static_cast<int>((m_isTrigon ? 0.7 : 0.5) * m_fieldWidth));
     m_font.setPixelSize(fontSize);
     m_fontSemiCondensed.setPixelSize(fontSize);
     m_fontCondensed.setPixelSize(fontSize);
@@ -145,46 +148,32 @@ void BoardPainter::paintEmptyBoard(QPainter& painter, unsigned width,
     painter.translate(m_boardOffset);
     if (m_coordinates)
         paintCoordinates(painter);
-    m_startingPoints.init(variant, *m_geo);
     if (m_isNexos)
         painter.fillRect(QRectF(m_fieldWidth / 4, m_fieldHeight / 4,
                                 m_width * m_fieldWidth - m_fieldWidth / 2,
                                 m_height * m_fieldHeight - m_fieldHeight / 2),
                          QColor(174, 167, 172));
+    auto nu_players = get_nu_players(m_variant);
     for (Point p : *m_geo)
     {
         int x = m_geo->get_x(p);
         int y = m_geo->get_y(p);
         qreal fieldX = x * m_fieldWidth;
         qreal fieldY = y * m_fieldHeight;
-        auto pointType = m_geo->get_point_type(x, y);
+        auto pointType = m_geo->get_point_type(p);
         if (m_isTrigon)
         {
             bool isUpward = (pointType == 0);
-            if (m_startingPoints.is_colorless_starting_point(p))
-                Util::paintEmptyTriangleStartingPoint(painter, isUpward,
-                                                      fieldX, fieldY,
-                                                      m_fieldWidth,
-                                                      m_fieldHeight);
-            else
-                Util::paintEmptyTriangle(painter, isUpward, fieldX, fieldY,
-                                         m_fieldWidth, m_fieldHeight);
+            Util::paintEmptyTriangle(painter, isUpward, fieldX, fieldY,
+                                     m_fieldWidth, m_fieldHeight);
         }
         else if (m_isNexos)
         {
             if (pointType == 1 || pointType == 2)
             {
                 bool isHorizontal = (pointType == 1);
-                if (m_startingPoints.is_colored_starting_point(p))
-                {
-                    Color c = m_startingPoints.get_starting_point_color(p);
-                    Util::paintEmptySegmentStartingPoint(painter, variant, c,
-                                                         isHorizontal, fieldX,
-                                                         fieldY, m_fieldWidth);
-                }
-                else
-                    Util::paintEmptySegment(painter, isHorizontal, fieldX,
-                                            fieldY, m_fieldWidth);
+                Util::paintEmptySegment(painter, isHorizontal, fieldX, fieldY,
+                                        m_fieldWidth);
             }
             else
             {
@@ -193,18 +182,15 @@ void BoardPainter::paintEmptyBoard(QPainter& painter, unsigned width,
                                          m_fieldWidth);
             }
         }
+        else if (m_isCallisto
+                 && CallistoGeometry::is_center_section(x, y, nu_players))
+            Util::paintEmptySquareCallistoCenter(painter, fieldX, fieldY,
+                                                 m_fieldWidth);
+        else if (m_isCallisto)
+            Util::paintEmptySquareCallisto(painter, fieldX, fieldY,
+                                           m_fieldWidth);
         else
-        {
-            if (m_startingPoints.is_colored_starting_point(p))
-            {
-                Color c = m_startingPoints.get_starting_point_color(p);
-                Util::paintEmptySquareStartingPoint(painter, variant, c,
-                                                    fieldX, fieldY,
-                                                    m_fieldWidth);
-            }
-            else
-                Util::paintEmptySquare(painter, fieldX, fieldY, m_fieldWidth);
-        }
+            Util::paintEmptySquare(painter, fieldX, fieldY, m_fieldWidth);
     }
     painter.restore();
 }
@@ -365,6 +351,8 @@ void BoardPainter::paintMarks(QPainter& painter,
                     y -= 0.167 * m_fieldHeight;
                 size = 0.1 * m_fieldHeight;
             }
+            else if (m_isCallisto)
+                size = 0.1 * m_fieldHeight;
             else
                 size = 0.12 * m_fieldHeight;
             QColor color = Util::getMarkColor(variant, pointState[p]);
@@ -397,6 +385,7 @@ void BoardPainter::paintPieces(QPainter& painter,
     painter.setRenderHint(QPainter::Antialiasing, true);
     painter.save();
     painter.translate(m_boardOffset);
+    ColorMap<bool> isFirstPiece(true);
     for (Point p : *m_geo)
     {
         int x = m_geo->get_x(p);
@@ -404,15 +393,16 @@ void BoardPainter::paintPieces(QPainter& painter,
         PointState s = pointState[p];
         qreal fieldX = x * m_fieldWidth;
         qreal fieldY = y * m_fieldHeight;
-        auto pointType = m_geo->get_point_type(x, y);
+        auto pointType = m_geo->get_point_type(p);
         if (m_isTrigon)
         {
             if (s.is_empty())
                 continue;
+            Color c = s.to_color();
+            isFirstPiece[c] = false;
             bool isUpward = (pointType == 0);
-            Util::paintColorTriangle(painter, m_variant, s.to_color(),
-                                     isUpward, fieldX, fieldY, m_fieldWidth,
-                                     m_fieldHeight);
+            Util::paintColorTriangle(painter, m_variant, c, isUpward, fieldX,
+                                     fieldY, m_fieldWidth, m_fieldHeight);
         }
         else if (m_isNexos)
         {
@@ -420,10 +410,11 @@ void BoardPainter::paintPieces(QPainter& painter,
             {
                 if (s.is_empty())
                     continue;
+                Color c = s.to_color();
+                isFirstPiece[c] = false;
                 bool isHorizontal = (pointType == 1);
-                Util::paintColorSegment(painter, m_variant, s.to_color(),
-                                        isHorizontal, fieldX, fieldY,
-                                        m_fieldWidth);
+                Util::paintColorSegment(painter, m_variant, c, isHorizontal,
+                                        fieldX, fieldY, m_fieldWidth);
             }
             else
             {
@@ -436,10 +427,34 @@ void BoardPainter::paintPieces(QPainter& painter,
         {
             if (s.is_empty())
                 continue;
-            Util::paintColorSquare(painter, m_variant, s.to_color(), fieldX,
-                                   fieldY, m_fieldWidth);
+            Color c = s.to_color();
+            isFirstPiece[c] = false;
+            if (m_isCallisto)
+            {
+                bool hasLeft =
+                        (x > 0 && m_geo->is_onboard(x - 1, y)
+                         && pieceId[p] == pieceId[m_geo->get_point(x - 1, y)]);
+                bool hasRight =
+                        (x < m_width - 1 && m_geo->is_onboard(x + 1, y)
+                         && pieceId[p] == pieceId[m_geo->get_point(x + 1, y)]);
+                bool hasUp =
+                        (y > 0 && m_geo->is_onboard(x, y - 1)
+                         && pieceId[p] == pieceId[m_geo->get_point(x, y - 1)]);
+                bool hasDown =
+                        (y < m_height - 1 && m_geo->is_onboard(x, y + 1)
+                         && pieceId[p] == pieceId[m_geo->get_point(x, y + 1)]);
+                bool isOnePiece =
+                        (! hasLeft && ! hasRight && ! hasUp && ! hasDown);
+                Util::paintColorSquareCallisto(painter, m_variant, c, fieldX,
+                                               fieldY, m_fieldWidth, hasRight,
+                                               hasDown, isOnePiece);
+            }
+            else
+                Util::paintColorSquare(painter, m_variant, c, fieldX, fieldY,
+                                       m_fieldWidth);
         }
     }
+    paintStartingPoints(painter, m_variant, pointState, isFirstPiece);
     if (marks)
         paintMarks(painter, pointState, m_variant, *marks);
     if (labels)
@@ -509,6 +524,18 @@ void BoardPainter::paintSelectedPiece(QPainter& painter, Color c,
                 }
             }
         }
+        else if (m_isCallisto)
+        {
+            bool hasRight = (m_geo->is_onboard(CoordPoint(x + 1, y))
+                             && points.contains(m_geo->get_point(x + 1, y)));
+            bool hasDown = (m_geo->is_onboard(CoordPoint(x, y + 1))
+                            && points.contains(m_geo->get_point(x, y + 1)));
+            bool isOnePiece = (points.size() == 1);
+            Util::paintColorSquareCallisto(painter, m_variant, c, fieldX,
+                                           fieldY, m_fieldWidth, hasRight,
+                                           hasDown, isOnePiece, alpha,
+                                           saturation, flat);
+        }
         else
             Util::paintColorSquare(painter, m_variant, c, fieldX, fieldY,
                                    m_fieldWidth, alpha, saturation, flat);
@@ -534,4 +561,60 @@ void BoardPainter::paintSelectedPiece(QPainter& painter, Color c,
     painter.restore();
 }
 
+void BoardPainter::paintStartingPoints(QPainter& painter, Variant variant,
+                                       const Grid<PointState>& pointState,
+                                       const ColorMap<bool>& isFirstPiece)
+{
+    m_startingPoints.init(variant, *m_geo);
+    auto colors = get_colors(variant);
+    if (m_isTrigon)
+    {
+        bool isFirstPieceAny = false;
+        for (Color c : colors)
+            if (isFirstPiece[c])
+            {
+                isFirstPieceAny = true;
+                break;
+            }
+        if (! isFirstPieceAny)
+            return;
+        for (Point p : m_startingPoints.get_starting_points(Color(0)))
+        {
+            if (! pointState[p].is_empty())
+                continue;
+            int x = m_geo->get_x(p);
+            int y = m_geo->get_y(p);
+            qreal fieldX = x * m_fieldWidth;
+            qreal fieldY = y * m_fieldHeight;
+            bool isUpward = (m_geo->get_point_type(p) == 0);
+            Util::paintTriangleStartingPoint(painter, isUpward, fieldX, fieldY,
+                                             m_fieldWidth, m_fieldHeight);
+        }
+    }
+    else
+    {
+        for (Color c : colors)
+        {
+            if (! isFirstPiece[c])
+                continue;
+            for (Point p : m_startingPoints.get_starting_points(c))
+            {
+                if (! pointState[p].is_empty())
+                    continue;
+                int x = m_geo->get_x(p);
+                int y = m_geo->get_y(p);
+                qreal fieldX = x * m_fieldWidth;
+                qreal fieldY = y * m_fieldHeight;
+                if (m_isNexos)
+                    Util::paintSegmentStartingPoint(painter, variant, c,
+                                                    fieldX, fieldY,
+                                                    m_fieldWidth);
+                else
+                    Util::paintSquareStartingPoint(painter, variant, c, fieldX,
+                                                   fieldY, m_fieldWidth);
+            }
+        }
+    }
+}
+
 //-----------------------------------------------------------------------------
