@@ -10,7 +10,9 @@
 
 #include "HelpWindow.h"
 
+#include <QApplication>
 #include <QAction>
+#include <QDesktopWidget>
 #include <QFile>
 #include <QLocale>
 #include <QSettings>
@@ -41,7 +43,7 @@ HelpWindow::HelpWindow(QWidget* parent, const QString& title,
     if (QIcon::hasThemeIcon("help-browser"))
         setWindowIcon(QIcon::fromTheme("help-browser"));
     m_mainPageUrl = QUrl::fromLocalFile(mainPage);
-    auto browser = new QTextBrowser(this);
+    auto browser = new QTextBrowser;
     setCentralWidget(browser);
     browser->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
     browser->setSource(m_mainPageUrl);
@@ -70,7 +72,7 @@ HelpWindow::HelpWindow(QWidget* parent, const QString& title,
     actionClose->setShortcut(QKeySequence::Close);
     connect(actionClose, SIGNAL(triggered()), SLOT(hide()));
     addAction(actionClose);
-    auto toolBar = new QToolBar(this);
+    auto toolBar = new QToolBar;
     toolBar->setMovable(false);
     toolBar->setToolButtonStyle(Qt::ToolButtonFollowStyle);
     toolBar->addAction(actionBack);
@@ -85,15 +87,14 @@ HelpWindow::HelpWindow(QWidget* parent, const QString& title,
 QString HelpWindow::findMainPage(QString helpDir, QString appName)
 {
     auto locale = QLocale::system().name();
-    auto path = QString("%1/%2/%3/index.html")
-            .arg(helpDir).arg(locale).arg(appName);
+    auto path = QString("%1/%2/%3/index.html").arg(helpDir, locale, appName);
     if (QFile(path).exists())
         return path;
     path = QString("%1/%2/%3/index.html")
-            .arg(helpDir).arg(locale.split("_")[0]).arg(appName);
+            .arg(helpDir, locale.split("_")[0], appName);
     if (QFile(path).exists())
         return path;
-    return QString("%1/C/%3/index.html").arg(helpDir).arg(appName);
+    return QString("%1/C/%3/index.html").arg(helpDir, appName);
 }
 
 void HelpWindow::closeEvent(QCloseEvent* event)
@@ -110,7 +111,8 @@ void HelpWindow::handleSourceChanged(const QUrl& src)
 
 QSize HelpWindow::sizeHint() const
 {
-    return QSize(600, 800);
+    auto geo = QApplication::desktop()->screenGeometry();
+    return QSize(geo.width() * 4 / 10, geo.height() * 9 / 10);
 }
 
 //-----------------------------------------------------------------------------
