@@ -14,9 +14,8 @@
 
 //-----------------------------------------------------------------------------
 
-LineEdit::LineEdit(QWidget* parent, int nuCharactersHint)
-    : QLineEdit(parent),
-      m_nuCharactersHint(nuCharactersHint)
+LineEdit::LineEdit(int nuCharactersHint)
+    : m_nuCharactersHint(nuCharactersHint)
 {
 }
 
