@@ -12,10 +12,12 @@
 #include <config.h>
 #endif
 
+#include <string>
 #include <QWidget>
 #include "libpentobi_base/Board.h"
 #include "libpentobi_base/Color.h"
 
+using namespace std;
 using libboardgame_base::Transform;
 using libboardgame_util::ArrayList;
 using libpentobi_base::Color;
@@ -38,8 +40,6 @@ public:
         game variant. */
     void init();
 
-    int heightForWidth(int width) const override;
-
     /** Call update() if pieces left have changed since last paint. */
     void checkUpdate();
 
@@ -78,6 +78,8 @@ private:
 
     qreal m_selectorHeight;
 
+    string m_pieceLayout;
+
 
     void filterCrossableJunctions(PiecePoints& points) const;
 
