@@ -4,44 +4,34 @@
 <context>
     <name>ComputerColorDialog</name>
     <message>
-        <location filename="../ComputerColorDialog.cpp" line="28"/>
         <source>Computer Colors</source>
         <translation>Computer-Farben</translation>
     </message>
     <message>
-        <location filename="../ComputerColorDialog.cpp" line="32"/>
-        <source>Colors played by the computer:</source>
-        <translation>Vom Computer gespielte Farben:</translation>
+        <source>Computer plays:</source>
+        <translation>Computer spielt:</translation>
     </message>
     <message>
-        <location filename="../ComputerColorDialog.cpp" line="74"/>
-        <location filename="../ComputerColorDialog.cpp" line="78"/>
         <source>&amp;Blue</source>
         <translation>&amp;Blau</translation>
     </message>
     <message>
-        <location filename="../ComputerColorDialog.cpp" line="74"/>
-        <location filename="../ComputerColorDialog.cpp" line="83"/>
         <source>&amp;Green</source>
         <translation>&amp;Grün</translation>
     </message>
     <message>
-        <location filename="../ComputerColorDialog.cpp" line="80"/>
         <source>&amp;Yellow</source>
         <translation>G&amp;elb</translation>
     </message>
     <message>
-        <location filename="../ComputerColorDialog.cpp" line="82"/>
         <source>&amp;Red</source>
         <translation>&amp;Rot</translation>
     </message>
     <message>
-        <location filename="../ComputerColorDialog.cpp" line="76"/>
         <source>&amp;Blue/Red</source>
         <translation>&amp;Blau/Rot</translation>
     </message>
     <message>
-        <location filename="../ComputerColorDialog.cpp" line="76"/>
         <source>&amp;Yellow/Green</source>
         <translation>&amp;Gelb/Grün</translation>
     </message>
@@ -49,69 +39,53 @@
 <context>
     <name>GameInfoDialog</name>
     <message>
-        <location filename="../GameInfoDialog.cpp" line="26"/>
         <source>Game Info</source>
         <translation>Spielinformation</translation>
     </message>
     <message>
-        <location filename="../GameInfoDialog.cpp" line="35"/>
-        <location filename="../GameInfoDialog.cpp" line="40"/>
-        <location filename="../GameInfoDialog.cpp" line="47"/>
         <source>Player &amp;Blue:</source>
         <oldsource>Player Blue:</oldsource>
         <translation>Spieler &amp;Blau:</translation>
     </message>
     <message>
-        <location filename="../GameInfoDialog.cpp" line="36"/>
-        <location filename="../GameInfoDialog.cpp" line="43"/>
         <source>Player &amp;Green:</source>
         <oldsource>Player Green:</oldsource>
         <translation>Spieler &amp;Grün:</translation>
     </message>
     <message>
-        <location filename="../GameInfoDialog.cpp" line="41"/>
-        <location filename="../GameInfoDialog.cpp" line="48"/>
         <source>Player &amp;Yellow:</source>
         <oldsource>Player Yellow:</oldsource>
         <translation>Spieler G&amp;elb:</translation>
     </message>
     <message>
-        <location filename="../GameInfoDialog.cpp" line="42"/>
-        <location filename="../GameInfoDialog.cpp" line="49"/>
         <source>Player &amp;Red:</source>
         <oldsource>Player Red:</oldsource>
         <translation>Spieler &amp;Rot:</translation>
     </message>
     <message>
-        <location filename="../GameInfoDialog.cpp" line="55"/>
         <source>Player &amp;Blue/Red:</source>
         <oldsource>Player Blue/Red:</oldsource>
         <translation>Spieler &amp;Blau/Rot:</translation>
     </message>
     <message>
-        <location filename="../GameInfoDialog.cpp" line="57"/>
         <source>Player &amp;Yellow/Green:</source>
         <oldsource>Player Yellow/Green:</oldsource>
         <translation>Spieler &amp;Gelb/Grün:</translation>
     </message>
     <message>
-        <location filename="../GameInfoDialog.cpp" line="59"/>
         <source>&amp;Date:</source>
         <oldsource>Date:</oldsource>
         <translation>&amp;Datum:</translation>
     </message>
     <message>
-        <location filename="../GameInfoDialog.cpp" line="60"/>
         <source>&amp;Time limits:</source>
         <translation>Bedenk&amp;zeit:</translation>
     </message>
     <message>
-        <location filename="../GameInfoDialog.cpp" line="61"/>
         <source>&amp;Event:</source>
         <translation>&amp;Veranstaltung:</translation>
     </message>
     <message>
-        <location filename="../GameInfoDialog.cpp" line="62"/>
         <source>R&amp;ound:</source>
         <translation>R&amp;unde:</translation>
     </message>
@@ -119,37 +93,26 @@
 <context>
     <name>HelpWindow</name>
     <message>
-        <location filename="../HelpWindow.cpp" line="40"/>
-        <source>Pentobi User Manual</source>
-        <translation>Pentobi-Benutzerhandbuch</translation>
-    </message>
-    <message>
-        <location filename="../HelpWindow.cpp" line="48"/>
         <source>Back</source>
         <translation>Zurück</translation>
     </message>
     <message>
-        <location filename="../HelpWindow.cpp" line="49"/>
         <source>Show previous page in history</source>
         <translation>Die vorherige Seite in der Chronik anzeigen</translation>
     </message>
     <message>
-        <location filename="../HelpWindow.cpp" line="55"/>
         <source>Forward</source>
         <translation>Vorwärts</translation>
     </message>
     <message>
-        <location filename="../HelpWindow.cpp" line="56"/>
         <source>Show next page in history</source>
         <translation>Die nächste Seite in der Chronik anzeigen</translation>
     </message>
     <message>
-        <location filename="../HelpWindow.cpp" line="62"/>
         <source>Contents</source>
         <translation>Inhalt</translation>
     </message>
     <message>
-        <location filename="../HelpWindow.cpp" line="63"/>
         <source>Show table of contents</source>
         <translation>Das Inhaltsverzeichnis anzeigen</translation>
     </message>
@@ -157,27 +120,22 @@
 <context>
     <name>InitialRatingDialog</name>
     <message>
-        <location filename="../InitialRatingDialog.cpp" line="23"/>
         <source>Initial Rating</source>
         <translation>Anfangswertung</translation>
     </message>
     <message>
-        <location filename="../InitialRatingDialog.cpp" line="28"/>
         <source>You have not yet played rated games in this game variant. Estimate your playing strength to initialize your rating.</source>
         <translation>Sie haben noch keine gewerteten Spiele in dieser Spielvariante gespielt. Schätzen Sie Ihre Spielstärke, um Ihre Wertung zu initialisieren.</translation>
     </message>
     <message>
-        <location filename="../InitialRatingDialog.cpp" line="35"/>
         <source>Beginner</source>
         <translation>Anfänger</translation>
     </message>
     <message>
-        <location filename="../InitialRatingDialog.cpp" line="42"/>
         <source>Expert</source>
         <translation>Experte</translation>
     </message>
     <message>
-        <location filename="../InitialRatingDialog.cpp" line="57"/>
         <source>Your initial rating: %1</source>
         <translation>Ihre Anfangswertung: %1</translation>
     </message>
@@ -185,34 +143,26 @@
 <context>
     <name>Util</name>
     <message>
-        <location filename="../Util.cpp" line="222"/>
-        <location filename="../Util.cpp" line="228"/>
         <source>Blue</source>
         <translation>Blau</translation>
     </message>
     <message>
-        <location filename="../Util.cpp" line="223"/>
-        <location filename="../Util.cpp" line="233"/>
         <source>Green</source>
         <translation>Grün</translation>
     </message>
     <message>
-        <location filename="../Util.cpp" line="230"/>
         <source>Yellow</source>
         <translation>Gelb</translation>
     </message>
     <message>
-        <location filename="../Util.cpp" line="232"/>
         <source>Red</source>
         <translation>Rot</translation>
     </message>
     <message>
-        <location filename="../Util.cpp" line="225"/>
         <source>Blue/Red</source>
         <translation>Blau/Rot</translation>
     </message>
     <message>
-        <location filename="../Util.cpp" line="226"/>
         <source>Yellow/Green</source>
         <translation>Gelb/Grün</translation>
     </message>
