@@ -16,6 +16,7 @@
 
 namespace libpentobi_mcts {
 
+using libboardgame_sgf::InvalidTree;
 using libboardgame_sgf::SgfNode;
 using libboardgame_util::clear_abort;
 using libboardgame_util::get_abort;
@@ -36,11 +37,19 @@ void AnalyzeGame::run(const Game& game, Search& search, size_t nu_simulations,
     auto& root = game.get_root();
     auto node = &root;
     unsigned total_moves = 0;
-    while (node)
+    try {
+        while (node)
+        {
+            if (tree.has_move(*node))
+                ++total_moves;
+            node = node->get_first_child_or_null();
+        }
+    }
+    catch (const InvalidTree&)
     {
-        if (tree.has_move(*node))
-            ++total_moves;
-        node = node->get_first_child_or_null();
+        // PentobiTree::has_move() can throw on invalid SGF tree read from
+        // external file. We simply abort the analysis.
+        return;
     }
     WallTimeSource time_source;
     clear_abort();
@@ -81,7 +90,7 @@ void AnalyzeGame::run(const Game& game, Search& search, size_t nu_simulations,
                     m_moves.push_back(mv);
                     m_values.push_back(search.get_root_val().get_mean());
                 }
-                catch (const runtime_error&)
+                catch (const InvalidTree&)
                 {
                     // BoardUpdater::update() can throw on invalid SGF tree
                     // read from external file. We simply abort the analysis.
