@@ -27,30 +27,40 @@ using libpentobi_base::BoardType;
 namespace {
 
 // Rationale for choosing the number of simulations:
-// * The number at level 1 is very small in Classic/Duo to avoid that
-//   level 1 is too strong for absolute beginners. (Searches with such a
-//   small number of simulations still produce reasonable moves because
-//   of the prior initialization of node values.) In Trigon, it starts
-//   with a higher number because the playing strength is weaker there.
-// * The number at the highest level is chosen such that the average
-//   time per game and player is 120s in Duo, 240s in Classic, 300s
-//   in Trigon, 330s in Nexos on an Intel i3-4130.
-// * The numbers for other levels are chosen such that they roughly
-//   correspond to equal Elo differences in self-play experiments.
-// * [Note that the numbers will be multiplied with a weight factor in
-//    Player::genmove() that depends on the move number]
+// * Level 9, the highest in the desktop version, should be as strong as
+//   possible on a mid-range PC with reasonable thinking times. The average
+//   time per game and player is targeted at 2-3 min for the 2-color game
+//   variants and 5-6 min for the others.
+// * Level 7, the highest in the Android version, should be as strong as
+//   possible on typical mobile hardware. It is set to 4% of level 9.
+// * Level 8 is set to 20% of level 9, the middle (on a log scale) between
+//   level 7 and 9. Since most parameter tuning is done at level 7 or 8, it is
+//   better for development purposes to define level 8 in terms of time, even
+//   if it doesn't necessarily correspond to the middle wrt. playing strength.
+// * The numbers for level 1 are set to a value that is weak enough for
+//   beginners without playing silly moves. They are currently chosen depending
+//   on how strong we estimate Pentobi is in a game variant. It is also taken
+//   into consideration how much the Elo difference level 1-9 is in self-play
+//   experiments. After applying the scale factor (see comment in
+//   Player::get_rating()), we want a range of about 1000 Elo (difference
+//   between beginner and lower master level).
+// * The numbers for level 1-6 are chosen such that they correspond to roughly
+//   equidistant Elo differences measured in self-play experiments.
 
 static const float counts_classic[Player::max_supported_level] =
-    { 3, 24, 87, 213, 667, 1989, 13688, 130368, 1829033 };
+    { 3, 18, 75, 311, 1260, 8949, 66179, 330894, 1654470 };
 
 static const float counts_duo[Player::max_supported_level] =
-    { 3, 17, 44, 123, 426, 1672, 8163, 67058, 6929504 };
+    { 3, 14, 63, 253, 2203, 13614, 202994, 1014969, 5074843 };
 
 static const float counts_trigon[Player::max_supported_level] =
-    { 228, 433, 727, 1501, 2912, 7395, 27182, 79074, 489723 };
+    { 228, 376, 733, 1214, 2606, 6802, 18428, 92138, 460691 };
 
 static const float counts_nexos[Player::max_supported_level] =
-    { 100, 470, 800, 1650, 3300, 8000, 30000, 87000, 533290 };
+    { 250, 347, 625, 1223, 3117, 8270, 22626, 113130, 565651 };
+
+static const float counts_callisto_2[Player::max_supported_level] =
+    { 100, 192, 405, 1079, 3323, 12258, 94104, 470522, 2352609 };
 
 } // namespace
 
@@ -80,6 +90,8 @@ Player::Player(Variant initial_variant, unsigned max_level, string  books_dir,
         m_weight_max_count_duo[i] = 0.7f * exp(0.1f * static_cast<float>(i));
         m_weight_max_count_classic[i] = m_weight_max_count_duo[i];
         m_weight_max_count_trigon[i] = m_weight_max_count_duo[i];
+        m_weight_max_count_callisto[i] = m_weight_max_count_duo[i];
+        m_weight_max_count_callisto_2[i] = m_weight_max_count_duo[i];
         // Less weight for the first move(s) because number of legal moves
         // is lower and the search applies some pruning rules to reduce the
         // branching factor in early moves
@@ -88,11 +100,15 @@ Player::Player(Variant initial_variant, unsigned max_level, string  books_dir,
             m_weight_max_count_classic[i] *= 0.2f;
             m_weight_max_count_trigon[i] *= 0.2f;
             m_weight_max_count_duo[i] *= 0.6f;
+            m_weight_max_count_callisto[i] *= 0.2f;
+            m_weight_max_count_callisto_2[i] *= 0.2f;
         }
         else if (i == 1)
         {
             m_weight_max_count_classic[i] *= 0.2f;
             m_weight_max_count_trigon[i] *= 0.5f;
+            m_weight_max_count_callisto[i] *= 0.6f;
+            m_weight_max_count_callisto_2[i] *= 0.2f;
         }
         else if (i == 2)
         {
@@ -106,9 +122,7 @@ Player::Player(Variant initial_variant, unsigned max_level, string  books_dir,
     }
 }
 
-Player::~Player()
-{
-}
+Player::~Player() = default;
 
 Move Player::genmove(const Board& bd, Color c)
 {
@@ -151,11 +165,16 @@ Move Player::genmove(const Board& bd, Color c)
             break;
         case BoardType::trigon:
         case BoardType::trigon_3:
+        case BoardType::callisto:
+        case BoardType::callisto_3:
             max_count = counts_trigon[level - 1];
             break;
         case BoardType::nexos:
             max_count = counts_nexos[level - 1];
             break;
+        case BoardType::callisto_2:
+            max_count = counts_callisto_2[level - 1];
+            break;
         }
         // Don't weight max_count in low levels, otherwise it is still too
         // strong for beginners (later in the game, the weight becomes much
@@ -173,6 +192,13 @@ Move Player::genmove(const Board& bd, Color c)
             case BoardType::duo:
                 weight = m_weight_max_count_duo[player_move];
                 break;
+            case BoardType::callisto:
+            case BoardType::callisto_3:
+                weight = m_weight_max_count_callisto[player_move];
+                break;
+            case BoardType::callisto_2:
+                weight = m_weight_max_count_callisto_2[player_move];
+                break;
             case BoardType::trigon:
             case BoardType::trigon_3:
             case BoardType::nexos:
@@ -216,13 +242,17 @@ size_t Player::get_memory()
     size_t wanted = 2000000000;
     if (m_max_level < max_supported_level)
     {
-        // Trigon has the lowest relative number of simulations on lower levels
-        // compared to the highest level. We approximate the memory used in a
-        // search as being proportional to the number of simulations.
-        float factor =
-                counts_trigon[max_supported_level - 1]
-                / counts_trigon[m_max_level - 1];
-        wanted /= static_cast<size_t>(factor);
+        // We don't need so much memory if m_max_level is smaller than
+        // max_supported_level. Trigon has the highest relative number of
+        // simulations on lower levels compared to the highest level. The
+        // memory used in a search is not proportional to the number of
+        // simulations (e.g. because the expand threshold increases with the
+        // depth). We approximate this by adding an exponent to the ratio
+        // and not taking into account if m_max_level is very small.
+        LIBBOARDGAME_ASSERT(max_supported_level >= 5);
+        auto factor = pow(counts_trigon[max_supported_level - 1]
+                          / counts_trigon[max(m_max_level, 5u) - 1], 0.8);
+        wanted = static_cast<size_t>(double(wanted) / factor);
     }
     size_t memory = min(wanted, reasonable);
     LIBBOARDGAME_LOG("Using ", memory / 1000000, " MB of ",
@@ -232,21 +262,15 @@ size_t Player::get_memory()
 
 Rating Player::get_rating(Variant variant, unsigned level)
 {
-    // The ratings are based on experiments that measured the winning rates in
-    // games between subsequent playing levels of Pentobi. The number of
-    // simulations per level is already chosen such that the levels have
-    // roughly equal Elo differences, see Player::genmove(). A scale factor
-    // less than 1 was applied to the Elo differences to take into account that
-    // self-play experiments usually overestimate the performance against
-    // humans. The scale factor and lower anchor for the Elo ratings were
-    // chosen such that the Elo range is roughly between 1000 (beginner level)
-    // and 2000 (lower expert level) taking into account that Pentobi is
-    // relatively strong against humans in Duo and relatively weak in Trigon.
-    // Only the most popular game variants were tested (Classic 2, Duo,
-    // Trigon 2) and these ratings were used for other game variants on the
-    // same board type. The ratings are not always measured again for new
-    // versions of Pentobi and not really comparable between different versions
-    // of Pentobi.
+    // The ratings are roughly based on Elo differences measured in self-play
+    // experiments. The measured values are scaled with a factor smaller than 1
+    // to take into account that self-play usually overestimates the strength
+    // against humans. The anchor is set to about 1000 (beginner level) for
+    // level 1. The exact value for anchor and scale is chosen according to our
+    // estimate how strong Pentobi plays at level 1 and level 9 in each game
+    // variant (2000 Elo would be lower expert level). Currently, only 2-player
+    // variants are tested and the ratings are used for multi-player variants
+    // on the same board.
     auto max_supported_level = Player::max_supported_level;
     level = min(max(level, 1u), max_supported_level);
     Rating result;
@@ -256,25 +280,42 @@ Rating Player::get_rating(Variant variant, unsigned level)
         {
             // Anchor 1000, scale 0.63
             static float elo[Player::max_supported_level] =
-                { 1000, 1134, 1267, 1400, 1534, 1668, 1801, 1935, 2068 };
+                { 1000, 1145, 1290, 1435, 1580, 1725, 1870, 1957, 2021 };
             result = Rating(elo[level - 1]);
         }
         break;
     case BoardType::duo:
         {
-            // Anchor 1100, scale 0.80
+            // Anchor 1100, scale 0.7
             static float elo[Player::max_supported_level] =
-                { 1100, 1229, 1359, 1489, 1618, 1748, 1878, 2008, 2137 };
+                { 1100, 1269, 1438, 1607, 1776, 1945, 2114, 2165, 2209 };
+            result = Rating(elo[level - 1]);
+        }
+        break;
+    case BoardType::callisto_2:
+        {
+            // Anchor 1000, scale 0.63
+            static float elo[Player::max_supported_level] =
+                { 1000, 1101, 1203, 1304, 1405, 1507, 1608, 1673, 1756 };
             result = Rating(elo[level - 1]);
         }
         break;
     case BoardType::trigon:
     case BoardType::trigon_3:
-    case BoardType::nexos: // rating of Nexos not yet measured
         {
             // Anchor 1000, scale 0.60
             static float elo[Player::max_supported_level] =
-                { 1000, 1100, 1200, 1300, 1400, 1500, 1600, 1700, 1800 };
+                { 1000, 1103, 1206, 1308, 1411, 1514, 1617, 1757, 1856 };
+            result = Rating(elo[level - 1]);
+        }
+        break;
+    case BoardType::nexos:
+    case BoardType::callisto: // Not measured
+    case BoardType::callisto_3: // Not measured
+        {
+            // Anchor 1000, scale 0.60
+            static float elo[Player::max_supported_level] =
+                { 1000, 1101, 1202, 1304, 1406, 1507, 1608, 1698, 1799 };
             result = Rating(elo[level - 1]);
         }
         break;
