@@ -14,7 +14,9 @@ namespace libpentobi_mcts {
 
 using namespace std;
 using libboardgame_base::ArrayList;
+using libboardgame_util::Range;
 using libpentobi_base::Board;
+using libpentobi_base::BoardConst;
 using libpentobi_base::Color;
 using libpentobi_base::ColorMove;
 using libpentobi_base::Geometry;
@@ -95,11 +97,14 @@ public:
     void restore_snapshot(const Board& bd);
 
     /** Set points of move to forbidden. */
-    void set_forbidden(const MoveInfo& info);
+    template<unsigned MAX_SIZE>
+    void set_forbidden(const MoveInfo<MAX_SIZE>& info);
 
     /** Set adjacent points of move to forbidden. */
-    void set_forbidden(const MoveInfoExt& info_ext);
+    template<unsigned MAX_ADJ_ATTACH>
+    void set_forbidden(const MoveInfoExt<MAX_ADJ_ATTACH>& info_ext);
 
+    template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH>
     void set_local(const Board& bd);
 
 private:
@@ -125,21 +130,25 @@ inline void PlayoutFeatures::restore_snapshot(const Board& bd)
     m_point_value.copy_from(m_snapshot, bd.get_geometry());
 }
 
-inline void PlayoutFeatures::set_forbidden(const MoveInfo& info)
+template<unsigned MAX_SIZE>
+inline void PlayoutFeatures::set_forbidden(const MoveInfo<MAX_SIZE>& info)
 {
     auto p = info.begin();
-    for (unsigned i = 0; i < PieceInfo::max_size; ++i, ++p)
+    for (unsigned i = 0; i < MAX_SIZE; ++i, ++p)
         m_point_value[*p] = 0x1000u;
     m_point_value[Point::null()] = 0;
 }
 
-inline void PlayoutFeatures::set_forbidden(const MoveInfoExt& info_ext)
+template<unsigned MAX_ADJ_ATTACH>
+inline void PlayoutFeatures::set_forbidden(
+        const MoveInfoExt<MAX_ADJ_ATTACH>& info_ext)
 {
     for (auto i = info_ext.begin_adj(), end = info_ext.end_adj(); i != end;
          ++i)
         m_point_value[*i] = 0x1000u;
 }
 
+template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH>
 inline void PlayoutFeatures::set_local(const Board& bd)
 {
     // Clear old info about local points
@@ -155,6 +164,7 @@ inline void PlayoutFeatures::set_local(const Board& bd)
         second_color = bd.get_second_color(to_play);
     auto& geo = bd.get_geometry();
     auto& moves = bd.get_moves();
+    auto move_info_ext_array = bd.get_board_const().get_move_info_ext_array();
     // Consider last 3 moves for local points (i.e. last 2 opponent moves in
     // two-color variants)
     auto end = moves.end();
@@ -166,7 +176,8 @@ inline void PlayoutFeatures::set_local(const Board& bd)
             continue;
         Move mv = i->move;
         auto& is_forbidden = bd.is_forbidden(c);
-        auto& info_ext = bd.get_move_info_ext(mv);
+        auto& info_ext = BoardConst::get_move_info_ext<MAX_ADJ_ATTACH>(
+                    mv, move_info_ext_array);
         auto j = info_ext.begin_attach();
         auto end = info_ext.end_attach();
         do
@@ -180,14 +191,17 @@ inline void PlayoutFeatures::set_local(const Board& bd)
                         1 + static_cast<IntType>(
                             bd.is_attach_point(*j, to_play));
             }
-            for (Point k : geo.get_adj(*j))
-                if (! is_forbidden[k] && m_point_value[k] == 0)
-                {
-                    m_local_points.get_unchecked(nu_local++) = k;
-                    m_point_value[k] =
-                            1 + static_cast<IntType>(
-                                bd.is_attach_point(k, to_play));
-                }
+            if (MAX_SIZE == 7) // Nexos
+                LIBBOARDGAME_ASSERT(geo.get_adj(*j).empty());
+            else
+                for (Point k : geo.get_adj(*j))
+                    if (! is_forbidden[k] && m_point_value[k] == 0)
+                    {
+                        m_local_points.get_unchecked(nu_local++) = k;
+                        m_point_value[k] =
+                                1 + static_cast<IntType>(
+                                    bd.is_attach_point(k, to_play));
+                    }
         }
         while (++j != end);
     }
