@@ -27,9 +27,7 @@ Search::Search(Variant initial_variant, unsigned nu_threads, size_t memory)
     create_threads();
 }
 
-Search::~Search()
-{
-}
+Search::~Search() = default;
 
 bool Search::check_followup(ArrayList<Move, max_moves>& sequence)
 {
@@ -45,7 +43,8 @@ bool Search::check_followup(ArrayList<Move, max_moves>& sequence)
     // reuse parts of the old search tree if the computer plays both colors.
     if (m_shared_const.avoid_symmetric_draw
             && is_followup && m_to_play != m_last_history.get_to_play()
-            && ! check_symmetry_broken(bd, m_shared_const.symmetric_points))
+            && has_central_symmetry(bd.get_variant())
+            && ! check_symmetry_broken(bd))
         is_followup = false;
 
     m_last_history = m_history;
@@ -80,14 +79,27 @@ bool Search::search(Move& mv, const Board& bd, Color to_play,
     m_variant = variant;
     bool result = SearchBase::search(mv, max_count, min_simulations, max_time,
                                       time_source);
+    // Search doesn't generate all useless one-piece moves in Callisto
+    if (result && mv.is_null() && bd.get_piece_set() == PieceSet::callisto
+            && bd.is_piece_left(to_play, bd.get_one_piece()))
+    {
+        for (Point p : bd)
+            if (! bd.is_forbidden(p, to_play) && ! bd.is_center_section(p))
+            {
+                auto moves = bd.get_board_const().get_moves(bd.get_one_piece(),
+                                                            p, 0);
+                LIBBOARDGAME_ASSERT(moves.size() == 1);
+                mv = *moves.begin();
+                result = true;
+                break;
+            }
+    }
     return result;
 }
 
 void Search::set_default_param(Variant variant)
 {
     LIBBOARDGAME_LOG("Setting default parameters for ", to_string(variant));
-    set_full_select_min(10000);
-    set_full_select_interval(20);
     set_expand_threshold(1);
     set_expand_threshold_inc(0.5f);
     set_rave_weight(0.7f);
@@ -99,22 +111,31 @@ void Search::set_default_param(Variant variant)
     case Variant::classic:
     case Variant::classic_2:
     case Variant::classic_3:
-        set_bias_term_constant(0.07f);
+        set_exploration_constant(0.021f);
         set_rave_parent_max(50000);
         break;
     case Variant::duo:
     case Variant::junior:
-        set_bias_term_constant(0.05f);
+        set_exploration_constant(0.020f);
         set_rave_parent_max(25000);
         break;
     case Variant::trigon:
     case Variant::trigon_2:
     case Variant::trigon_3:
+    case Variant::callisto:
+    case Variant::callisto_3:
+        set_exploration_constant(0.014f);
+        set_rave_parent_max(50000);
+        break;
     case Variant::nexos:
     case Variant::nexos_2:
-        set_bias_term_constant(0.07f);
+        set_exploration_constant(0.008f);
         set_rave_parent_max(50000);
         break;
+    case Variant::callisto_2:
+        set_exploration_constant(0.011f);
+        set_rave_parent_max(25000);
+        break;
     }
 }
 
