@@ -12,18 +12,17 @@
 
 namespace libpentobi_mcts {
 
-using libboardgame_base::PointTransfRot180;
 using libpentobi_base::BoardConst;
 using libpentobi_base::BoardType;
-using libpentobi_base::Move;
-using libpentobi_base::Point;
 using libpentobi_base::Piece;
+using libpentobi_base::PieceSet;
+using libpentobi_base::ScoreType;
 
 //-----------------------------------------------------------------------------
 
 namespace {
 
-void filter_min_size(const BoardConst& bc, unsigned min_size,
+void filter_min_size(const BoardConst& bc, ScoreType min_size,
                      PieceMap<bool>& is_piece_considered)
 {
     for (Piece::IntType i = 0; i < bc.get_nu_pieces(); ++i)
@@ -123,6 +122,17 @@ void set_pieces_considered(const Board& bd, unsigned nu_moves,
         else if (nu_moves < 5 * nu_colors)
             filter_min_size(bc, 3, is_piece_considered);
         break;
+    case BoardType::callisto:
+    case BoardType::callisto_2:
+    case BoardType::callisto_3:
+        is_piece_considered[bd.get_one_piece()] = false;
+        if (nu_moves < 3 * nu_colors)
+            filter_min_size(bc, 5, is_piece_considered);
+        else if (nu_moves < 8 * nu_colors)
+            filter_min_size(bc, 4, is_piece_considered);
+        else if (nu_moves < 12 * nu_colors)
+            filter_min_size(bc, 3, is_piece_considered);
+        break;
     }
 }
 
@@ -134,8 +144,7 @@ SharedConst::SharedConst(const Color& to_play)
     : board(nullptr),
       to_play(to_play),
       avoid_symmetric_draw(true)
-{
-}
+{ }
 
 void SharedConst::init(bool is_followup)
 {
@@ -145,9 +154,8 @@ void SharedConst::init(bool is_followup)
     // Initialize precomp_moves
     for (Color c : bd.get_colors())
     {
-        auto& precomp_moves = this->precomp_moves[c];
-        const auto& old_precomp_moves =
-            (is_followup ? precomp_moves : bc.get_precomp_moves());
+        auto& precomp = precomp_moves[c];
+        auto& old_precomp = (is_followup ? precomp : bc.get_precomp_moves());
 
         m_is_forbidden.set();
         for (Point p : bd)
@@ -155,13 +163,33 @@ void SharedConst::init(bool is_followup)
             {
                 auto adj_status = bd.get_adj_status(p, c);
                 for (Piece piece : bd.get_pieces_left(c))
-                    if (old_precomp_moves.has_moves(piece, p, adj_status))
-                        for (Move mv :
-                             old_precomp_moves.get_moves(piece, p, adj_status))
-                            if (m_is_forbidden[mv] && ! bd.is_forbidden(c, mv))
-                                m_is_forbidden.clear(mv);
+                {
+                    if (! old_precomp.has_moves(piece, p, adj_status))
+                        continue;
+                    for (Move mv : old_precomp.get_moves(piece, p, adj_status))
+                        if (m_is_forbidden[mv] && ! bd.is_forbidden(c, mv))
+                            m_is_forbidden.clear(mv);
+                }
             }
 
+        // Don't use bd.get_pieces_left() because its ordering is not preserved
+        // during a game. The in-place construction requires that the loop
+        // iterates in the same order as during the last construction such that
+        // it doesn't overwrite elements it still needs to read.
+        Board::PiecesLeftList pieces;
+        for (Piece::IntType i = 0; i < bc.get_nu_pieces(); ++i)
+            if (bd.is_piece_left(c, Piece(i)))
+                pieces.push_back(Piece(i));
+        if (! is_followup)
+            for (Point p : bd)
+                if (! bd.is_forbidden(p, c))
+                {
+                    auto adj_status = bd.get_adj_status(p, c);
+                    for (unsigned i = 0; i < PrecompMoves::nu_adj_status; ++i)
+                        if (is_followup_adj_status(i, adj_status))
+                            for (auto piece : pieces)
+                                precomp.set_list_range(p, i, piece, 0, 0);
+                }
         unsigned n = 0;
         for (Point p : bd)
         {
@@ -172,25 +200,15 @@ void SharedConst::init(bool is_followup)
             {
                 if (! is_followup_adj_status(i, adj_status))
                     continue;
-                // Don't iterate over bd.get_pieces_left(*i) because its
-                // ordering is not preserved if a piece is removed and the
-                // in-place construction requires that the iteration in these
-                // loops is in the same order as during the last construction
-                // such that it will never overwrite any old content it still
-                // needs to read during the construction.
-                for (Piece::IntType j = 0; j < bc.get_nu_pieces(); ++j)
+                for (auto piece : pieces)
                 {
-                    Piece piece(j);
-                    if (! bd.is_piece_left(c, piece))
+                    if (! old_precomp.has_moves(piece, p, i))
                         continue;
                     auto begin = n;
-                    if (old_precomp_moves.has_moves(piece, p, i))
-                        for (auto& mv :
-                             old_precomp_moves.get_moves(piece, p, i))
-                            if (! m_is_forbidden[mv])
-                                precomp_moves.set_move(n++, mv);
-                    precomp_moves.set_list_range(p, i, piece, begin,
-                                                 n - begin);
+                    for (auto& mv : old_precomp.get_moves(piece, p, i))
+                        if (! m_is_forbidden[mv])
+                            precomp.set_move(n++, mv);
+                    precomp.set_list_range(p, i, piece, begin, n - begin);
                 }
             }
         }
@@ -198,8 +216,47 @@ void SharedConst::init(bool is_followup)
 
     if (! is_followup)
         init_pieces_considered();
+    if (bd.get_piece_set() == PieceSet::callisto)
+        init_one_piece_callisto(is_followup);
+}
 
-    symmetric_points.init(bd.get_geometry(), PointTransfRot180<Point>());
+void SharedConst::init_one_piece_callisto(bool is_followup)
+{
+    auto& bd = *board;
+    auto& bc = bd.get_board_const();
+    Piece one_piece = bd.get_one_piece();
+    unsigned n = 0;
+    if (! is_followup)
+    {
+        for (Point p : bd)
+            if (! bd.is_center_section(p) && bd.get_point_state(p).is_empty())
+            {
+                auto moves = bc.get_moves(one_piece, p, 0);
+                LIBBOARDGAME_ASSERT(moves.size() == 1);
+                Move mv = *moves.begin();
+                if (! is_useless_one_piece_point(p))
+                {
+                    one_piece_points_callisto.get_unchecked(n) = p;
+                    one_piece_moves_callisto.get_unchecked(n) = mv;
+                    ++n;
+                }
+            }
+    }
+    else
+        for (unsigned i = 0; i < one_piece_points_callisto.size(); ++i)
+        {
+            Point p = one_piece_points_callisto[i];
+            Move mv = one_piece_moves_callisto[i];
+            if (bd.get_point_state(p).is_empty()
+                    && ! is_useless_one_piece_point(p))
+            {
+                one_piece_points_callisto.get_unchecked(n) = p;
+                one_piece_moves_callisto.get_unchecked(n) = mv;
+                ++n;
+            }
+        }
+    one_piece_points_callisto.resize(n);
+    one_piece_moves_callisto.resize(n);
 }
 
 void SharedConst::init_pieces_considered()
@@ -207,13 +264,15 @@ void SharedConst::init_pieces_considered()
     auto& bd = *board;
     auto& bc = bd.get_board_const();
     is_piece_considered_list.clear();
+    bool is_callisto = (bd.get_piece_set() == PieceSet::callisto);
     for (auto i = bd.get_nu_onboard_pieces(); i < Board::max_game_moves; ++i)
     {
         PieceMap<bool> is_piece_considered;
         set_pieces_considered(bd, i, is_piece_considered);
         bool are_all_considered = true;
         for (Piece::IntType j = 0; j < bc.get_nu_pieces(); ++j)
-            if (! is_piece_considered[Piece(j)])
+            if (! is_piece_considered[Piece(j)]
+                    && ! (is_callisto && Piece(j) == bd.get_one_piece()))
             {
                 are_all_considered = false;
                 break;
@@ -235,6 +294,22 @@ void SharedConst::init_pieces_considered()
         }
     }
     is_piece_considered_all.fill(true);
+    if (is_callisto)
+        is_piece_considered_all[bd.get_one_piece()] = false;
+    is_piece_considered_none.fill(false);
+}
+
+/** Check if a point is a useless move for the 1-piece.
+    @return true if all neighbors are occupied, because the 1-piece doesn't
+    contribute to the score and playing there neither enables own moves
+    nor prevents opponent moves with larger pieces. */
+bool SharedConst::is_useless_one_piece_point(Point p) const
+{
+    auto& bd = *board;
+    for (Point pp: bd.get_geometry().get_diag(p))
+        if (bd.get_point_state(pp).is_empty())
+            return false;
+    return true;
 }
 
 //-----------------------------------------------------------------------------
