@@ -9,7 +9,6 @@
 
 #include "libpentobi_base/Board.h"
 #include "libpentobi_base/MoveMarker.h"
-#include "libpentobi_base/SymmetricPoints.h"
 
 namespace libpentobi_mcts {
 
@@ -18,10 +17,12 @@ using libboardgame_util::ArrayList;
 using libpentobi_base::Board;
 using libpentobi_base::Color;
 using libpentobi_base::ColorMap;
+using libpentobi_base::Move;
 using libpentobi_base::MoveMarker;
 using libpentobi_base::PieceMap;
+using libpentobi_base::Point;
+using libpentobi_base::PointList;
 using libpentobi_base::PrecompMoves;
-using libpentobi_base::SymmetricPoints;
 
 //-----------------------------------------------------------------------------
 
@@ -42,9 +43,6 @@ public:
 
     bool avoid_symmetric_draw;
 
-    /** Lookup table for symmetric points (only used in Duo and Trigon). */
-    SymmetricPoints symmetric_points;
-
     /** Minimum total number of pieces on the board where all pieces are
         considered until the rest of the simulation. */
     unsigned min_move_all_considered;
@@ -65,6 +63,15 @@ public:
         no moves). */
     PieceMap<bool> is_piece_considered_all;
 
+    PieceMap<bool> is_piece_considered_none;
+
+    /** List of legal points in the root position for the 1x1-piece in
+        Callisto. */
+    PointList one_piece_points_callisto;
+
+    /** Moves corresponding to one_piece_points_callisto. */
+    ArrayList<Move, Point::max_onboard> one_piece_moves_callisto;
+
 
     explicit SharedConst(const Color& to_play);
 
@@ -75,7 +82,11 @@ private:
         Reused for efficiency. */
     MoveMarker m_is_forbidden;
 
+    void init_one_piece_callisto(bool is_followup);
+
     void init_pieces_considered();
+
+    bool is_useless_one_piece_point(Point p) const;
 };
 
 //-----------------------------------------------------------------------------
