@@ -22,6 +22,7 @@ using libboardgame_util::fast_exp;
 using libpentobi_base::get_multiplayer_result;
 using libpentobi_base::BoardType;
 using libpentobi_base::PointState;
+using libpentobi_base::ScoreType;
 
 //-----------------------------------------------------------------------------
 
@@ -31,8 +32,8 @@ namespace {
     The value of nu_local dominates all other features, so we use a high
     gamma. Above some limit, we don't care about the exact value. */
 float gamma_local[PlayoutFeatures::max_local + 1] =
-  { 1, 1e6f, 1e12f, 1e18f, 1e24f, 1e30f, 1e30f, 1e30f, 1e30f, 1e30f, 1e30f,
-    1e30f, 1e30f, 1e30f, 1e30f };
+  { 1, 1e6f, 1e12f, 1e18f, 1e24f, 1e25f, 1e25f, 1e25f, 1e25f, 1e25f, 1e25f,
+    1e25f, 1e25f, 1e25f, 1e25f };
 
 inline Float sigmoid(Float steepness, Float x)
 {
@@ -49,7 +50,7 @@ State::State(Variant initial_variant, const SharedConst& shared_const)
 {
 }
 
-template<unsigned PIECE_SIZE>
+template<unsigned MAX_SIZE, bool IS_CALLISTO>
 inline void State::add_moves(Point p, Color c,
                              const Board::PiecesLeftList& pieces,
                              float& total_gamma, MoveList& moves,
@@ -65,14 +66,42 @@ inline void State::add_moves(Point p, Color c,
         auto gamma_piece = m_gamma_piece[piece];
         for (Move mv : get_moves(c, piece, p, adj_status))
             if (! marker[mv]
-                    && check_move<PIECE_SIZE>(
-                        mv, get_move_info(mv), gamma_piece, moves, nu_moves,
-                        playout_features, total_gamma))
+                    && check_move<MAX_SIZE, IS_CALLISTO>(
+                           mv, get_move_info<MAX_SIZE>(mv), gamma_piece, moves,
+                           nu_moves, playout_features, total_gamma))
                 marker.set(mv);
     }
 }
 
-template<unsigned PIECE_SIZE>
+template<unsigned MAX_SIZE>
+void State::add_one_piece_moves(Color c, bool with_gamma, float& total_gamma,
+                                MoveList& moves, unsigned& nu_moves)
+{
+    Piece one_piece = m_bd.get_one_piece();
+    auto nu_left = m_bd.get_nu_left_piece(c, one_piece);
+    if (nu_left == 0)
+        return;
+    for (unsigned i = 0; i < m_shared_const.one_piece_points_callisto.size();
+         ++i)
+    {
+        Point p = m_shared_const.one_piece_points_callisto[i];
+        if (m_bd.is_forbidden(p, c))
+            continue;
+        Move mv = m_shared_const.one_piece_moves_callisto[i];
+        LIBBOARDGAME_ASSERT(nu_moves < MoveList::max_size);
+        moves.get_unchecked(nu_moves) = mv;
+        ++nu_moves;
+        LIBBOARDGAME_ASSERT(! m_marker[c][mv]);
+        m_marker[c].set(mv);
+        if (with_gamma)
+        {
+            total_gamma += m_gamma_piece[one_piece];
+            m_cumulative_gamma[nu_moves - 1] = total_gamma;
+        }
+    }
+}
+
+template<unsigned MAX_SIZE>
 void State::add_starting_moves(Color c, const Board::PiecesLeftList& pieces,
                                bool with_gamma, MoveList& moves)
 {
@@ -91,7 +120,7 @@ void State::add_starting_moves(Color c, const Board::PiecesLeftList& pieces,
         for (Move mv : get_moves(c, piece, p, 0))
         {
             LIBBOARDGAME_ASSERT(! marker[mv]);
-            if (check_forbidden<PIECE_SIZE>(is_forbidden, mv, moves, nu_moves))
+            if (check_forbidden<MAX_SIZE>(is_forbidden, mv, moves, nu_moves))
             {
                 marker.set(mv);
                 if (with_gamma)
@@ -104,14 +133,13 @@ void State::add_starting_moves(Color c, const Board::PiecesLeftList& pieces,
     moves.resize(nu_moves);
 }
 
-template<unsigned PIECE_SIZE>
+template<unsigned MAX_SIZE>
 bool State::check_forbidden(const GridExt<bool>& is_forbidden, Move mv,
                             MoveList& moves, unsigned& nu_moves)
 {
-    auto& info = get_move_info(mv);
-    auto p = info.begin();
+    auto p = get_move_info<MAX_SIZE>(mv).begin();
     unsigned forbidden = is_forbidden[*p];
-    for (unsigned i = 1; i < PIECE_SIZE; ++i)
+    for (unsigned i = 1; i < MAX_SIZE; ++i)
         // Logically, forbidden is a bool and the next line should be
         //   forbidden = forbidden || is_forbidden[*(++p)]
         // But this generates branches, which are bad for performance in this
@@ -126,19 +154,22 @@ bool State::check_forbidden(const GridExt<bool>& is_forbidden, Move mv,
     return true;
 }
 
-template<unsigned PIECE_SIZE>
-bool State::check_move(Move mv, const MoveInfo& info, float gamma_piece,
-                       MoveList& moves, unsigned& nu_moves,
+template<unsigned MAX_SIZE, bool IS_CALLISTO>
+bool State::check_move(Move mv, const MoveInfo<MAX_SIZE>& info,
+                       float gamma_piece, MoveList& moves, unsigned& nu_moves,
                        const PlayoutFeatures& playout_features,
                        float& total_gamma)
 {
+    LIBBOARDGAME_ASSERT(IS_CALLISTO == m_is_callisto);
     auto p = info.begin();
     PlayoutFeatures::Compute features(*p, playout_features);
-    for (unsigned i = 1; i < PIECE_SIZE; ++i)
+    for (unsigned i = 1; i < MAX_SIZE; ++i)
         features.add(*(++p), playout_features);
     if (features.is_forbidden())
         return false;
-    auto gamma = gamma_piece * gamma_local[features.get_nu_local()];
+    auto gamma = gamma_piece;
+    if (! (IS_CALLISTO && info.get_size() == 1))
+        gamma *= gamma_local[features.get_nu_local()];
     total_gamma += gamma;
     m_cumulative_gamma[nu_moves] = total_gamma;
     LIBBOARDGAME_ASSERT(nu_moves < MoveList::max_size);
@@ -147,13 +178,13 @@ bool State::check_move(Move mv, const MoveInfo& info, float gamma_piece,
     return true;
 }
 
-template<unsigned PIECE_SIZE>
-inline bool State::check_move(Move mv, const MoveInfo& info, MoveList& moves,
-                              unsigned& nu_moves,
+template<unsigned MAX_SIZE, bool IS_CALLISTO>
+inline bool State::check_move(Move mv, const MoveInfo<MAX_SIZE>& info,
+                              MoveList& moves, unsigned& nu_moves,
                               const PlayoutFeatures& playout_features,
                               float& total_gamma)
 {
-    return check_move<PIECE_SIZE>(
+    return check_move<MAX_SIZE, IS_CALLISTO>(
                 mv, info, m_gamma_piece[info.get_piece()], moves, nu_moves,
                 playout_features, total_gamma);
 }
@@ -167,37 +198,6 @@ string State::dump() const
 }
 #endif
 
-/** Evaluation function for Duo and Junior. */
-void State::evaluate_duo(array<Float, 6>& result)
-{
-    LIBBOARDGAME_ASSERT(m_bd.get_nu_players() == 2);
-    LIBBOARDGAME_ASSERT(m_bd.get_nu_colors() == 2);
-    if (! m_is_symmetry_broken
-            && m_bd.get_nu_onboard_pieces() >= m_symmetry_min_nu_pieces)
-    {
-        if (log_simulations)
-            LIBBOARDGAME_LOG("Result: 0.5 (symmetry)");
-        result[0] = result[1] = 0.5;
-        return;
-    }
-    Color c(0);
-    auto s = static_cast<Float>(m_bd.get_score_twocolor(c));
-    Float res;
-    if (s > 0)
-        res = 1;
-    else if (s < 0)
-        res = 0;
-    else
-        res = 0.5;
-    if (log_simulations)
-        LIBBOARDGAME_LOG("Result color ", c, ": sco=", s, " game_res=", res);
-    res += get_quality_bonus(c, res, s, false);
-    if (log_simulations)
-        LIBBOARDGAME_LOG("res=", res);
-    result[0] = res;
-    result[1] = 1.f - res;
-}
-
 /** Evaluation function for game variants with 2 players and 2 colors per
     player. */
 void State::evaluate_multicolor(array<Float, 6>& result)
@@ -217,8 +217,7 @@ void State::evaluate_multicolor(array<Float, 6>& result)
         return;
     }
 
-    Color c(0);
-    auto s = static_cast<Float>(m_bd.get_score_multicolor(c));
+    auto s = m_bd.get_score_multicolor(Color(0));
     Float res;
     if (s > 0)
         res = 1;
@@ -227,8 +226,9 @@ void State::evaluate_multicolor(array<Float, 6>& result)
     else
         res = 0.5;
     if (log_simulations)
-        LIBBOARDGAME_LOG("Result color ", c, ": sco=", s, " game_res=", res);
-    res += get_quality_bonus(c, res, s, true);
+        LIBBOARDGAME_LOG("Result color 0: sco=", s, " game_res=", res);
+    res += get_quality_bonus(Color(0), res, s)
+            + get_quality_bonus_attach_multicolor();
     if (log_simulations)
         LIBBOARDGAME_LOG("res=", res);
     result[0] = result[2] = res;
@@ -248,20 +248,19 @@ void State::evaluate_multiplayer(array<Float, 6>& result)
 {
     auto nu_players = m_bd.get_nu_players();
     LIBBOARDGAME_ASSERT(nu_players > 2);
-    array<unsigned, Color::range> points;
+    array<ScoreType, Color::range> points;
     for (Color::IntType i = 0; i < nu_players; ++i)
         points[i] = m_bd.get_points(Color(i));
     array<Float, Color::range> game_result;
-    get_multiplayer_result(nu_players, points, game_result);
+    get_multiplayer_result(nu_players, points, game_result, m_is_callisto);
     for (Color::IntType i = 0; i < nu_players; ++i)
     {
         Color c(i);
-        auto s = static_cast<Float>(m_bd.get_score_multiplayer(c));
-        result[i] =
-                game_result[i] + get_quality_bonus(c, game_result[i], s, true);
+        auto s = m_bd.get_score_multiplayer(c);
+        result[i] = game_result[i] + get_quality_bonus(c, game_result[i], s);
         if (log_simulations)
-            LIBBOARDGAME_LOG("Result color ", c, ": sco=", s, " game_res=",
-                             game_result[i], " res=", result[i]);
+            LIBBOARDGAME_LOG("Result sco=", s, " game_res=", game_result[i],
+                             " res=", result[i]);
     }
     if (m_bd.get_variant() == Variant::classic_3)
     {
@@ -271,6 +270,39 @@ void State::evaluate_multiplayer(array<Float, 6>& result)
     }
 }
 
+/** Evaluation function for Duo, Junior and Callisto Two-Player. */
+void State::evaluate_twocolor(array<Float, 6>& result)
+{
+    LIBBOARDGAME_ASSERT(m_bd.get_nu_players() == 2);
+    LIBBOARDGAME_ASSERT(m_bd.get_nu_colors() == 2);
+    ScoreType s;
+    if (! m_is_symmetry_broken
+            && m_bd.get_nu_onboard_pieces() >= m_symmetry_min_nu_pieces)
+    {
+        if (log_simulations)
+            LIBBOARDGAME_LOG("Symmetry not broken");
+        s = 0;
+    }
+    else
+        s = m_bd.get_score_twocolor(Color(0));
+    Float res;
+    if (s > 0)
+        res = 1;
+    else if (s < 0 || (m_is_callisto && s == 0))
+        res = 0;
+    else
+        res = 0.5;
+    if (log_simulations)
+        LIBBOARDGAME_LOG("Result sco=", s, " game_res=", res);
+    res += get_quality_bonus(Color(0), res, s);
+    if (m_is_callisto)
+        res += get_quality_bonus_attach_twocolor();
+    if (log_simulations)
+        LIBBOARDGAME_LOG("res=", res);
+    result[0] = res;
+    result[1] = 1.f - res;
+}
+
 Point State::find_best_starting_point(Color c) const
 {
     // We use the starting point that maximizes the distance to occupied
@@ -326,27 +358,33 @@ Point State::find_best_starting_point(Color c) const
     return best;
 }
 
-bool State::gen_children(Tree::NodeExpander& expander, Float init_val)
+bool State::gen_children(Tree::NodeExpander& expander, Float root_val)
 {
     if (m_nu_passes == m_nu_colors)
         return true;
     Color to_play = m_bd.get_to_play();
-    switch (m_piece_set)
+    if (m_max_piece_size == 5)
     {
-    case PieceSet::classic:
-    case PieceSet::junior:
         init_moves_without_gamma<5>(to_play);
-        break;
-    case PieceSet::trigon:
+        return m_prior_knowledge.gen_children<5, 16>(m_bd, m_moves[to_play],
+                                                     m_is_symmetry_broken,
+                                                     expander, root_val);
+    }
+    else if (m_max_piece_size == 6)
+    {
         init_moves_without_gamma<6>(to_play);
-        break;
-    case PieceSet::nexos:
+        return m_prior_knowledge.gen_children<6, 22>(m_bd, m_moves[to_play],
+                                                     m_is_symmetry_broken,
+                                                     expander, root_val);
+    }
+    else
+    {
+        LIBBOARDGAME_ASSERT(m_max_piece_size == 7);
         init_moves_without_gamma<7>(to_play);
-        break;
+        return m_prior_knowledge.gen_children<7, 12>(m_bd, m_moves[to_play],
+                                                     m_is_symmetry_broken,
+                                                     expander, root_val);
     }
-    return m_prior_knowledge.gen_children(m_bd, m_moves[to_play],
-                                          m_is_symmetry_broken, expander,
-                                          init_val);
 }
 
 bool State::gen_playout_move_full(PlayerMove<Move>& mv)
@@ -356,35 +394,31 @@ bool State::gen_playout_move_full(PlayerMove<Move>& mv)
     {
         if (! m_is_move_list_initialized[to_play])
         {
-            switch (m_piece_set)
+            if (m_max_piece_size == 5)
             {
-            case PieceSet::classic:
-            case PieceSet::junior:
-                init_moves_with_gamma<5>(to_play);
-                break;
-            case PieceSet::trigon:
-                init_moves_with_gamma<6>(to_play);
-                break;
-            case PieceSet::nexos:
-                init_moves_with_gamma<7>(to_play);
-                break;
+                if (m_is_callisto)
+                    init_moves_with_gamma<5, 16, true>(to_play);
+                else
+                    init_moves_with_gamma<5, 16, false>(to_play);
             }
+            else if (m_max_piece_size == 6)
+                init_moves_with_gamma<6, 22, false>(to_play);
+            else
+                init_moves_with_gamma<7, 12, false>(to_play);
         }
         else if (m_has_moves[to_play])
         {
-            switch (m_piece_set)
+            if (m_max_piece_size == 5)
             {
-            case PieceSet::classic:
-            case PieceSet::junior:
-                update_moves<5>(to_play);
-                break;
-            case PieceSet::trigon:
-                update_moves<6>(to_play);
-                break;
-            case PieceSet::nexos:
-                update_moves<7>(to_play);
-                break;
+                if (m_is_callisto)
+                    update_moves<5, 16, true>(to_play);
+                else
+                    update_moves<5, 16, false>(to_play);
             }
+            else if (m_max_piece_size == 6)
+                update_moves<6, 22, false>(to_play);
+            else
+                update_moves<7, 12, false>(to_play);
         }
         if ((m_has_moves[to_play] = ! m_moves[to_play].empty()))
             break;
@@ -404,6 +438,7 @@ bool State::gen_playout_move_full(PlayerMove<Move>& mv)
     }
 
     auto& moves = m_moves[to_play];
+    LIBBOARDGAME_ASSERT(! moves.empty());
     auto total_gamma = m_cumulative_gamma[moves.size() - 1];
     if (log_simulations)
         LIBBOARDGAME_LOG("Moves: ", moves.size(), ", total_gamma: ",
@@ -430,16 +465,18 @@ string State::get_info() const
     return s.str();
 }
 
-inline const PieceMap<bool>& State::get_is_piece_considered() const
+inline const PieceMap<bool>& State::get_is_piece_considered(Color c) const
 {
+    if (m_is_callisto
+            && m_bd.get_nu_left_piece(c, m_bd.get_one_piece()) > 1)
+        return m_shared_const.is_piece_considered_none;
     // Use number of on-board pieces for move number to handle the case where
     // there are more pieces on the board than moves (setup positions)
     unsigned nu_moves = m_bd.get_nu_onboard_pieces();
     if (nu_moves >= m_shared_const.min_move_all_considered
             || m_force_consider_all_pieces)
         return m_shared_const.is_piece_considered_all;
-    else
-        return *m_shared_const.is_piece_considered[nu_moves];
+    return *m_shared_const.is_piece_considered[nu_moves];
 }
 
 /** Initializes and returns m_pieces_considered if not all pieces are
@@ -448,7 +485,8 @@ inline const Board::PiecesLeftList& State::get_pieces_considered(Color c)
 {
     auto is_piece_considered = m_is_piece_considered[c];
     auto& pieces_left = m_bd.get_pieces_left(c);
-    if (is_piece_considered == &m_shared_const.is_piece_considered_all)
+    if (is_piece_considered == &m_shared_const.is_piece_considered_all
+            && ! m_is_callisto)
         return pieces_left;
     unsigned n = 0;
     for (Piece piece : pieces_left)
@@ -458,11 +496,10 @@ inline const Board::PiecesLeftList& State::get_pieces_considered(Color c)
     return m_pieces_considered;
 }
 
-/** Bonus added to the result for quality-based rewards.
+/** Basic bonus added to the result for quality-based rewards.
     See also: Pepels et al.: Quality-based Rewards for Monte-Carlo Tree Search
     Simulations. ECAI 2014. */
-inline Float State::get_quality_bonus(Color c, Float result, Float score,
-                                      bool use_nu_attach)
+inline Float State::get_quality_bonus(Color c, Float result, Float score)
 {
     Float bonus = 0;
 
@@ -471,8 +508,7 @@ inline Float State::get_quality_bonus(Color c, Float result, Float score,
     m_stat_len.add(l);
     Float var = m_stat_len.get_variance();
     if (var > 0)
-        bonus +=
-                (result == 1 ? -0.06f : 0.06f)
+        bonus += -0.12f * (result - 0.5f)
                 * sigmoid(2.f, (l - m_stat_len.get_mean()) / sqrt(var));
 
     // Game score
@@ -481,57 +517,83 @@ inline Float State::get_quality_bonus(Color c, Float result, Float score,
     var = stat.get_variance();
     if (var > 0)
         bonus += 0.3f * sigmoid(2.f, (score - stat.get_mean()) / sqrt(var));
-
-    // Number of non-forbidden attach points is another feature of a superior
-    // final position. Not used in Duo/Junior, mainly helps in Trigon.
-    if (! use_nu_attach)
-        return bonus;
-    auto second_color = m_bd.get_second_color(c);
-    Float opp_weight = (m_bd.get_nu_players() == 2 ? 1.f : 1.f / m_nu_colors);
-    Float attach = 0;
-    for (Color i : Color::Range(m_nu_colors))
-        if (i == c || i == second_color)
-        {
-            for (Point p : m_bd.get_attach_points(i))
-                if (! m_bd.is_forbidden(p, i))
-                    ++attach;
-        }
-        else
-        {
-            Float n = 0;
-            for (Point p : m_bd.get_attach_points(i))
-                if (! m_bd.is_forbidden(p, i))
-                    ++n;
-            attach -= opp_weight * n;
-        }
-    m_stat_attach.add(attach);
-    var = m_stat_attach.get_variance();
-    if (var > 0)
-        bonus +=
-          0.1f * sigmoid(2.f, (attach - m_stat_attach.get_mean()) / sqrt(var));
     return bonus;
 }
 
-template<unsigned PIECE_SIZE>
+/** Additional quality-based rewards based on number of attach points.
+    The number of non-forbidden attach points is another feature of a superior
+    final position. Only used in some two-player variants, mainly helps in
+    Trigon. */
+inline Float State::get_quality_bonus_attach_twocolor()
+{
+    LIBBOARDGAME_ASSERT(m_bd.get_nu_players() == 2);
+    int n = m_bd.get_attach_points(Color(0)).size()
+            - m_bd.get_attach_points(Color(1)).size();
+    for (Point p : m_bd.get_attach_points(Color(0)))
+        n -= m_bd.is_forbidden(p, Color(0));
+    for (Point p : m_bd.get_attach_points(Color(1)))
+        n += m_bd.is_forbidden(p, Color(1));
+    Float attach = static_cast<Float>(n);
+    m_stat_attach.add(attach);
+    auto var = m_stat_attach.get_variance();
+    if (var > 0)
+        return 0.1f * sigmoid(2.f,
+                              (attach - m_stat_attach.get_mean()) / sqrt(var));
+    return 0;
+}
+
+/** Like get_quality_bonus_attach_twocolor() but for 2 colors per player. */
+inline Float State::get_quality_bonus_attach_multicolor()
+{
+    LIBBOARDGAME_ASSERT(m_bd.get_nu_players() == 2);
+    LIBBOARDGAME_ASSERT(m_bd.get_nu_colors() == 4);
+    int n = m_bd.get_attach_points(Color(0)).size()
+            + m_bd.get_attach_points(Color(2)).size()
+            - m_bd.get_attach_points(Color(1)).size()
+            - m_bd.get_attach_points(Color(3)).size();
+    for (Point p : m_bd.get_attach_points(Color(0)))
+        n -= m_bd.is_forbidden(p, Color(0));
+    for (Point p : m_bd.get_attach_points(Color(2)))
+        n -= m_bd.is_forbidden(p, Color(2));
+    for (Point p : m_bd.get_attach_points(Color(1)))
+        n += m_bd.is_forbidden(p, Color(1));
+    for (Point p : m_bd.get_attach_points(Color(3)))
+        n += m_bd.is_forbidden(p, Color(3));
+    Float attach = static_cast<Float>(n);
+    m_stat_attach.add(attach);
+    auto var = m_stat_attach.get_variance();
+    if (var > 0)
+        return 0.1f * sigmoid(2.f,
+                              (attach - m_stat_attach.get_mean()) / sqrt(var));
+    return 0;
+}
+
+template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH, bool IS_CALLISTO>
 void State::init_moves_with_gamma(Color c)
 {
-    m_is_piece_considered[c] = &get_is_piece_considered();
-    m_playout_features[c].set_local(m_bd);
+    m_is_piece_considered[c] = &get_is_piece_considered(c);
+    m_playout_features[c].set_local<MAX_SIZE, MAX_ADJ_ATTACH>(m_bd);
     auto& marker = m_marker[c];
     auto& moves = m_moves[c];
     marker.clear(moves);
     auto& pieces = get_pieces_considered(c);
-    if (m_bd.is_first_piece(c))
-        add_starting_moves<PIECE_SIZE>(c, pieces, true, moves);
+    if (m_bd.is_first_piece(c) && ! (MAX_SIZE == 5 && m_is_callisto))
+        add_starting_moves<MAX_SIZE>(c, pieces, true, moves);
     else
     {
         unsigned nu_moves = 0;
         float total_gamma = 0;
-        for (Point p : m_bd.get_attach_points(c))
-            if (! m_bd.is_forbidden(p, c))
+        if (MAX_SIZE == 5 && m_is_callisto)
+            add_one_piece_moves<MAX_SIZE>(c, true, total_gamma, moves,
+                                          nu_moves);
+        if (m_is_piece_considered[c]
+                != &m_shared_const.is_piece_considered_none)
+            for (Point p : m_bd.get_attach_points(c))
             {
-                add_moves<PIECE_SIZE>(
-                            p, c, pieces, total_gamma, moves, nu_moves);
+                if (m_bd.is_forbidden(p, c))
+                    continue;
+                add_moves<MAX_SIZE, IS_CALLISTO>(p, c, pieces, total_gamma,
+                                                 moves, nu_moves);
                 m_moves_added_at[c][p] = true;
             }
         moves.resize(nu_moves);
@@ -539,30 +601,41 @@ void State::init_moves_with_gamma(Color c)
     m_is_move_list_initialized[c] = true;
     m_nu_new_moves[c] = 0;
     m_last_attach_points_end[c] = m_bd.get_attach_points(c).end();
-    if (moves.empty() && &pieces == &m_pieces_considered)
+    if (moves.empty() &&
+            m_is_piece_considered[c]
+            != &m_shared_const.is_piece_considered_all)
     {
         m_force_consider_all_pieces = true;
-        init_moves_with_gamma<PIECE_SIZE>(c);
+        init_moves_with_gamma<MAX_SIZE, MAX_ADJ_ATTACH, IS_CALLISTO>(c);
     }
 }
 
-template<unsigned PIECE_SIZE>
+template<unsigned MAX_SIZE>
 void State::init_moves_without_gamma(Color c)
 {
-    m_is_piece_considered[c] = &get_is_piece_considered();
+    m_is_piece_considered[c] = &get_is_piece_considered(c);
     auto& marker = m_marker[c];
     auto& moves = m_moves[c];
     marker.clear(moves);
     auto& pieces = get_pieces_considered(c);
     auto& is_forbidden = m_bd.is_forbidden(c);
-    if (m_bd.is_first_piece(c))
-        add_starting_moves<PIECE_SIZE>(c, pieces, false, moves);
+    if (m_bd.is_first_piece(c) && ! (MAX_SIZE == 5 && m_is_callisto))
+        add_starting_moves<MAX_SIZE>(c, pieces, false, moves);
     else
     {
         unsigned nu_moves = 0;
-        for (Point p : m_bd.get_attach_points(c))
-            if (! is_forbidden[p])
+        if (MAX_SIZE == 5 && m_is_callisto)
+        {
+            float total_gamma_dummy;
+            add_one_piece_moves<MAX_SIZE>(c, false, total_gamma_dummy, moves,
+                                          nu_moves);
+        }
+        if (m_is_piece_considered[c]
+                != &m_shared_const.is_piece_considered_none)
+            for (Point p : m_bd.get_attach_points(c))
             {
+                if (is_forbidden[p])
+                    continue;
                 auto adj_status = m_bd.get_adj_status(p, c);
                 for (Piece piece : pieces)
                 {
@@ -570,7 +643,7 @@ void State::init_moves_without_gamma(Color c)
                         continue;
                     for (Move mv : get_moves(c, piece, p, adj_status))
                         if (! marker[mv]
-                                && check_forbidden<PIECE_SIZE>(
+                                && check_forbidden<MAX_SIZE>(
                                     is_forbidden, mv, moves, nu_moves))
                             marker.set(mv);
                 }
@@ -581,10 +654,12 @@ void State::init_moves_without_gamma(Color c)
     m_is_move_list_initialized[c] = true;
     m_nu_new_moves[c] = 0;
     m_last_attach_points_end[c] = m_bd.get_attach_points(c).end();
-    if (moves.empty() && &pieces == &m_pieces_considered)
+    if (moves.empty() &&
+            m_is_piece_considered[c]
+            != &m_shared_const.is_piece_considered_all)
     {
         m_force_consider_all_pieces = true;
-        init_moves_without_gamma<PIECE_SIZE>(c);
+        init_moves_without_gamma<MAX_SIZE>(c);
     }
 }
 
@@ -615,10 +690,11 @@ void State::start_search()
     m_bd.set_to_play(m_shared_const.to_play);
     m_bd.take_snapshot();
     m_nu_colors = bd.get_nu_colors();
+    m_is_callisto = (bd.get_piece_set() == PieceSet::callisto);
     for (Color c : Color::Range(m_nu_colors))
         m_playout_features[c].init_snapshot(m_bd, c);
     m_bc = &m_bd.get_board_const();
-    m_piece_set = m_bc->get_piece_set();
+    m_max_piece_size = m_bc->get_max_piece_size();
     m_move_info_array = m_bc->get_move_info_array();
     m_move_info_ext_array = m_bc->get_move_info_ext_array();
     m_check_terminate_early =
@@ -626,20 +702,23 @@ void State::start_search()
              && m_bd.get_nu_players() == 2);
     auto variant = bd.get_variant();
     m_check_symmetric_draw =
-        ((variant == Variant::duo || variant == Variant::junior
-          || variant == Variant::trigon_2)
-         && ! ((m_shared_const.to_play == Color(1)
-                || m_shared_const.to_play == Color(3))
-               && m_shared_const.avoid_symmetric_draw)
-         && ! check_symmetry_broken(bd, m_shared_const.symmetric_points));
+            (has_central_symmetry(variant)
+             && ! ((m_shared_const.to_play == Color(1)
+                    || m_shared_const.to_play == Color(3))
+                   && m_shared_const.avoid_symmetric_draw)
+             && ! check_symmetry_broken(bd));
     if (! m_check_symmetric_draw)
         // Pretending that the symmetry is always broken is equivalent to
         // ignoring symmetric draws
         m_is_symmetry_broken = true;
-    if (variant == Variant::trigon_2)
+    if (variant == Variant::trigon_2 || variant == Variant::callisto_2)
         m_symmetry_min_nu_pieces = 5;
     else
-        m_symmetry_min_nu_pieces = 3; // Only used in Duo
+    {
+        LIBBOARDGAME_ASSERT(! m_check_symmetric_draw || variant == Variant::duo
+                            || variant == Variant::junior);
+        m_symmetry_min_nu_pieces = 3;
+    }
 
     m_prior_knowledge.start_search(bd);
     m_stat_len.clear();
@@ -660,26 +739,30 @@ void State::start_search()
         gamma_nu_attach_factor = 1.8f;
         break;
     case BoardType::trigon:
-    case BoardType::trigon_3:
+    case BoardType::trigon_3: // Not tuned
         gamma_size_factor = 5;
         break;
-    case BoardType::nexos:
-        // Not yet tuned
+    case BoardType::nexos: // Not tuned
         gamma_size_factor = 5;
         gamma_nu_attach_factor = 1.8f;
         break;
+    case BoardType::callisto_2:
+    case BoardType::callisto: // Not tuned
+    case BoardType::callisto_3: // Not tuned
+        gamma_size_factor = 12;
+        gamma_nu_attach_factor = 1.8f;
+        break;
     }
     for (Piece::IntType i = 0; i < m_bc->get_nu_pieces(); ++i)
     {
         Piece piece(i);
-        auto piece_size = static_cast<float>(
-                    m_bc->get_piece_info(piece).get_score_points());
+        auto score_points = m_bc->get_piece_info(piece).get_score_points();
         auto piece_nu_attach =
                 static_cast<float>(m_bc->get_nu_attach_points(piece));
-        LIBBOARDGAME_ASSERT(piece_size > 0);
+        LIBBOARDGAME_ASSERT(score_points >= 0);
         LIBBOARDGAME_ASSERT(piece_nu_attach > 0);
         m_gamma_piece[piece] =
-            pow(gamma_size_factor, piece_size - 1)
+            pow(gamma_size_factor, score_points)
             * pow(gamma_nu_attach_factor, piece_nu_attach - 1);
     }
 }
@@ -706,50 +789,43 @@ void State::start_simulation(size_t n)
     m_nu_passes = 0;
 }
 
-template<unsigned PIECE_SIZE>
+template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH, bool IS_CALLISTO>
 void State::update_moves(Color c)
 {
     auto& playout_features = m_playout_features[c];
-    playout_features.set_local(m_bd);
+    playout_features.set_local<MAX_SIZE, MAX_ADJ_ATTACH>(m_bd);
 
     auto& marker = m_marker[c];
 
     // Find old moves that are still legal
     auto& is_forbidden = m_bd.is_forbidden(c);
     auto& moves = m_moves[c];
-    auto old_size = moves.size();
     unsigned nu_moves = 0;
     float total_gamma = 0;
-    if (m_nu_new_moves[c] == 1 && m_bd.get_nu_piece_instances() == 1)
-    {
-        Piece piece = get_move_info(m_last_move[c]).get_piece();
-        for (unsigned i = 0; i < old_size; ++i)
+    Piece piece;
+    if (m_nu_new_moves[c] == 1 &&
+            ! m_bd.is_piece_left(
+                c, (piece =
+                    get_move_info<MAX_SIZE>(m_last_move[c]).get_piece())))
+        for (Move mv : moves)
         {
-            LIBBOARDGAME_ASSERT(i >= nu_moves);
-            Move mv = moves[i];
-            auto& info = get_move_info(mv);
+            auto& info = get_move_info<MAX_SIZE>(mv);
             if (info.get_piece() == piece
-                    || ! check_move<PIECE_SIZE>(mv, info, moves, nu_moves,
-                                                playout_features, total_gamma))
+                    || ! check_move<MAX_SIZE, IS_CALLISTO>(
+                             mv, info, moves, nu_moves, playout_features,
+                             total_gamma))
                 marker.clear(mv);
         }
-    }
     else
-    {
-        PieceMap<bool> is_piece_left(false);
-        for (Piece piece : m_bd.get_pieces_left(c))
-            is_piece_left[piece] = true;
-        for (unsigned i = 0; i < old_size; ++i)
+        for (Move mv : moves)
         {
-            LIBBOARDGAME_ASSERT(i >= nu_moves);
-            Move mv = moves[i];
-            auto& info = get_move_info(mv);
-            if (! is_piece_left[info.get_piece()]
-                    || ! check_move<PIECE_SIZE>(mv, info, moves,  nu_moves,
-                                                playout_features, total_gamma))
+            auto& info = get_move_info<MAX_SIZE>(mv);
+            if (! m_bd.is_piece_left(c, info.get_piece())
+                    || ! check_move<MAX_SIZE, IS_CALLISTO>(
+                             mv, info, moves, nu_moves, playout_features,
+                             total_gamma))
                 marker.clear(mv);
         }
-    }
 
     // Find new legal moves because of new pieces played by this color
     auto& pieces = get_pieces_considered(c);
@@ -760,19 +836,20 @@ void State::update_moves(Color c)
         if (! is_forbidden[*i] && ! m_moves_added_at[c][*i])
         {
             m_moves_added_at[c][*i] = true;
-            add_moves<PIECE_SIZE>(*i, c, pieces, total_gamma, moves, nu_moves);
+            add_moves<MAX_SIZE, IS_CALLISTO>(*i, c, pieces, total_gamma, moves,
+                                             nu_moves);
         }
     m_nu_new_moves[c] = 0;
     m_last_attach_points_end[c] = end;
 
     // Generate moves for pieces not considered in the last position
-    if (&pieces == &m_pieces_considered)
+    if (m_is_piece_considered[c] != &m_shared_const.is_piece_considered_all)
     {
         auto& is_piece_considered = *m_is_piece_considered[c];
         if (nu_moves == 0)
             m_force_consider_all_pieces = true;
-        auto& is_piece_considered_new = get_is_piece_considered();
-        if (&is_piece_considered !=  &is_piece_considered_new)
+        auto& is_piece_considered_new = get_is_piece_considered(c);
+        if (&is_piece_considered != &is_piece_considered_new)
         {
             Board::PiecesLeftList new_pieces;
             unsigned n = 0;
@@ -783,54 +860,14 @@ void State::update_moves(Color c)
             new_pieces.resize(n);
             for (Point p : attach_points)
                 if (! is_forbidden[p])
-                    add_moves<PIECE_SIZE>(p, c, new_pieces, total_gamma, moves,
-                                          nu_moves);
+                    add_moves<MAX_SIZE, IS_CALLISTO>(
+                        p, c, new_pieces, total_gamma, moves, nu_moves);
             m_is_piece_considered[c] = &is_piece_considered_new;
         }
     }
     moves.resize(nu_moves);
 }
 
-void State::update_symmetry_broken(Move mv)
-{
-    Color to_play = m_bd.get_to_play();
-    Color second_color = m_bd.get_second_color(to_play);
-    auto& info = get_move_info(mv);
-    auto i = info.begin();
-    auto end = info.end();
-    if (to_play == Color(0) || to_play == Color(2))
-    {
-        // First player to play: Check that all symmetric points of the last
-        // move of the second player are occupied by the first player
-        do
-        {
-            Point symm_p = m_shared_const.symmetric_points[*i];
-            if (m_bd.get_point_state(symm_p) != second_color)
-            {
-                m_is_symmetry_broken = true;
-                return;
-            }
-        }
-        while (++i != end);
-    }
-    else
-    {
-        // Second player to play: Check that all symmetric points of the last
-        // move of the first player are empty (i.e. the second player can play
-        // there to preserve the symmetry)
-        do
-        {
-            Point symm_p = m_shared_const.symmetric_points[*i];
-            if (! m_bd.get_point_state(symm_p).is_empty())
-            {
-                m_is_symmetry_broken = true;
-                return;
-            }
-        }
-        while (++i != end);
-    }
-}
-
 //-----------------------------------------------------------------------------
 
 } // namespace libpentobi_mcts
