@@ -14,6 +14,7 @@ namespace libpentobi_mcts {
 
 using libpentobi_base::Color;
 using libpentobi_base::ColorMove;
+using libpentobi_base::Geometry;
 using libpentobi_base::Point;
 using libpentobi_base::PointState;
 
@@ -21,44 +22,41 @@ using libpentobi_base::PointState;
 
 namespace {
 
-/** Return the symmetric point state for symmetry detection.
-    Only used for Variant::duo. Returns the other color or empty, if the
-    given point state is empty. */
-PointState get_symmetric_state(Color c)
-{
-    if (c == Color(0))
-        return PointState(Color(1));
-    else if (c == Color(1))
-        return PointState(Color(0));
-    else if (c == Color(2))
-        return PointState(Color(3));
-    else
-    {
-        LIBBOARDGAME_ASSERT(c == Color(3));
-        return PointState(Color(2));
-    }
-}
+array<Color, Color::range> symmetric_state =
+    { Color(1), Color(0), Color(3), Color(2) };
 
 } // namespace
 
 //-----------------------------------------------------------------------------
 
-bool check_symmetry_broken(const Board& bd,
-                           const SymmetricPoints& symmetric_points)
+bool check_symmetry_broken(const Board& bd)
 {
+    LIBBOARDGAME_ASSERT(has_central_symmetry(bd.get_variant()));
+    auto& symmetric_points = bd.get_board_const().get_symmetrc_points();
     Color to_play = bd.get_to_play();
+    auto& geo = bd.get_geometry();
+    // No need to iterator over the whole board when checking symmetry (this
+    // makes the assumption that the symmetric points of the points in the
+    // first half of the integer range are in the second half).
+    Geometry::Iterator begin = geo.begin();
+    LIBBOARDGAME_ASSERT(geo.get_range() % 2 == 0);
+    Geometry::Iterator end(static_cast<Point::IntType>(geo.get_range() / 2));
+#if LIBBOARDGAME_DEBUG
+    for (auto p = begin; p != end; ++p)
+        LIBBOARDGAME_ASSERT(symmetric_points[*p].to_int() >= (*end).to_int());
+#endif
     if (to_play == Color(0) || to_play == Color(2))
     {
         // First player to play: the symmetry is broken if the position is
         // not symmetric.
-        for (Point p : bd)
+        for (auto p = begin; p != end; ++p)
         {
-            PointState s1 = bd.get_point_state(p);
+            PointState s1 = bd.get_point_state(*p);
             if (! s1.is_empty())
             {
-                Point symm_p = symmetric_points[p];
+                Point symm_p = symmetric_points[*p];
                 PointState s2 = bd.get_point_state(symm_p);
-                if (s2 != get_symmetric_state(s1.to_color()))
+                if (s2 != symmetric_state[s1.to_int()])
                     return true;
             }
         }
@@ -78,17 +76,19 @@ bool check_symmetry_broken(const Board& bd,
         if (last_mv.color != previous_color)
             // Don't try to handle non-alternating moves in board history
             return true;
-        auto& info = bd.get_move_info(last_mv.move);
-        for (Point p : bd)
+        auto points = bd.get_move_points(last_mv.move);
+        for (Point p : points)
+            if (! bd.get_point_state(symmetric_points[p]).is_empty())
+                return true;
+        for (auto p = begin; p != end; ++p)
         {
-            PointState s1 = bd.get_point_state(p);
+            PointState s1 = bd.get_point_state(*p);
             if (! s1.is_empty())
             {
-                Point symm_p = symmetric_points[p];
-                PointState s2 = bd.get_point_state(symm_p);
-                if (s2 != get_symmetric_state(s1.to_color()))
-                    if (! (info.contains(p) && s2.is_empty()))
-                        return true;
+                PointState s2 = bd.get_point_state(symmetric_points[*p]);
+                if (s2 != symmetric_state[s1.to_int()]
+                        && ! points.contains(*p))
+                    return true;
             }
         }
     }
