@@ -169,7 +169,10 @@ void AnalyzeGameWidget::setCurrentPosition(const Game& game,
 
 void AnalyzeGameWidget::showProgress(int progress)
 {
-    m_progressDialog->setValue(progress);
+    // m_progressDialog might already be closed if cancel was pressed and
+    // setValue makes it visible again (only with some Qt versions/platforms?)
+    if (m_progressDialog->isVisible())
+        m_progressDialog->setValue(progress);
     // Repaint the window with the current status of the analysis
     update();
 }
@@ -188,15 +191,18 @@ void AnalyzeGameWidget::start(const Game& game, Search& search,
     m_search = &search;
     m_nuSimulations = nuSimulations;
     initSize();
-    m_progressDialog = new QProgressDialog(this);
-    m_progressDialog->setWindowModality(Qt::WindowModal);
-    m_progressDialog->setWindowFlags(m_progressDialog->windowFlags()
-                                     & ~Qt::WindowContextHelpButtonHint);
-    m_progressDialog->setLabel(new QLabel(tr("Running game analysis..."),
-                                          this));
-    Util::setNoTitle(*m_progressDialog);
-    m_progressDialog->setMinimumDuration(0);
-    connect(m_progressDialog, SIGNAL(canceled()), SLOT(cancel()));
+    if (! m_progressDialog)
+    {
+        m_progressDialog = new QProgressDialog(this);
+        m_progressDialog->setWindowModality(Qt::WindowModal);
+        m_progressDialog->setWindowFlags(m_progressDialog->windowFlags()
+                                         & ~Qt::WindowContextHelpButtonHint);
+        m_progressDialog->setLabel(new QLabel(tr("Running game analysis..."),
+                                              this));
+        Util::setNoTitle(*m_progressDialog);
+        m_progressDialog->setMinimumDuration(0);
+        connect(m_progressDialog, SIGNAL(canceled()), SLOT(cancel()));
+    }
     m_progressDialog->show();
     m_isRunning = true;
     m_future = QtConcurrent::run(this, &AnalyzeGameWidget::threadFunction);
@@ -213,12 +219,11 @@ void AnalyzeGameWidget::threadFunction()
                 return;
             int progress = 100 * movesAnalyzed / totalMoves;
             QMetaObject::invokeMethod(this, "showProgress",
-                                      Qt::QueuedConnection,
+                                      Qt::BlockingQueuedConnection,
                                       Q_ARG(int, progress));
         };
     m_analyzeGame.run(*m_game, *m_search, m_nuSimulations, progressCallback);
-    QMetaObject::invokeMethod(this, "showProgress", Qt::QueuedConnection,
-                              Q_ARG(int, 100));
+    QMetaObject::invokeMethod(m_progressDialog, "hide", Qt::QueuedConnection);
     m_isRunning = false;
     emit finished();
 }
