@@ -13,8 +13,7 @@
 //-----------------------------------------------------------------------------
 
 AnalyzeSpeedDialog::AnalyzeSpeedDialog(QWidget* parent, const QString& title)
-    : QInputDialog(parent),
-      m_speedValue(0)
+    : QInputDialog(parent)
 {
     m_items << tr("Fast") << tr("Normal") << tr("Slow");
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
@@ -25,6 +24,10 @@ AnalyzeSpeedDialog::AnalyzeSpeedDialog(QWidget* parent, const QString& title)
     setComboBoxEditable(false);
 }
 
+AnalyzeSpeedDialog::~AnalyzeSpeedDialog()
+{
+}
+
 void AnalyzeSpeedDialog::accept()
 {
     m_speedValue = m_items.indexOf(textValue());
