@@ -19,7 +19,7 @@ Q_DECL_EXPORT ThumbCreator* new_creator() { return new PentobiThumbCreator; }
 
 //-----------------------------------------------------------------------------
 
-PentobiThumbCreator::~PentobiThumbCreator() { }
+PentobiThumbCreator::~PentobiThumbCreator() = default;
 
 bool PentobiThumbCreator::create(const QString& path, int width, int height,
                                  QImage& image)
