@@ -13,7 +13,6 @@
 #include <QSettings>
 #include "libboardgame_sgf/SgfUtil.h"
 #include "libboardgame_sgf/TreeReader.h"
-#include "libboardgame_util/Log.h"
 #include "libpentobi_base/PentobiTreeWriter.h"
 #include "libpentobi_base/TreeUtil.h"
 
@@ -23,7 +22,7 @@ using libboardgame_sgf::TreeReader;
 using libboardgame_sgf::util::back_to_main_variation;
 using libboardgame_sgf::util::get_last_node;
 using libboardgame_sgf::util::is_main_variation;
-using libpentobi_base::get_board_type;
+using libpentobi_base::get_piece_set;
 using libpentobi_base::to_string_id;
 using libpentobi_base::BoardType;
 using libpentobi_base::Color;
@@ -36,6 +35,7 @@ using libpentobi_base::PentobiTreeWriter;
 using libpentobi_base::Piece;
 using libpentobi_base::PieceInfo;
 using libpentobi_base::PiecePoints;
+using libpentobi_base::PieceSet;
 using libpentobi_base::Point;
 using libpentobi_base::tree_util::get_position_info;
 
@@ -62,23 +62,12 @@ bool compareTransform(const PieceInfo& pieceInfo, const Transform* t1,
 QPointF getGameCoord(const Board& bd, Move mv)
 {
     auto& geo = bd.get_geometry();
-    auto moveInfo = bd.get_move_info(mv);
     PiecePoints movePoints;
-    for (Point p : moveInfo)
+    for (Point p : bd.get_move_points(mv))
         movePoints.push_back(CoordPoint(geo.get_x(p), geo.get_y(p)));
     return PieceModel::findCenter(bd, movePoints, false);
 }
 
-/** Set a variable to a value and return if it has changed. */
-template<typename T>
-bool set(T& target, const T& value)
-{
-    if (target == value)
-        return false;
-    target = value;
-    return true;
-}
-
 } //namespace
 
 //-----------------------------------------------------------------------------
@@ -131,22 +120,44 @@ void GameModel::createPieceModels()
 
 void GameModel::createPieceModels(Color c, QList<PieceModel*>& pieceModels)
 {
-    pieceModels.clear();
     auto& bd = getBoard();
-    for (unsigned i = 0; i < bd.get_nu_uniq_pieces(); ++i)
+    auto nuPieces = bd.get_nu_uniq_pieces();
+    pieceModels.clear();
+    pieceModels.reserve(nuPieces);
+    for (Piece::IntType i = 0; i < nuPieces; ++i)
     {
         Piece piece(i);
-        for (unsigned j = 0; j < bd.get_nu_piece_instances(); ++j)
+        for (unsigned j = 0; j < bd.get_piece_info(piece).get_nu_instances();
+             ++j)
             pieceModels.append(new PieceModel(this, bd, piece, c));
     }
 }
 
-bool GameModel::findMove(const PieceModel& piece, QString state,
+void GameModel::deleteAllVar()
+{
+    if (! is_main_variation(m_game.get_current()))
+        emit positionAboutToChange();
+    m_game.delete_all_variations();
+    updateProperties();
+}
+
+bool GameModel::findMove(const PieceModel& pieceModel, const QString& state,
                          QPointF coord, Move& mv) const
 {
+    auto piece = pieceModel.getPiece();
     auto& bd = getBoard();
-    auto& info = bd.get_piece_info(piece.getPiece());
-    auto transform = piece.getTransform(state);
+    if (piece.to_int() >= bd.get_nu_uniq_pieces())
+    {
+        qWarning("GameModel::findMove: pieceModel invalid in game variant");
+        return false;
+    }
+    auto transform = pieceModel.getTransform(state);
+    if (! transform)
+    {
+        qWarning("GameModel::findMove: transform not found");
+        return false;
+    }
+    auto& info = bd.get_piece_info(piece);
     PiecePoints piecePoints = info.get_points();
     transform->transform(piecePoints.begin(), piecePoints.end());
     auto boardType = bd.get_board_type();
@@ -167,7 +178,7 @@ bool GameModel::findMove(const PieceModel& piece, QString state,
         int y = p.y + offY;
         if (! geo.is_onboard(CoordPoint(x, y)))
             return false;
-        auto pointType = geo.get_point_type(p.x, p.y);
+        auto pointType = geo.get_point_type(p);
         auto boardPointType = geo.get_point_type(x, y);
         if (! pointTypeChanged && pointType != boardPointType)
             return false;
@@ -175,7 +186,114 @@ bool GameModel::findMove(const PieceModel& piece, QString state,
             return false;
         points.push_back(geo.get_point(x, y));
     }
-    return bd.find_move(points, piece.getPiece(), mv);
+    return bd.find_move(points, piece, mv);
+}
+
+QString GameModel::getResultMessage()
+{
+    auto& bd = getBoard();
+    auto nuPlayers = bd.get_nu_players();
+    bool breakTies = (bd.get_piece_set() == PieceSet::callisto);
+    if (m_nuColors == 2)
+    {
+        auto score = m_points0 - m_points1;
+        if (score == 1)
+            return tr("Blue wins with 1 point.");
+        if (score > 0)
+            return tr("Blue wins with %1 points.").arg(score);
+        if (score == -1)
+            return tr("Green wins with 1 point.");
+        if (score < 0)
+            return tr("Green wins with %1 points.").arg(-score);
+        if (breakTies)
+            return tr("Green wins (tie resolved).");
+        return tr("Game ends in a tie.");
+    }
+    if (m_nuColors == 4 && nuPlayers == 2)
+    {
+        auto score = m_points0 + m_points2 - m_points1 - m_points3;
+        if (score == 1)
+            return tr("Blue/Red wins with 1 point.");
+        if (score > 0)
+            return tr("Blue/Red wins with %1 points.").arg(score);
+        if (score == -1)
+            return tr("Yellow/Green wins with 1 point.");
+        if (score < 0)
+            return tr("Yellow/Green wins with %1 points.").arg(-score);
+        if (breakTies)
+            return tr("Yellow/Green wins (tie resolved).");
+        return tr("Game ends in a tie.");
+    }
+    if (nuPlayers == 3)
+    {
+        auto maxPoints = max(max(m_points0, m_points1), m_points2);
+        unsigned nuWinners = 0;
+        if (m_points0 == maxPoints)
+            ++nuWinners;
+        if (m_points1 == maxPoints)
+            ++nuWinners;
+        if (m_points2 == maxPoints)
+            ++nuWinners;
+        if (m_points0 == maxPoints && nuWinners == 1)
+            return tr("Blue wins.");
+        if (m_points1 == maxPoints && nuWinners == 1)
+            return tr("Yellow wins.");
+        if (m_points2 == maxPoints && nuWinners == 1)
+            return tr("Red wins.");
+        if (m_points2 == maxPoints && breakTies)
+            return tr("Red wins (tie resolved).");
+        if (m_points1 == maxPoints && breakTies)
+            return tr("Yellow wins (tie resolved).");
+        if (m_points0 == maxPoints && m_points1 == maxPoints && nuWinners == 2)
+            return tr("Game ends in a tie between Blue and Yellow.");
+        if (m_points0 == maxPoints && m_points2 == maxPoints && nuWinners == 2)
+            return tr("Game ends in a tie between Blue and Red.");
+        if (nuWinners == 2)
+            return tr("Game ends in a tie between Yellow and Red.");
+        return tr("Game ends in a tie between all players.");
+    }
+    auto maxPoints = max(max(m_points0, m_points1), max(m_points2, m_points3));
+    unsigned nuWinners = 0;
+    if (m_points0 == maxPoints)
+        ++nuWinners;
+    if (m_points1 == maxPoints)
+        ++nuWinners;
+    if (m_points2 == maxPoints)
+        ++nuWinners;
+    if (m_points3 == maxPoints)
+        ++nuWinners;
+    if (m_points0 == maxPoints && nuWinners == 1)
+        return tr("Blue wins.");
+    if (m_points1 == maxPoints && nuWinners == 1)
+        return tr("Yellow wins.");
+    if (m_points2 == maxPoints && nuWinners == 1)
+        return tr("Red wins.");
+    if (m_points3 == maxPoints && nuWinners == 1)
+        return tr("Green wins.");
+    if (m_points3 == maxPoints && breakTies)
+        return tr("Green wins (tie resolved).");
+    if (m_points2 == maxPoints && breakTies)
+        return tr("Red wins (tie resolved).");
+    if (m_points1 == maxPoints && breakTies)
+        return tr("Yellow wins (tie resolved).");
+    if (m_points0 == maxPoints && m_points1 == maxPoints
+            && m_points2 == maxPoints && nuWinners == 3)
+        return tr("Game ends in a tie between Blue, Yellow and Red.");
+    if (m_points0 == maxPoints && m_points1 == maxPoints
+            && m_points3 == maxPoints && nuWinners == 3)
+        return tr("Game ends in a tie between Blue, Yellow and Green.");
+    if (m_points0 == maxPoints && m_points2 == maxPoints
+            && m_points3 == maxPoints && nuWinners == 3)
+        return tr("Game ends in a tie between Blue, Red and Green.");
+    if (nuWinners == 3)
+        return tr("Game ends in a tie between Yellow, Red and Green.");
+    if (m_points0 == maxPoints && m_points1 == maxPoints && nuWinners == 2)
+        return tr("Game ends in a tie between Blue and Yellow.");
+    if (m_points0 == maxPoints && m_points2 == maxPoints && nuWinners == 2)
+        return tr("Game ends in a tie between Blue and Red.");
+    if (nuWinners == 2)
+        return tr("Game ends in a tie between Yellow and Red.");
+    return tr("Game ends in a tie between all players.");
 }
 
 Variant GameModel::getInitialGameVariant()
@@ -183,27 +301,11 @@ Variant GameModel::getInitialGameVariant()
     QSettings settings;
     auto variantString = settings.value("variant", "").toString();
     Variant variant;
-    // Nexos not yet supported in the GUI
-    if (! parse_variant_id(variantString.toLocal8Bit().constData(), variant)
-            || get_board_type(variant) == BoardType::nexos)
+    if (! parse_variant_id(variantString.toLocal8Bit().constData(), variant))
         variant = Variant::duo;
     return variant;
 }
 
-int GameModel::getLastMoveColor()
-{
-    auto& bd = getBoard();
-    auto nuMoves = bd.get_nu_moves();
-    if (nuMoves == 0)
-        return 0;
-    return bd.get_move(nuMoves - 1).color.to_int();
-}
-
-PieceModel* GameModel::getLastMovePieceModel()
-{
-    return m_lastMovePieceModel;
-}
-
 QList<PieceModel*>& GameModel::getPieceModels(Color c)
 {
     if (c == Color(0))
@@ -213,7 +315,7 @@ QList<PieceModel*>& GameModel::getPieceModels(Color c)
     else if (c == Color(2))
         return m_pieceModels2;
     else
-        return  m_pieceModels3;
+        return m_pieceModels3;
 }
 
 void GameModel::goBackward()
@@ -250,6 +352,7 @@ void GameModel::gotoNode(const SgfNode& node)
 {
     if (&node == &m_game.get_current())
         return;
+    emit positionAboutToChange();
     try
     {
         m_game.goto_node(node);
@@ -266,7 +369,7 @@ void GameModel::gotoNode(const SgfNode* node)
         gotoNode(*node);
 }
 
-void GameModel::initGameVariant(QString gameVariant)
+void GameModel::initGameVariant(const QString& gameVariant)
 {
     Variant variant;
     if (! parse_variant_id(gameVariant.toLocal8Bit().constData(), variant))
@@ -277,21 +380,22 @@ void GameModel::initGameVariant(QString gameVariant)
     if (m_game.get_variant() != variant)
         m_game.init(variant);
     auto& bd = getBoard();
-    if (set(m_nuColors, static_cast<int>(bd.get_nu_colors())))
-        emit nuColorsChanged(m_nuColors);
+    set(m_nuColors, static_cast<int>(bd.get_nu_colors()),
+        &GameModel::nuColorsChanged);
+    m_lastMovePieceModel = nullptr;
     createPieceModels();
     m_gameVariant = gameVariant;
     emit gameVariantChanged(gameVariant);
     updateProperties();
 }
 
-bool GameModel::isLegalPos(PieceModel* pieceModel, QString state,
+bool GameModel::isLegalPos(PieceModel* pieceModel, const QString& state,
                            QPointF coord) const
 {
     Move mv;
     if (! findMove(*pieceModel, state, coord, mv))
         return false;
-    Color c(pieceModel->color());
+    Color c(static_cast<Color::IntType>(pieceModel->color()));
     bool result = getBoard().is_legal(c, mv);
     return result;
 }
@@ -325,17 +429,26 @@ void GameModel::moveUpVar()
     updateProperties();
 }
 
+void GameModel::nextColor()
+{
+    emit positionAboutToChange();
+    auto& bd = getBoard();
+    m_game.set_to_play(bd.get_next(bd.get_to_play()));
+    updateProperties();
+}
+
 void GameModel::newGame()
 {
+    emit positionAboutToChange();
     m_game.init();
     for (auto pieceModel : m_pieceModels0)
-        pieceModel->setState("");
+        pieceModel->setDefaultState();
     for (auto pieceModel : m_pieceModels1)
-        pieceModel->setState("");
+        pieceModel->setDefaultState();
     for (auto pieceModel : m_pieceModels2)
-        pieceModel->setState("");
+        pieceModel->setDefaultState();
     for (auto pieceModel : m_pieceModels3)
-        pieceModel->setState("");
+        pieceModel->setDefaultState();
     updateProperties();
 }
 
@@ -346,12 +459,7 @@ bool GameModel::open(istream& in)
         TreeReader reader;
         reader.read(in);
         auto root = reader.get_tree_transfer_ownership();
-        auto boardType = get_board_type(PentobiTree::get_variant(*root));
-        if (boardType == BoardType::nexos)
-        {
-            m_lastInputOutputError = tr("Unsupported game variant");
-            return false;
-        }
+        emit positionAboutToChange();
         m_game.init(root);
         auto variant = to_string_id(m_game.get_variant());
         if (variant != m_gameVariant)
@@ -369,7 +477,7 @@ bool GameModel::open(istream& in)
     return true;
 }
 
-bool GameModel::open(QString file)
+bool GameModel::open(const QString& file)
 {
     ifstream in(file.toLocal8Bit().constData());
     if (! in)
@@ -402,23 +510,24 @@ QQmlListProperty<PieceModel> GameModel::pieceModels3()
 
 void GameModel::playMove(int move)
 {
-    Color c = m_game.get_effective_to_play();
-    Move mv(move);
-    if(mv.is_null())
+    Move mv(static_cast<Move::IntType>(move));
+    if (mv.is_null())
         return;
-    m_game.play(c, mv, false);
+    emit positionAboutToChange();
+    m_game.play(m_game.get_to_play(), mv, false);
     updateProperties();
 }
 
 void GameModel::playPiece(PieceModel* pieceModel, QPointF coord)
 {
-    Color c(pieceModel->color());
+    Color c(static_cast<Color::IntType>(pieceModel->color()));
     Move mv;
     if (! findMove(*pieceModel, pieceModel->state(), coord, mv))
     {
         qWarning("GameModel::play: illegal move");
         return;
     }
+    emit positionAboutToChange();
     preparePieceGameCoord(pieceModel, mv);
     pieceModel->setIsPlayed(true);
     preparePieceTransform(pieceModel, mv);
@@ -428,9 +537,10 @@ void GameModel::playPiece(PieceModel* pieceModel, QPointF coord)
 
 PieceModel* GameModel::preparePiece(int color, int move)
 {
-    Move mv(move);
-    Piece piece = getBoard().get_move_info(mv).get_piece();
-    for (auto pieceModel : getPieceModels(Color(color)))
+    Move mv(static_cast<Move::IntType>(move));
+    Color c(static_cast<Color::IntType>(color));
+    Piece piece = getBoard().get_move_piece(mv);
+    for (auto pieceModel : getPieceModels(c))
         if (pieceModel->getPiece() == piece && ! pieceModel->isPlayed())
         {
             preparePieceTransform(pieceModel, mv);
@@ -449,13 +559,12 @@ void GameModel::preparePieceTransform(PieceModel* pieceModel, Move mv)
 {
     auto& bd = getBoard();
     auto transform = bd.find_transform(mv);
-    Piece piece = bd.get_move_info(mv).get_piece();
-    auto& pieceInfo = bd.get_piece_info(piece);
+    auto& pieceInfo = bd.get_piece_info(bd.get_move_piece(mv));
     if (! compareTransform(pieceInfo, pieceModel->getTransform(), transform))
         pieceModel->setTransform(transform);
 }
 
-bool GameModel::save(QString file)
+bool GameModel::save(const QString& file)
 {
     ofstream out(file.toLocal8Bit().constData());
     PentobiTreeWriter writer(out, m_game.get_tree());
@@ -470,10 +579,22 @@ bool GameModel::save(QString file)
     return true;
 }
 
+template<typename T>
+void GameModel::set(T& target, const T& value,
+                    void (GameModel::*changedSignal)(T))
+{
+    if (target != value)
+    {
+        target = value;
+        emit (this->*changedSignal)(value);
+    }
+}
+
 void GameModel::truncate()
 {
     if (! m_game.get_current().has_parent())
         return;
+    emit positionAboutToChange();
     m_game.truncate();
     updateProperties();
 }
@@ -486,8 +607,10 @@ void GameModel::truncateChildren()
 
 void GameModel::undo()
 {
-    if (m_canUndo)
-        m_game.undo();
+    if (! m_canUndo)
+        return;
+    emit positionAboutToChange();
+    m_game.undo();
     updateProperties();
 }
 
@@ -496,12 +619,11 @@ PieceModel* GameModel::updatePiece(Color c, Move mv,
                                    array<bool, Board::max_pieces>& isPlayed)
 {
     auto& bd = getBoard();
-    Piece piece = bd.get_move_info(mv).get_piece();
+    Piece piece = bd.get_move_piece(mv);
     auto& pieceInfo = bd.get_piece_info(piece);
     auto gameCoord = getGameCoord(bd, mv);
     auto transform = bd.find_transform(mv);
     auto& pieceModels = getPieceModels(c);
-    PieceModel* pieceModel = nullptr;
     // Prefer piece models already played with the given gameCoord and
     // transform because class Board doesn't make a distinction between
     // instances of the same piece (in Junior) and we want to avoid
@@ -513,79 +635,109 @@ PieceModel* GameModel::updatePiece(Color c, Move mv,
                 && compareTransform(pieceInfo, pieceModels[i]->getTransform(),
                                     transform))
         {
-            pieceModel = pieceModels[i];
             isPlayed[i] = true;
-            break;
+            return pieceModels[i];
         }
-    if (pieceModel == nullptr)
-    {
-        for (int i = 0; i < pieceModels.length(); ++i)
-            if (pieceModels[i]->getPiece() == piece && ! isPlayed[i])
-            {
-                pieceModel = pieceModels[i];
-                isPlayed[i] = true;
-                break;
-            }
-        // Order is important: isPlayed will trigger an animation to move
-        // the piece, so it needs to be set after gameCoord.
-        pieceModel->setGameCoord(gameCoord);
-        pieceModel->setIsPlayed(true);
-        pieceModel->setTransform(transform);
-    }
-    return pieceModel;
+    for (int i = 0; i < pieceModels.length(); ++i)
+        if (pieceModels[i]->getPiece() == piece && ! isPlayed[i])
+        {
+            isPlayed[i] = true;
+            // Order is important: isPlayed will trigger an animation to move
+            // the piece, so it needs to be set after gameCoord.
+            pieceModels[i]->setGameCoord(gameCoord);
+            pieceModels[i]->setIsPlayed(true);
+            pieceModels[i]->setTransform(transform);
+            return pieceModels[i];
+        }
+    LIBBOARDGAME_ASSERT(false);
+    return nullptr;
 }
 
 void GameModel::updateProperties()
 {
     auto& bd = getBoard();
+    auto& geo = bd.get_geometry();
     auto& tree = m_game.get_tree();
-    if (set(m_points0, static_cast<int>(bd.get_points(Color(0)))))
-        emit points0Changed(m_points0);
-    if (set(m_points1, static_cast<int>(bd.get_points(Color(1)))))
-        emit points1Changed(m_points1);
-    if (set(m_bonus0, static_cast<int>(bd.get_bonus(Color(0)))))
-        emit bonus0Changed(m_bonus0);
-    if (set(m_bonus1, static_cast<int>(bd.get_bonus(Color(1)))))
-        emit bonus1Changed(m_bonus1);
-    if (set(m_hasMoves0, bd.has_moves(Color(0))))
-        emit hasMoves0Changed(m_hasMoves0);
-    if (set(m_hasMoves1, bd.has_moves(Color(1))))
-        emit hasMoves1Changed(m_hasMoves1);
+    bool isTrigon = (bd.get_piece_set() == PieceSet::trigon);
+    set(m_points0, bd.get_points(Color(0)), &GameModel::points0Changed);
+    set(m_points1, bd.get_points(Color(1)), &GameModel::points1Changed);
+    set(m_bonus0, bd.get_bonus(Color(0)), &GameModel::bonus0Changed);
+    set(m_bonus1, bd.get_bonus(Color(1)), &GameModel::bonus1Changed);
+    set(m_hasMoves0, bd.has_moves(Color(0)), &GameModel::hasMoves0Changed);
+    set(m_hasMoves1, bd.has_moves(Color(1)), &GameModel::hasMoves1Changed);
+    bool isFirstPieceAny = false;
     if (m_nuColors > 2)
     {
-        if (set(m_points2, static_cast<int>(bd.get_points(Color(2)))))
-            emit points2Changed(m_points2);
-        if (set(m_bonus2, static_cast<int>(bd.get_bonus(Color(2)))))
-            emit bonus2Changed(m_bonus2);
-        if (set(m_hasMoves2, bd.has_moves(Color(2))))
-            emit hasMoves2Changed(m_hasMoves2);
+        set(m_points2, bd.get_points(Color(2)), &GameModel::points2Changed);
+        set(m_bonus2, bd.get_bonus(Color(2)), &GameModel::bonus2Changed);
+        set(m_hasMoves2, bd.has_moves(Color(2)), &GameModel::hasMoves2Changed);
     }
     if (m_nuColors > 3)
     {
-        if (set(m_points3, static_cast<int>(bd.get_points(Color(3)))))
-            emit points3Changed(m_points3);
-        if (set(m_bonus3, static_cast<int>(bd.get_bonus(Color(3)))))
-            emit bonus3Changed(m_bonus3);
-        if (set(m_hasMoves3, bd.has_moves(Color(3))))
-            emit hasMoves3Changed(m_hasMoves3);
+        set(m_points3, bd.get_points(Color(3)), &GameModel::points3Changed);
+        set(m_bonus3, bd.get_bonus(Color(3)), &GameModel::bonus3Changed);
+        set(m_hasMoves3, bd.has_moves(Color(3)), &GameModel::hasMoves3Changed);
     }
+    m_tmpPoints.clear();
+    if (bd.is_first_piece(Color(0)))
+    {
+        isFirstPieceAny = true;
+        if (! isTrigon)
+            for (Point p : bd.get_starting_points(Color(0)))
+                m_tmpPoints.append(QPointF(geo.get_x(p), geo.get_y(p)));
+    }
+    set(m_startingPoints0, m_tmpPoints, &GameModel::startingPoints0Changed);
+    m_tmpPoints.clear();
+    if (bd.is_first_piece(Color(1)))
+    {
+        isFirstPieceAny = true;
+        if (! isTrigon)
+            for (Point p : bd.get_starting_points(Color(1)))
+                m_tmpPoints.append(QPointF(geo.get_x(p), geo.get_y(p)));
+    }
+    set(m_startingPoints1, m_tmpPoints, &GameModel::startingPoints1Changed);
+    m_tmpPoints.clear();
+    if (m_nuColors > 2 && bd.is_first_piece(Color(2)))
+    {
+        isFirstPieceAny = true;
+        if (! isTrigon)
+            for (Point p : bd.get_starting_points(Color(2)))
+                m_tmpPoints.append(QPointF(geo.get_x(p), geo.get_y(p)));
+    }
+    set(m_startingPoints2, m_tmpPoints, &GameModel::startingPoints2Changed);
+    m_tmpPoints.clear();
+    if (m_nuColors > 3 && bd.is_first_piece(Color(3)))
+    {
+        isFirstPieceAny = true;
+        if (! isTrigon)
+            for (Point p : bd.get_starting_points(Color(3)))
+                m_tmpPoints.append(QPointF(geo.get_x(p), geo.get_y(p)));
+    }
+    set(m_startingPoints3, m_tmpPoints, &GameModel::startingPoints3Changed);
+    m_tmpPoints.clear();
+    if (isTrigon && isFirstPieceAny)
+        for (Point p : bd.get_starting_points(Color(0)))
+            m_tmpPoints.append(QPointF(geo.get_x(p), geo.get_y(p)));
+    set(m_startingPointsAll, m_tmpPoints,
+        &GameModel::startingPointsAllChanged);
     auto& current = m_game.get_current();
-    if (set(m_canUndo,
-            ! current.has_children()
-            && m_game.get_tree().has_move_ignore_invalid(current)
-            && current.has_parent()))
-        emit canUndoChanged(m_canUndo);
-    if (set(m_canGoForward, current.has_children()))
-        emit canGoForwardChanged(m_canGoForward);
-    if (set(m_canGoBackward, current.has_parent()))
-        emit canGoBackwardChanged(m_canGoBackward);
-    if (set(m_hasPrevVar, (current.get_previous_sibling() != nullptr)))
-        emit hasPrevVarChanged(m_hasPrevVar);
-    if (set(m_hasNextVar, (current.get_sibling() != nullptr)))
-        emit hasNextVarChanged(m_hasNextVar);
-    if (set(m_isMainVar, is_main_variation(current)))
-        emit isMainVarChanged(m_isMainVar);
-    QString positionInfo
+    set(m_canUndo,
+           ! current.has_children() && tree.has_move_ignore_invalid(current)
+           && current.has_parent(),
+           &GameModel::canUndoChanged);
+    set(m_canGoForward, current.has_children(),
+        &GameModel::canGoForwardChanged);
+    set(m_canGoBackward, current.has_parent(),
+        &GameModel::canGoBackwardChanged);
+    set(m_hasPrevVar, (current.get_previous_sibling() != nullptr),
+        &GameModel::hasPrevVarChanged);
+    set(m_hasNextVar, (current.get_sibling() != nullptr),
+        &GameModel::hasNextVarChanged);
+    set(m_hasVariations, tree.has_variations(),
+        &GameModel::hasVariationsChanged);
+    set(m_isMainVar, is_main_variation(current),
+        &GameModel::isMainVarChanged);
+    auto positionInfo
             = QString::fromLocal8Bit(get_position_info(tree, current).c_str());
     if (positionInfo.isEmpty())
         positionInfo = bd.has_setup() ? tr("(Setup)") : tr("(No moves)");
@@ -598,8 +750,7 @@ void GameModel::updateProperties()
             positionInfo.append(tr("(Setup)"));
         }
     }
-    if (set(m_positionInfo, positionInfo))
-        emit positionInfoChanged(m_positionInfo);
+    set(m_positionInfo, positionInfo, &GameModel::positionInfoChanged);
     bool isGameOver = true;
     for (Color c : bd.get_colors())
         if (bd.has_moves(c))
@@ -607,11 +758,9 @@ void GameModel::updateProperties()
             isGameOver = false;
             break;
         }
-    if (set(m_isGameOver, isGameOver))
-        emit isGameOverChanged(m_isGameOver);
-    if (set(m_isGameEmpty,
-            libboardgame_sgf::util::is_empty(m_game.get_tree())))
-        emit isGameEmptyChanged(m_isGameEmpty);
+    set(m_isGameOver, isGameOver, &GameModel::isGameOverChanged);
+    set(m_isGameEmpty, libboardgame_sgf::util::is_empty(tree),
+        &GameModel::isGameEmptyChanged);
 
     ColorMap<array<bool, Board::max_pieces>> isPlayed;
     for (Color c : bd.get_colors())
@@ -620,27 +769,37 @@ void GameModel::updateProperties()
         for (Move mv : bd.get_setup().placements[c])
             updatePiece(c, mv, isPlayed[c]);
     }
-    m_lastMovePieceModel = nullptr;
+    PieceModel* lastMovePieceModel = nullptr;
     for (unsigned i = 0; i < bd.get_nu_moves(); ++i)
     {
         auto mv = bd.get_move(i);
-        auto pieceModel = updatePiece(mv.color, mv.move, isPlayed[mv.color]);
-        if (i == bd.get_nu_moves() - 1)
-            m_lastMovePieceModel = pieceModel;
+        auto c = mv.color;
+        lastMovePieceModel = updatePiece(c, mv.move, isPlayed[c]);
+    }
+    if (lastMovePieceModel != m_lastMovePieceModel)
+    {
+        if (m_lastMovePieceModel != nullptr)
+            m_lastMovePieceModel->setIsLastMove(false);
+        if (lastMovePieceModel != nullptr)
+            lastMovePieceModel->setIsLastMove(true);
+        m_lastMovePieceModel = lastMovePieceModel;
     }
     for (Color c : bd.get_colors())
     {
         auto& pieceModels = getPieceModels(c);
         for (int i = 0; i < pieceModels.length(); ++i)
-            if (! isPlayed[c][i])
+            if (! isPlayed[c][i] && pieceModels[i]->isPlayed())
+            {
+                pieceModels[i]->setDefaultState();
                 pieceModels[i]->setIsPlayed(false);
+            }
     }
 
-    if (set(m_toPlay, m_isGameOver ? 0 : bd.get_effective_to_play().to_int()))
-        emit toPlayChanged(m_toPlay);
-    if (set(m_altPlayer, (bd.get_variant() == Variant::classic_3 ?
-                          bd.get_alt_player() : 0)))
-        emit altPlayerChanged(m_altPlayer);
+    set(m_toPlay, m_isGameOver ? 0 : bd.get_effective_to_play().to_int(),
+        &GameModel::toPlayChanged);
+    set(m_altPlayer,
+        bd.get_variant() == Variant::classic_3 ? bd.get_alt_player() : 0,
+        &GameModel::altPlayerChanged);
 
     emit positionChanged();
 }
