@@ -25,6 +25,8 @@ class PlayerModel
                NOTIFY levelClassicChanged)
     Q_PROPERTY(unsigned levelClassic2 MEMBER m_levelClassic2
                NOTIFY levelClassic2Changed)
+    Q_PROPERTY(unsigned levelClassic3 MEMBER m_levelClassic3
+               NOTIFY levelClassic3Changed)
     Q_PROPERTY(unsigned levelDuo MEMBER m_levelDuo NOTIFY levelDuoChanged)
     Q_PROPERTY(unsigned levelTrigon MEMBER m_levelTrigon
                NOTIFY levelTrigonChanged)
@@ -32,15 +34,40 @@ class PlayerModel
                NOTIFY levelTrigon2Changed)
     Q_PROPERTY(unsigned levelTrigon3 MEMBER m_levelTrigon3
                NOTIFY levelTrigon3Changed)
-    Q_PROPERTY(unsigned levelJunior MEMBER m_levelJunior NOTIFY levelJuniorChanged)
+    Q_PROPERTY(unsigned levelJunior MEMBER m_levelJunior
+               NOTIFY levelJuniorChanged)
+    Q_PROPERTY(unsigned levelNexos MEMBER m_levelNexos NOTIFY
+               levelNexosChanged)
+    Q_PROPERTY(unsigned levelNexos2 MEMBER m_levelNexos2 NOTIFY
+               levelNexos2Changed)
+    Q_PROPERTY(unsigned levelCallisto MEMBER m_levelCallisto
+               NOTIFY levelCallistoChanged)
+    Q_PROPERTY(unsigned levelCallisto2 MEMBER m_levelCallisto2
+               NOTIFY levelCallisto2Changed)
+    Q_PROPERTY(unsigned levelCallisto3 MEMBER m_levelCallisto3
+               NOTIFY levelCallisto3Changed)
     Q_PROPERTY(bool isGenMoveRunning MEMBER m_isGenMoveRunning
                NOTIFY isGenMoveRunningChanged)
 
 public:
+    /** Global variable to disable opening books. */
+    static bool noBook;
+
+    /** Global variable to disable the minimum thinking time. */
+    static bool noDelay;
+
+    /** Global variable to set the number of threads the player is constructed
+        with.
+        The default value 0 means that the number of threads depends on the
+        hardware. */
+    static unsigned nuThreads;
+
+
     explicit PlayerModel(QObject* parent = nullptr);
 
     ~PlayerModel();
 
+
     /** Start a move generation in a background thread.
         The state of the board model may not be changed until the move
         generation was finished (computerPlayed signal) or aborted
@@ -54,10 +81,18 @@ public:
     Q_INVOKABLE void cancelGenMove();
 
 signals:
+    void levelCallistoChanged(unsigned);
+
+    void levelCallisto2Changed(unsigned);
+
+    void levelCallisto3Changed(unsigned);
+
     void levelClassicChanged(unsigned);
 
     void levelClassic2Changed(unsigned);
 
+    void levelClassic3Changed(unsigned);
+
     void levelDuoChanged(unsigned);
 
     void levelTrigonChanged(unsigned);
@@ -68,6 +103,10 @@ signals:
 
     void levelJuniorChanged(unsigned);
 
+    void levelNexosChanged(unsigned);
+
+    void levelNexos2Changed(unsigned);
+
     void isGenMoveRunningChanged(bool);
 
     void moveGenerated(int move);
@@ -86,10 +125,18 @@ private:
 
     bool m_isGenMoveRunning = false;
 
+    unsigned m_levelCallisto;
+
+    unsigned m_levelCallisto2;
+
+    unsigned m_levelCallisto3;
+
     unsigned m_levelClassic;
 
     unsigned m_levelClassic2;
 
+    unsigned m_levelClassic3;
+
     unsigned m_levelDuo;
 
     unsigned m_levelTrigon;
@@ -100,12 +147,17 @@ private:
 
     unsigned m_levelJunior;
 
+    unsigned m_levelNexos;
+
+    unsigned m_levelNexos2;
+
     unsigned m_genMoveId = 0;
 
     Player m_player;
 
     QFutureWatcher<GenMoveResult> m_genMoveWatcher;
 
+
     GenMoveResult asyncGenMove(GameModel* gm, unsigned genMoveId);
 
     void loadBook(Variant variant);
