@@ -53,8 +53,8 @@ LIBBOARDGAME_TEST_CASE(pentobi_base_board_updater_setup)
     BoardUpdater updater;
     updater.update(*bd, tree, tree.get_root());
     LIBBOARDGAME_CHECK_EQUAL(bd->get_nu_moves(), 0u);
-    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(0)), 10u);
-    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(1)), 10u);
+    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(0)), ScoreType(10));
+    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(1)), ScoreType(10));
 }
 
 /** Test BoardUpdater with setup properties in an inner node. */
@@ -75,8 +75,8 @@ LIBBOARDGAME_TEST_CASE(pentobi_base_board_updater_setup_inner_node)
     // BoardUpdater merges setup properties with existing position, so
     // get_nu_moves() should return the number of moves played after the setup
     LIBBOARDGAME_CHECK_EQUAL(bd->get_nu_moves(), 1u);
-    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(0)), 10u);
-    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(1)), 10u);
+    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(0)), ScoreType(10));
+    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(1)), ScoreType(10));
 }
 
 /** Test removing a piece with the AE property. */
@@ -97,8 +97,8 @@ LIBBOARDGAME_TEST_CASE(pentobi_base_board_updater_setup_empty)
     // BoardUpdater merges setup properties with existing position, so
     // get_nu_moves() should return the number of moves played after the setup
     LIBBOARDGAME_CHECK_EQUAL(bd->get_nu_moves(), 0u);
-    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(0)), 0u);
-    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(1)), 5u);
+    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(0)), ScoreType(0));
+    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(1)), ScoreType(5));
 }
 
 //-----------------------------------------------------------------------------
